@@ -33,6 +33,8 @@
  * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
  * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
  */
+#include <Python.h>
+#include <numpy/ndarrayobject.h>
 
 #include "common.h"
 
@@ -67,7 +69,8 @@
 
 #include "hierarchy.h"
 
-static inline double euclidean_distance(const double *u, const double *v, int n) {
+
+static NPY_INLINE double euclidean_distance(const double *u, const double *v, int n) {
   int i = 0;
   double s = 0.0, d;
   for (i = 0; i < n; i++) {
@@ -154,6 +157,7 @@ void dist_average(cinfo *info, int mini, int minj, int np, int n) {
   int i, xi, xn;
   cnode *rn = info->nodes + inds[mini];
   cnode *sn = info->nodes + inds[minj];
+  cnode *xnd;
   bit = buf;
   rc = (double)rn->n;
   sc = (double)sn->n;
@@ -164,7 +168,7 @@ void dist_average(cinfo *info, int mini, int minj, int np, int n) {
     drx = *(rows[i] + mini - i - 1);
     dsx = *(rows[i] + minj - i - 1);
     xi = inds[i];
-    cnode *xnd = info->nodes + xi;
+    xnd = info->nodes + xi;
     xn = xnd->n;
     mply = (double)1.0 / (((double)xn) * rscnt);
     *bit = mply * ((drx * (rc * xn)) + (dsx * (sc * xn)));
@@ -173,7 +177,7 @@ void dist_average(cinfo *info, int mini, int minj, int np, int n) {
     drx = *(rows[mini] + i - mini - 1);
     dsx = *(rows[i] + minj - i - 1);
     xi = inds[i];
-    cnode *xnd = info->nodes + xi;
+    xnd = info->nodes + xi;
     xn = xnd->n;
     mply = (double)1.0 / (((double)xn) * rscnt);
     *bit = mply * ((drx * (rc * xn)) + (dsx * (sc * xn)));
@@ -182,7 +186,7 @@ void dist_average(cinfo *info, int mini, int minj, int np, int n) {
     drx = *(rows[mini] + i - mini - 1);
     dsx = *(rows[minj] + i - minj - 1);
     xi = inds[i];
-    cnode *xnd = info->nodes + xi;
+    xnd = info->nodes + xi;
     xn = xnd->n;
     mply = (double)1.0 / (((double)xn) * rscnt);
     *bit = mply * ((drx * (rc * xn)) + (dsx * (sc * xn)));
@@ -253,6 +257,7 @@ void dist_ward(cinfo *info, int mini, int minj, int np, int n) {
   int i, m, xi, rind, sind;
   double drx, dsx, rf, sf, xf, xn, rn, sn, drsSq;
   cnode *newNode;
+  cnode *xnd;
 
   rind = inds[mini];
   sind = inds[minj];
@@ -270,7 +275,7 @@ void dist_ward(cinfo *info, int mini, int minj, int np, int n) {
     drx = *(rows[i] + mini - i - 1);
     dsx = *(rows[i] + minj - i - 1);
     xi = inds[i];
-    cnode *xnd = info->nodes + xi;
+    xnd = info->nodes + xi;
     xn = xnd->n;
     rf = (rn + xn) / (rn + sn + xn);
     sf = (sn + xn) / (rn + sn + xn);
@@ -284,7 +289,7 @@ void dist_ward(cinfo *info, int mini, int minj, int np, int n) {
     drx = *(rows[mini] + i - mini - 1);
     dsx = *(rows[i] + minj - i - 1);
     xi = inds[i];
-    cnode *xnd = info->nodes + xi;
+    xnd = info->nodes + xi;
     xn = xnd->n;
     rf = (rn + xn) / (rn + sn + xn);
     sf = (sn + xn) / (rn + sn + xn);
@@ -297,7 +302,7 @@ void dist_ward(cinfo *info, int mini, int minj, int np, int n) {
     drx = *(rows[mini] + i - mini - 1);
     dsx = *(rows[minj] + i - minj - 1);
     xi = inds[i];
-    cnode *xnd = info->nodes + xi;
+    xnd = info->nodes + xi;
     xn = xnd->n;
     rf = (rn + xn) / (rn + sn + xn);
     sf = (sn + xn) / (rn + sn + xn);
@@ -345,6 +350,7 @@ void print_vec(const double *d, int n) {
   CPY_DEBUG_MSG("]");
 }
 
+
 /**
  * notes to self:
  * dm:    The distance matrix.
@@ -354,34 +360,42 @@ void print_vec(const double *d, int n) {
  * ml:    A boolean indicating whether a list of objects in the forest
  *        clusters should be maintained.
  * kc:    Keep track of the centroids.
+ *
+ * Return values:
+ * 0:  success
+ * -1: out of memory--malloc() failed.
  */
-void linkage(double *dm, double *Z, double *X,
+int linkage(double *dm, double *Z, double *X,
 	     int m, int n, int ml, int kc, distfunc dfunc,
 	     int method) {
   int i, j, k, t, np, nid, mini, minj, npc2;
   double min, ln, rn, qn;
-  int *ind;
+  int *ind = NULL;
   /** An iterator through the distance matrix. */
-  double *dmit, *buf;
+  double *dmit, *buf = NULL;
 
-  int *rowsize;
+  int *rowsize = NULL;
 
   /** Temporary array to store modified distance matrix. */
-  double *dmt, **rows, *Zrow;
-  double *centroidsData;
-  double **centroids;
+  double *dmt = NULL, **rows = NULL, *Zrow;
+  double *centroidsData = NULL;
+  double **centroids = NULL;
   const double *centroidL, *centroidR;
   double *centroid;
-  clist *lists, *listL, *listR, *listC;
-  clnode *lnodes;
-  cnode *nodes, *node;
+  clist *lists = NULL, *listL, *listR, *listC;
+  clnode *lnodes = NULL;
+  cnode *nodes = NULL, *node;
 
   cinfo info;
 
+  int result = -1;
+
   /** The next two are only necessary for euclidean distance methods. */
   if (ml) {
     lists = (clist*)malloc(sizeof(clist) * (n-1));
+    if (!lists) goto finished;
     lnodes = (clnode*)malloc(sizeof(clnode) * n);
+    if (!lnodes) goto finished;
   }
   else {
     lists = 0;
@@ -389,7 +403,9 @@ void linkage(double *dm, double *Z, double *X,
   }
   if (kc) {
     centroids = (double**)malloc(sizeof(double*) * (2 * n));
+    if (!centroids) goto finished;
     centroidsData = (double*)malloc(sizeof(double) * n * m);
+    if (!centroidsData) goto finished;
     for (i = 0; i < n; i++) {
       centroids[i] = X + i * m;
     }
@@ -403,11 +419,17 @@ void linkage(double *dm, double *Z, double *X,
   }
 
   nodes = (cnode*)malloc(sizeof(cnode) * (n * 2) - 1);
+  if (!nodes) goto finished;
   ind = (int*)malloc(sizeof(int) * n);
+  if (!ind) goto finished;
   dmt = (double*)malloc(sizeof(double) * NCHOOSE2(n));
+  if (!dmt) goto finished;
   buf = (double*)malloc(sizeof(double) * n);
+  if (!buf) goto finished;
   rows = (double**)malloc(sizeof(double*) * n);
+  if (!rows) goto finished;
   rowsize = (int*)malloc(sizeof(int) * n);
+  if (!rowsize) goto finished;
   memcpy(dmt, dm, sizeof(double) * NCHOOSE2(n));
 
   info.X = X;
@@ -596,6 +618,9 @@ void linkage(double *dm, double *Z, double *X,
     ind[np - 2] = nid;
     /**    print_ind(ind, np - 1);**/
   }
+  result = 0;
+  
+finished:
   free(lists);
   free(lnodes);
   free(nodes);
@@ -606,11 +631,16 @@ void linkage(double *dm, double *Z, double *X,
   free(rowsize);
   free(centroidsData);
   free(centroids);
+  return result;
 }
 
 /** Trying to reimplement so that output is consistent with MATLAB's in
     cases where there are is than one correct choice to make at each
-    iteration of the algorithm. This implementation is not active. */
+    iteration of the algorithm. This implementation is not active.
+
+    XXX linkage_alt is not used.  If it ever does get used, it needs to
+    be updated to check the results of any calls to malloc().
+*/
 
 void linkage_alt(double *dm, double *Z, double *X,
 	     int m, int n, int ml, int kc, distfunc dfunc,
@@ -897,7 +927,7 @@ void cpy_to_tree(const double *Z, cnode **tnodes, int n) {
   }
 }
 
-inline void set_dist_entry(double *d, double val, int i, int j, int n) {
+NPY_INLINE void set_dist_entry(double *d, double val, int i, int j, int n) {
   if (i < j) {
     *(d + (NCHOOSE2(n)-NCHOOSE2(n - i)) + j) = val;
   }
@@ -1065,7 +1095,7 @@ void inconsistency_calculation_alt(const double *Z, double *R, int n, int d) {
   free(rvisited);
 }
 
-void calculate_cluster_sizes(const double *Z, double *CS, int n) {
+void calculate_cluster_sizes(const double *Z, double *cs, int n) {
   int i, j, k, q;
   const double *row;
   for (k = 0; k < n - 1; k++) {
@@ -1075,22 +1105,22 @@ void calculate_cluster_sizes(const double *Z, double *CS, int n) {
     /** If the left node is a non-singleton, add its count. */
     if (i >= n) {
       q = i - n;
-      CS[k] += CS[q];
+      cs[k] += cs[q];
     }
     /** Otherwise just add 1 for the leaf. */
     else {
-      CS[k] += 1.0;
+      cs[k] += 1.0;
     }
     /** If the right node is a non-singleton, add its count. */
     if (j >= n) {
       q = j - n;
-      CS[k] += CS[q];
+      cs[k] += cs[q];
     }
     /** Otherwise just add 1 for the leaf. */
     else {
-      CS[k] += 1.0;
+      cs[k] += 1.0;
     }
-    CPY_DEBUG_MSG("i=%d, j=%d, CS[%d]=%d\n", i, j, k, (int)CS[k]);
+    CPY_DEBUG_MSG("i=%d, j=%d, cs[%d]=%d\n", i, j, k, (int)cs[k]);
   }
 }
 
