@@ -15,11 +15,35 @@ static PyMethodDef vqmethods [] = {
     {NULL, NULL, 0, NULL}
 };
 
+#if defined(SCIPY_PY3K)
+static struct PyModuleDef moduledef = {
+    PyModuleDef_HEAD_INIT,
+    "_vq",
+    NULL,
+    -1,
+    vqmethods,
+    NULL,
+    NULL,
+    NULL,
+    NULL
+};
+
+PyObject *PyInit__vq(void)
+{
+    PyObject *m;
+
+    m = PyModule_Create(&moduledef);
+    import_array();
+
+    return m;
+}
+#else
 PyMODINIT_FUNC init_vq(void) 
 {
     Py_InitModule("_vq", vqmethods);
     import_array();
 }
+#endif
 
 PyObject* compute_vq(PyObject* self, PyObject* args)
 {
@@ -44,7 +68,7 @@ PyObject* compute_vq(PyObject* self, PyObject* args)
 
     typenum1 = PyArray_TYPE(obs);
     typenum2 = PyArray_TYPE(code);
-    if (typenum1 != typenum1) {
+    if (typenum1 != typenum2) {
 		PyErr_Format(PyExc_ValueError,
 			     "observation and code should have same type");
         return NULL;
@@ -98,7 +122,7 @@ PyObject* compute_vq(PyObject* self, PyObject* args)
             if (dist_a == NULL) {
                 goto clean_code_a;
             }
-            index_a = (PyArrayObject*)PyArray_EMPTY(1, &n, PyArray_INTP, 0);
+            index_a = (PyArrayObject*)PyArray_EMPTY(1, &n, NPY_INTP, 0);
             if (index_a == NULL) {
                 goto clean_dist_a;
             }
@@ -110,7 +134,7 @@ PyObject* compute_vq(PyObject* self, PyObject* args)
             if (dist_a == NULL) {
                 goto clean_code_a;
             }
-            index_a = (PyArrayObject*)PyArray_EMPTY(1, &n, PyArray_INTP, 0);
+            index_a = (PyArrayObject*)PyArray_EMPTY(1, &n, NPY_INTP, 0);
             if (index_a == NULL) {
                 goto clean_dist_a;
             }
