@@ -32,25 +32,28 @@
 # WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 # NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 # SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
+from __future__ import division, print_function, absolute_import
 
 import os.path
 
 import numpy as np
 from numpy.testing import TestCase, run_module_suite
 
+from scipy.lib.six.moves import xrange
+
 from scipy.cluster.hierarchy import linkage, from_mlab_linkage, to_mlab_linkage,\
         num_obs_linkage, inconsistent, cophenet, fclusterdata, fcluster, \
         is_isomorphic, single, complete, weighted, centroid, leaders, \
         correspond, is_monotonic, maxdists, maxinconsts, maxRstat, \
-        is_valid_linkage, is_valid_im, to_tree, leaves_list
+        is_valid_linkage, is_valid_im, to_tree, leaves_list, dendrogram
 from scipy.spatial.distance import squareform, pdist
 
-_tdist = np.array([[0,    662,  877,  255,  412,  996],
-                   [662,  0,    295,  468,  268,  400],
-                   [877,  295,  0,    754,  564,  138],
-                   [255,  468,  754,  0,    219,  869],
-                   [412,  268,  564,  219,  0,    669],
-                   [996,  400,  138,  869,  669,  0  ]], dtype='double')
+_tdist = np.array([[0, 662, 877, 255, 412, 996],
+                   [662, 0, 295, 468, 268, 400],
+                   [877, 295, 0, 754, 564, 138],
+                   [255, 468, 754, 0, 219, 869],
+                   [412, 268, 564, 219, 0, 669],
+                   [996, 400, 138, 869, 669, 0]], dtype='double')
 
 _ytdist = squareform(_tdist)
 
@@ -96,18 +99,17 @@ _filenames = ["iris.txt",
               "linkage-Q-ward.txt"
               ]
 
+
 def load_testing_files():
     for fn in _filenames:
         name = fn.replace(".txt", "").replace("-ml", "")
         fqfn = os.path.join(os.path.dirname(__file__), fn)
-        eo[name] = np.loadtxt(open(fqfn))
-        #print "%s: %s   %s" % (name, str(eo[name].shape), str(eo[name].dtype))
-    #eo['pdist-boolean-inp'] = np.bool_(eo['pdist-boolean-inp'])
+        eo[name] = np.loadtxt(fqfn)
 
 load_testing_files()
 
-class TestLinkage(TestCase):
 
+class TestLinkage(TestCase):
     def test_linkage_empty_distance_matrix(self):
         "Tests linkage(Y) where Y is a 0x4 linkage matrix. Exception expected."
         y = np.zeros((0,))
@@ -136,7 +138,6 @@ class TestLinkage(TestCase):
         Zmlab = eo['linkage-average-tdist']
         eps = 1e-05
         expectedZ = from_mlab_linkage(Zmlab)
-        #print Z, expectedZ, np.abs(Z - expectedZ).max()
         self.assertTrue(within_tol(Z, expectedZ, eps))
 
     def test_linkage_weighted_tdist(self):
@@ -145,7 +146,6 @@ class TestLinkage(TestCase):
         Zmlab = eo['linkage-weighted-tdist']
         eps = 1e-10
         expectedZ = from_mlab_linkage(Zmlab)
-        #print Z, expectedZ, np.abs(Z - expectedZ).max()
         self.assertTrue(within_tol(Z, expectedZ, eps))
 
     ################### linkage on Q
@@ -156,7 +156,6 @@ class TestLinkage(TestCase):
         Zmlab = eo['linkage-Q-single']
         eps = 1e-06
         expectedZ = from_mlab_linkage(Zmlab)
-        #print abs(Z-expectedZ).max()
         self.assertTrue(within_tol(Z, expectedZ, eps))
 
     def test_linkage_complete_q(self):
@@ -166,7 +165,6 @@ class TestLinkage(TestCase):
         Zmlab = eo['linkage-Q-complete']
         eps = 1e-07
         expectedZ = from_mlab_linkage(Zmlab)
-        #print abs(Z-expectedZ).max()
         self.assertTrue(within_tol(Z, expectedZ, eps))
 
     def test_linkage_centroid_q(self):
@@ -176,7 +174,6 @@ class TestLinkage(TestCase):
         Zmlab = eo['linkage-Q-centroid']
         eps = 1e-07
         expectedZ = from_mlab_linkage(Zmlab)
-        #print abs(Z-expectedZ).max()
         self.assertTrue(within_tol(Z, expectedZ, eps))
 
     def test_linkage_weighted_q(self):
@@ -186,11 +183,10 @@ class TestLinkage(TestCase):
         Zmlab = eo['linkage-Q-weighted']
         eps = 1e-07
         expectedZ = from_mlab_linkage(Zmlab)
-        #print abs(Z-expectedZ).max()
         self.assertTrue(within_tol(Z, expectedZ, eps))
 
-class TestInconsistent(TestCase):
 
+class TestInconsistent(TestCase):
     def test_single_inconsistent_tdist_1(self):
         "Tests inconsistency matrix calculation (depth=1) on a single linkage."
         Y = squareform(_tdist)
@@ -198,7 +194,6 @@ class TestInconsistent(TestCase):
         R = inconsistent(Z, 1)
         Rright = eo['inconsistent-single-tdist-depth-1']
         eps = 1e-15
-        #print np.abs(R - Rright).max()
         self.assertTrue(within_tol(R, Rright, eps))
 
     def test_single_inconsistent_tdist_2(self):
@@ -208,7 +203,6 @@ class TestInconsistent(TestCase):
         R = inconsistent(Z, 2)
         Rright = eo['inconsistent-single-tdist-depth-2']
         eps = 1e-05
-        #print np.abs(R - Rright).max()
         self.assertTrue(within_tol(R, Rright, eps))
 
     def test_single_inconsistent_tdist_3(self):
@@ -218,7 +212,6 @@ class TestInconsistent(TestCase):
         R = inconsistent(Z, 3)
         Rright = eo['inconsistent-single-tdist-depth-3']
         eps = 1e-05
-        #print np.abs(R - Rright).max()
         self.assertTrue(within_tol(R, Rright, eps))
 
     def test_single_inconsistent_tdist_4(self):
@@ -228,11 +221,9 @@ class TestInconsistent(TestCase):
         R = inconsistent(Z, 4)
         Rright = eo['inconsistent-single-tdist-depth-4']
         eps = 1e-05
-        #print np.abs(R - Rright).max()
         self.assertTrue(within_tol(R, Rright, eps))
 
     # with complete linkage...
-
     def test_complete_inconsistent_tdist_1(self):
         "Tests inconsistency matrix calculation (depth=1) on a complete linkage."
         Y = squareform(_tdist)
@@ -240,7 +231,6 @@ class TestInconsistent(TestCase):
         R = inconsistent(Z, 1)
         Rright = eo['inconsistent-complete-tdist-depth-1']
         eps = 1e-15
-        #print np.abs(R - Rright).max()
         self.assertTrue(within_tol(R, Rright, eps))
 
     def test_complete_inconsistent_tdist_2(self):
@@ -250,7 +240,6 @@ class TestInconsistent(TestCase):
         R = inconsistent(Z, 2)
         Rright = eo['inconsistent-complete-tdist-depth-2']
         eps = 1e-05
-        #print np.abs(R - Rright).max()
         self.assertTrue(within_tol(R, Rright, eps))
 
     def test_complete_inconsistent_tdist_3(self):
@@ -260,7 +249,6 @@ class TestInconsistent(TestCase):
         R = inconsistent(Z, 3)
         Rright = eo['inconsistent-complete-tdist-depth-3']
         eps = 1e-05
-        #print np.abs(R - Rright).max()
         self.assertTrue(within_tol(R, Rright, eps))
 
     def test_complete_inconsistent_tdist_4(self):
@@ -270,11 +258,9 @@ class TestInconsistent(TestCase):
         R = inconsistent(Z, 4)
         Rright = eo['inconsistent-complete-tdist-depth-4']
         eps = 1e-05
-        #print np.abs(R - Rright).max()
         self.assertTrue(within_tol(R, Rright, eps))
 
     # with single linkage and Q data set
-
     def test_single_inconsistent_Q_1(self):
         "Tests inconsistency matrix calculation (depth=1, dataset=Q) with single linkage."
         X = eo['Q-X']
@@ -282,7 +268,6 @@ class TestInconsistent(TestCase):
         R = inconsistent(Z, 1)
         Rright = eo['inconsistent-Q-single-1']
         eps = 1e-06
-        #print np.abs(R - Rright).max()
         self.assertTrue(within_tol(R, Rright, eps))
 
     def test_single_inconsistent_Q_2(self):
@@ -292,7 +277,6 @@ class TestInconsistent(TestCase):
         R = inconsistent(Z, 2)
         Rright = eo['inconsistent-Q-single-2']
         eps = 1e-06
-        #print np.abs(R - Rright).max()
         self.assertTrue(within_tol(R, Rright, eps))
 
     def test_single_inconsistent_Q_3(self):
@@ -302,7 +286,6 @@ class TestInconsistent(TestCase):
         R = inconsistent(Z, 3)
         Rright = eo['inconsistent-Q-single-3']
         eps = 1e-05
-        #print np.abs(R - Rright).max()
         self.assertTrue(within_tol(R, Rright, eps))
 
     def test_single_inconsistent_Q_4(self):
@@ -312,14 +295,14 @@ class TestInconsistent(TestCase):
         R = inconsistent(Z, 4)
         Rright = eo['inconsistent-Q-single-4']
         eps = 1e-05
-        #print np.abs(R - Rright).max()
         self.assertTrue(within_tol(R, Rright, eps))
 
-class TestCopheneticDistance(TestCase):
 
+class TestCopheneticDistance(TestCase):
     def test_linkage_cophenet_tdist_Z(self):
         "Tests cophenet(Z) on tdist data set."
-        expectedM = np.array([268, 295, 255, 255, 295, 295, 268, 268, 295, 295, 295, 138, 219, 295, 295]);
+        expectedM = np.array([268, 295, 255, 255, 295, 295, 268, 268, 295, 295,
+                              295, 138, 219, 295, 295])
         Z = linkage(_ytdist, 'single')
         M = cophenet(Z)
         eps = 1e-10
@@ -329,14 +312,15 @@ class TestCopheneticDistance(TestCase):
         "Tests cophenet(Z, Y) on tdist data set."
         Z = linkage(_ytdist, 'single')
         (c, M) = cophenet(Z, _ytdist)
-        expectedM = np.array([268, 295, 255, 255, 295, 295, 268, 268, 295, 295, 295, 138, 219, 295, 295]);
+        expectedM = np.array([268, 295, 255, 255, 295, 295, 268, 268, 295, 295,
+                              295, 138, 219, 295, 295])
         expectedc = 0.639931296433393415057366837573
         eps = 1e-10
         self.assertTrue(np.abs(c - expectedc) <= eps)
         self.assertTrue(within_tol(M, expectedM, eps))
 
-class TestFromMLabLinkage(TestCase):
 
+class TestFromMLabLinkage(TestCase):
     def test_from_mlab_linkage_empty(self):
         "Tests from_mlab_linkage on empty linkage array."
         X = np.asarray([])
@@ -345,7 +329,7 @@ class TestFromMLabLinkage(TestCase):
 
     def test_from_mlab_linkage_single_row(self):
         "Tests from_mlab_linkage on linkage array with single row."
-        expectedZP = np.asarray([[ 0.,  1.,  3.,  2.]])
+        expectedZP = np.asarray([[0., 1., 3., 2.]])
         Z = [[1,2,3]]
         ZP = from_mlab_linkage(Z)
         return self.assertTrue((ZP == expectedZP).all())
@@ -354,19 +338,17 @@ class TestFromMLabLinkage(TestCase):
         "Tests from_mlab_linkage on linkage array with multiple rows."
         Z = np.asarray([[3, 6, 138], [4, 5, 219],
                         [1, 8, 255], [2, 9, 268], [7, 10, 295]])
-        expectedZS = np.array([[   2.,    5.,  138.,    2.],
-                               [   3.,    4.,  219.,    2.],
-                               [   0.,    7.,  255.,    3.],
-                               [   1.,    8.,  268.,    4.],
-                               [   6.,    9.,  295.,    6.]],
+        expectedZS = np.array([[2., 5., 138., 2.],
+                               [3., 4., 219., 2.],
+                               [0., 7., 255., 3.],
+                               [1., 8., 268., 4.],
+                               [6., 9., 295., 6.]],
                               dtype=np.double)
         ZS = from_mlab_linkage(Z)
-        #print expectedZS, ZS
         self.assertTrue((expectedZS == ZS).all())
 
 
 class TestToMLabLinkage(TestCase):
-
     def test_to_mlab_linkage_empty(self):
         "Tests to_mlab_linkage on empty linkage array."
         X = np.asarray([])
@@ -375,7 +357,7 @@ class TestToMLabLinkage(TestCase):
 
     def test_to_mlab_linkage_single_row(self):
         "Tests to_mlab_linkage on linkage array with single row."
-        Z = np.asarray([[ 0.,  1.,  3.,  2.]])
+        Z = np.asarray([[0., 1., 3., 2.]])
         expectedZP = np.asarray([[1,2,3]])
         ZP = to_mlab_linkage(Z)
         return self.assertTrue((ZP == expectedZP).all())
@@ -384,18 +366,17 @@ class TestToMLabLinkage(TestCase):
         "Tests to_mlab_linkage on linkage array with multiple rows."
         expectedZM = np.asarray([[3, 6, 138], [4, 5, 219],
                         [1, 8, 255], [2, 9, 268], [7, 10, 295]])
-        Z = np.array([[   2.,    5.,  138.,    2.],
-                      [   3.,    4.,  219.,    2.],
-                      [   0.,    7.,  255.,    3.],
-                      [   1.,    8.,  268.,    4.],
-                      [   6.,    9.,  295.,    6.]],
+        Z = np.array([[2., 5., 138., 2.],
+                      [3., 4., 219., 2.],
+                      [0., 7., 255., 3.],
+                      [1., 8., 268., 4.],
+                      [6., 9., 295., 6.]],
                      dtype=np.double)
         ZM = to_mlab_linkage(Z)
-        #print expectedZM, ZM
         self.assertTrue((expectedZM == ZM).all())
 
-class TestFcluster(TestCase):
 
+class TestFcluster(TestCase):
     def test_fclusterdata_maxclusts_2(self):
         "Tests fclusterdata(X, criterion='maxclust', t=2) on a random 3-cluster data set."
         expectedT = np.int_(eo['fclusterdata-maxclusts-2'])
@@ -444,8 +425,8 @@ class TestFcluster(TestCase):
         T = fcluster(Z, criterion='maxclust', t=4)
         self.assertTrue(is_isomorphic(T, expectedT))
 
-class TestLeaders(TestCase):
 
+class TestLeaders(TestCase):
     def test_leaders_single(self):
         "Tests leaders using a flat clustering generated by single linkage."
         X = eo['Q-X']
@@ -454,11 +435,10 @@ class TestLeaders(TestCase):
         T = fcluster(Z, criterion='maxclust', t=3)
         Lright = (np.array([53, 55, 56]), np.array([2, 3, 1]))
         L = leaders(Z, T)
-        #print L, Lright, T
         self.assertTrue((L[0] == Lright[0]).all() and (L[1] == Lright[1]).all())
 
-class TestIsIsomorphic(TestCase):
 
+class TestIsIsomorphic(TestCase):
     def test_is_isomorphic_1(self):
         "Tests is_isomorphic on test case #1 (one flat cluster, different labellings)"
         a = [1, 1, 1]
@@ -544,26 +524,26 @@ class TestIsIsomorphic(TestCase):
         self.assertTrue(is_isomorphic(a, b) == (not noniso))
         self.assertTrue(is_isomorphic(b, a) == (not noniso))
 
-class TestIsValidLinkage(TestCase):
 
+class TestIsValidLinkage(TestCase):
     def test_is_valid_linkage_int_type(self):
         "Tests is_valid_linkage(Z) with integer type."
-        Z = np.asarray([[0,   1, 3.0, 2],
-                        [3,   2, 4.0, 3]], dtype=np.int)
+        Z = np.asarray([[0, 1, 3.0, 2],
+                        [3, 2, 4.0, 3]], dtype=np.int)
         self.assertTrue(is_valid_linkage(Z) == False)
         self.assertRaises(TypeError, is_valid_linkage, Z, throw=True)
 
     def test_is_valid_linkage_5_columns(self):
         "Tests is_valid_linkage(Z) with 5 columns."
-        Z = np.asarray([[0,   1, 3.0, 2, 5],
-                        [3,   2, 4.0, 3, 3]], dtype=np.double)
+        Z = np.asarray([[0, 1, 3.0, 2, 5],
+                        [3, 2, 4.0, 3, 3]], dtype=np.double)
         self.assertTrue(is_valid_linkage(Z) == False)
         self.assertRaises(ValueError, is_valid_linkage, Z, throw=True)
 
     def test_is_valid_linkage_3_columns(self):
         "Tests is_valid_linkage(Z) with 3 columns."
-        Z = np.asarray([[0,   1, 3.0],
-                        [3,   2, 4.0]], dtype=np.double)
+        Z = np.asarray([[0, 1, 3.0],
+                        [3, 2, 4.0]], dtype=np.double)
         self.assertTrue(is_valid_linkage(Z) == False)
         self.assertRaises(ValueError, is_valid_linkage, Z, throw=True)
 
@@ -575,78 +555,78 @@ class TestIsValidLinkage(TestCase):
 
     def test_is_valid_linkage_1x4(self):
         "Tests is_valid_linkage(Z) on linkage over 2 observations."
-        Z = np.asarray([[0,   1, 3.0, 2]], dtype=np.double)
+        Z = np.asarray([[0, 1, 3.0, 2]], dtype=np.double)
         self.assertTrue(is_valid_linkage(Z) == True)
 
     def test_is_valid_linkage_2x4(self):
         "Tests is_valid_linkage(Z) on linkage over 3 observations."
-        Z = np.asarray([[0,   1, 3.0, 2],
-                        [3,   2, 4.0, 3]], dtype=np.double)
+        Z = np.asarray([[0, 1, 3.0, 2],
+                        [3, 2, 4.0, 3]], dtype=np.double)
         self.assertTrue(is_valid_linkage(Z) == True)
 
     def test_is_valid_linkage_4_and_up(self):
         "Tests is_valid_linkage(Z) on linkage on observation sets between sizes 4 and 15 (step size 3)."
         for i in xrange(4, 15, 3):
-            y = np.random.rand(i*(i-1)/2)
+            y = np.random.rand(i*(i-1)//2)
             Z = linkage(y)
             self.assertTrue(is_valid_linkage(Z) == True)
 
     def test_is_valid_linkage_4_and_up_neg_index_left(self):
         "Tests is_valid_linkage(Z) on linkage on observation sets between sizes 4 and 15 (step size 3) with negative indices (left)."
         for i in xrange(4, 15, 3):
-            y = np.random.rand(i*(i-1)/2)
+            y = np.random.rand(i*(i-1)//2)
             Z = linkage(y)
-            Z[int(i/2),0] = -2
+            Z[i//2,0] = -2
             self.assertTrue(is_valid_linkage(Z) == False)
             self.assertRaises(ValueError, is_valid_linkage, Z, throw=True)
 
     def test_is_valid_linkage_4_and_up_neg_index_right(self):
         "Tests is_valid_linkage(Z) on linkage on observation sets between sizes 4 and 15 (step size 3) with negative indices (right)."
         for i in xrange(4, 15, 3):
-            y = np.random.rand(i*(i-1)/2)
+            y = np.random.rand(i*(i-1)//2)
             Z = linkage(y)
-            Z[int(i/2),1] = -2
+            Z[i//2,1] = -2
             self.assertTrue(is_valid_linkage(Z) == False)
             self.assertRaises(ValueError, is_valid_linkage, Z, throw=True)
 
     def test_is_valid_linkage_4_and_up_neg_dist(self):
         "Tests is_valid_linkage(Z) on linkage on observation sets between sizes 4 and 15 (step size 3) with negative distances."
         for i in xrange(4, 15, 3):
-            y = np.random.rand(i*(i-1)/2)
+            y = np.random.rand(i*(i-1)//2)
             Z = linkage(y)
-            Z[int(i/2),2] = -0.5
+            Z[i//2,2] = -0.5
             self.assertTrue(is_valid_linkage(Z) == False)
             self.assertRaises(ValueError, is_valid_linkage, Z, throw=True)
 
     def test_is_valid_linkage_4_and_up_neg_counts(self):
         "Tests is_valid_linkage(Z) on linkage on observation sets between sizes 4 and 15 (step size 3) with negative counts."
         for i in xrange(4, 15, 3):
-            y = np.random.rand(i*(i-1)/2)
+            y = np.random.rand(i*(i-1)//2)
             Z = linkage(y)
-            Z[int(i/2),3] = -2
+            Z[i//2,3] = -2
             self.assertTrue(is_valid_linkage(Z) == False)
             self.assertRaises(ValueError, is_valid_linkage, Z, throw=True)
 
-class TestIsValidInconsistent(TestCase):
 
+class TestIsValidInconsistent(TestCase):
     def test_is_valid_im_int_type(self):
         "Tests is_valid_im(R) with integer type."
-        R = np.asarray([[0,   1, 3.0, 2],
-                        [3,   2, 4.0, 3]], dtype=np.int)
+        R = np.asarray([[0, 1, 3.0, 2],
+                        [3, 2, 4.0, 3]], dtype=np.int)
         self.assertTrue(is_valid_im(R) == False)
         self.assertRaises(TypeError, is_valid_im, R, throw=True)
 
     def test_is_valid_im_5_columns(self):
         "Tests is_valid_im(R) with 5 columns."
-        R = np.asarray([[0,   1, 3.0, 2, 5],
-                        [3,   2, 4.0, 3, 3]], dtype=np.double)
+        R = np.asarray([[0, 1, 3.0, 2, 5],
+                        [3, 2, 4.0, 3, 3]], dtype=np.double)
         self.assertTrue(is_valid_im(R) == False)
         self.assertRaises(ValueError, is_valid_im, R, throw=True)
 
     def test_is_valid_im_3_columns(self):
         "Tests is_valid_im(R) with 3 columns."
-        R = np.asarray([[0,   1, 3.0],
-                        [3,   2, 4.0]], dtype=np.double)
+        R = np.asarray([[0, 1, 3.0],
+                        [3, 2, 4.0]], dtype=np.double)
         self.assertTrue(is_valid_im(R) == False)
         self.assertRaises(ValueError, is_valid_im, R, throw=True)
 
@@ -658,19 +638,19 @@ class TestIsValidInconsistent(TestCase):
 
     def test_is_valid_im_1x4(self):
         "Tests is_valid_im(R) on im over 2 observations."
-        R = np.asarray([[0,   1, 3.0, 2]], dtype=np.double)
+        R = np.asarray([[0, 1, 3.0, 2]], dtype=np.double)
         self.assertTrue(is_valid_im(R) == True)
 
     def test_is_valid_im_2x4(self):
         "Tests is_valid_im(R) on im over 3 observations."
-        R = np.asarray([[0,   1, 3.0, 2],
-                        [3,   2, 4.0, 3]], dtype=np.double)
+        R = np.asarray([[0, 1, 3.0, 2],
+                        [3, 2, 4.0, 3]], dtype=np.double)
         self.assertTrue(is_valid_im(R) == True)
 
     def test_is_valid_im_4_and_up(self):
         "Tests is_valid_im(R) on im on observation sets between sizes 4 and 15 (step size 3)."
         for i in xrange(4, 15, 3):
-            y = np.random.rand(i*(i-1)/2)
+            y = np.random.rand(i*(i-1)//2)
             Z = linkage(y)
             R = inconsistent(Z)
             self.assertTrue(is_valid_im(R) == True)
@@ -678,35 +658,35 @@ class TestIsValidInconsistent(TestCase):
     def test_is_valid_im_4_and_up_neg_index_left(self):
         "Tests is_valid_im(R) on im on observation sets between sizes 4 and 15 (step size 3) with negative link height means."
         for i in xrange(4, 15, 3):
-            y = np.random.rand(i*(i-1)/2)
+            y = np.random.rand(i*(i-1)//2)
             Z = linkage(y)
             R = inconsistent(Z)
-            R[int(i/2),0] = -2.0
+            R[i//2,0] = -2.0
             self.assertTrue(is_valid_im(R) == False)
             self.assertRaises(ValueError, is_valid_im, R, throw=True)
 
     def test_is_valid_im_4_and_up_neg_index_right(self):
         "Tests is_valid_im(R) on im on observation sets between sizes 4 and 15 (step size 3) with negative link height standard deviations."
         for i in xrange(4, 15, 3):
-            y = np.random.rand(i*(i-1)/2)
+            y = np.random.rand(i*(i-1)//2)
             Z = linkage(y)
             R = inconsistent(Z)
-            R[int(i/2),1] = -2.0
+            R[i//2,1] = -2.0
             self.assertTrue(is_valid_im(R) == False)
             self.assertRaises(ValueError, is_valid_im, R, throw=True)
 
     def test_is_valid_im_4_and_up_neg_dist(self):
         "Tests is_valid_im(R) on im on observation sets between sizes 4 and 15 (step size 3) with negative link counts."
         for i in xrange(4, 15, 3):
-            y = np.random.rand(i*(i-1)/2)
+            y = np.random.rand(i*(i-1)//2)
             Z = linkage(y)
             R = inconsistent(Z)
-            R[int(i/2),2] = -0.5
+            R[i//2,2] = -0.5
             self.assertTrue(is_valid_im(R) == False)
             self.assertRaises(ValueError, is_valid_im, R, throw=True)
 
-class TestNumObsLinkage(TestCase):
 
+class TestNumObsLinkage(TestCase):
     def test_num_obs_linkage_empty(self):
         "Tests num_obs_linkage(Z) with empty linkage."
         Z = np.zeros((0, 4), dtype=np.double)
@@ -714,34 +694,34 @@ class TestNumObsLinkage(TestCase):
 
     def test_num_obs_linkage_1x4(self):
         "Tests num_obs_linkage(Z) on linkage over 2 observations."
-        Z = np.asarray([[0,   1, 3.0, 2]], dtype=np.double)
+        Z = np.asarray([[0, 1, 3.0, 2]], dtype=np.double)
         self.assertTrue(num_obs_linkage(Z) == 2)
 
     def test_num_obs_linkage_2x4(self):
         "Tests num_obs_linkage(Z) on linkage over 3 observations."
-        Z = np.asarray([[0,   1, 3.0, 2],
-                        [3,   2, 4.0, 3]], dtype=np.double)
+        Z = np.asarray([[0, 1, 3.0, 2],
+                        [3, 2, 4.0, 3]], dtype=np.double)
         self.assertTrue(num_obs_linkage(Z) == 3)
 
     def test_num_obs_linkage_4_and_up(self):
         "Tests num_obs_linkage(Z) on linkage on observation sets between sizes 4 and 15 (step size 3)."
         for i in xrange(4, 15, 3):
-            y = np.random.rand(i*(i-1)/2)
+            y = np.random.rand(i*(i-1)//2)
             Z = linkage(y)
             self.assertTrue(num_obs_linkage(Z) == i)
 
-class TestLeavesList(TestCase):
 
+class TestLeavesList(TestCase):
     def test_leaves_list_1x4(self):
         "Tests leaves_list(Z) on a 1x4 linkage."
-        Z = np.asarray([[0,   1, 3.0, 2]], dtype=np.double)
+        Z = np.asarray([[0, 1, 3.0, 2]], dtype=np.double)
         node = to_tree(Z)
         self.assertTrue((leaves_list(Z) == [0, 1]).all())
 
     def test_leaves_list_2x4(self):
         "Tests leaves_list(Z) on a 2x4 linkage."
-        Z = np.asarray([[0,   1, 3.0, 2],
-                        [3,   2, 4.0, 3]], dtype=np.double)
+        Z = np.asarray([[0, 1, 3.0, 2],
+                        [3, 2, 4.0, 3]], dtype=np.double)
         node = to_tree(Z)
         self.assertTrue((leaves_list(Z) == [0, 1, 2]).all())
 
@@ -793,8 +773,8 @@ class TestLeavesList(TestCase):
         node = to_tree(Z)
         self.assertTrue((node.pre_order() == leaves_list(Z)).all())
 
-class TestCorrespond(TestCase):
 
+class TestCorrespond(TestCase):
     def test_correspond_empty(self):
         "Tests correspond(Z, y) with empty linkage and condensed distance matrix."
         y = np.zeros((0,))
@@ -804,19 +784,19 @@ class TestCorrespond(TestCase):
     def test_correspond_2_and_up(self):
         "Tests correspond(Z, y) on linkage and CDMs over observation sets of different sizes."
         for i in xrange(2, 4):
-            y = np.random.rand(i*(i-1)/2)
+            y = np.random.rand(i*(i-1)//2)
             Z = linkage(y)
             self.assertTrue(correspond(Z, y))
         for i in xrange(4, 15, 3):
-            y = np.random.rand(i*(i-1)/2)
+            y = np.random.rand(i*(i-1)//2)
             Z = linkage(y)
             self.assertTrue(correspond(Z, y))
 
     def test_correspond_4_and_up(self):
         "Tests correspond(Z, y) on linkage and CDMs over observation sets of different sizes. Correspondance should be false."
-        for (i, j) in zip(range(2, 4), range(3, 5)) + zip(range(3, 5), range(2, 4)):
-            y = np.random.rand(i*(i-1)/2)
-            y2 = np.random.rand(j*(j-1)/2)
+        for (i, j) in list(zip(list(range(2, 4)), list(range(3, 5)))) + list(zip(list(range(3, 5)), list(range(2, 4)))):
+            y = np.random.rand(i*(i-1)//2)
+            y2 = np.random.rand(j*(j-1)//2)
             Z = linkage(y)
             Z2 = linkage(y2)
             self.assertTrue(correspond(Z, y2) == False)
@@ -824,9 +804,9 @@ class TestCorrespond(TestCase):
 
     def test_correspond_4_and_up_2(self):
         "Tests correspond(Z, y) on linkage and CDMs over observation sets of different sizes. Correspondance should be false."
-        for (i, j) in zip(range(2, 7), range(16, 21)) + zip(range(2, 7), range(16, 21)):
-            y = np.random.rand(i*(i-1)/2)
-            y2 = np.random.rand(j*(j-1)/2)
+        for (i, j) in list(zip(list(range(2, 7)), list(range(16, 21)))) + list(zip(list(range(2, 7)), list(range(16, 21)))):
+            y = np.random.rand(i*(i-1)//2)
+            y2 = np.random.rand(j*(j-1)//2)
             Z = linkage(y)
             Z2 = linkage(y2)
             self.assertTrue(correspond(Z, y2) == False)
@@ -838,12 +818,10 @@ class TestCorrespond(TestCase):
             X = np.random.rand(n, 4)
             Y = pdist(X)
             Z = linkage(Y)
-            #print Z
-            #print A.shape, Y.shape, Yr.shape
             self.assertTrue(num_obs_linkage(Z) == n)
 
-class TestIsMonotonic(TestCase):
 
+class TestIsMonotonic(TestCase):
     def test_is_monotonic_empty(self):
         "Tests is_monotonic(Z) on an empty linkage."
         Z = np.zeros((0, 4))
@@ -851,7 +829,7 @@ class TestIsMonotonic(TestCase):
 
     def test_is_monotonic_1x4(self):
         "Tests is_monotonic(Z) on 1x4 linkage. Expecting True."
-        Z = np.asarray([[0, 1, 0.3, 2]], dtype=np.double);
+        Z = np.asarray([[0, 1, 0.3, 2]], dtype=np.double)
         self.assertTrue(is_monotonic(Z) == True)
 
     def test_is_monotonic_2x4_T(self):
@@ -902,7 +880,7 @@ class TestIsMonotonic(TestCase):
     def test_is_monotonic_tdist_linkage2(self):
         "Tests is_monotonic(Z) on clustering generated by single linkage on tdist data set. Perturbing. Expecting False."
         Z = linkage(_ytdist, 'single')
-        Z[2,2]=0.0
+        Z[2,2] = 0.0
         self.assertTrue(is_monotonic(Z) == False)
 
     def test_is_monotonic_iris_linkage(self):
@@ -912,8 +890,8 @@ class TestIsMonotonic(TestCase):
         Z = linkage(X, 'single')
         self.assertTrue(is_monotonic(Z) == True)
 
-class TestMaxDists(TestCase):
 
+class TestMaxDists(TestCase):
     def test_maxdists_empty_linkage(self):
         "Tests maxdists(Z) on empty linkage. Expecting exception."
         Z = np.zeros((0, 4), dtype=np.double)
@@ -977,8 +955,8 @@ class TestMaxDists(TestCase):
         expectedMD = calculate_maximum_distances(Z)
         self.assertTrue(within_tol(MD, expectedMD, eps))
 
-class TestMaxInconsts(TestCase):
 
+class TestMaxInconsts(TestCase):
     def test_maxinconsts_empty_linkage(self):
         "Tests maxinconsts(Z, R) on empty linkage. Expecting exception."
         Z = np.zeros((0, 4), dtype=np.double)
@@ -1055,8 +1033,8 @@ class TestMaxInconsts(TestCase):
         expectedMD = calculate_maximum_inconsistencies(Z, R)
         self.assertTrue(within_tol(MD, expectedMD, eps))
 
-class TestMaxRStat(TestCase):
 
+class TestMaxRStat(TestCase):
     def test_maxRstat_float_index(self):
         "Tests maxRstat(Z, R, 3.3). Expecting exception."
         Z = np.asarray([[0, 1, 0.3, 4]], dtype=np.double)
@@ -1379,6 +1357,16 @@ class TestMaxRStat(TestCase):
         expectedMD = calculate_maximum_inconsistencies(Z, R, 3)
         self.assertTrue(within_tol(MD, expectedMD, eps))
 
+
+class TestDendrogram(TestCase):
+    def test_dendrogram_single_linkage_tdist(self):
+        "Tests dendrogram calculation on single linkage of the tdist data set."
+        Z = linkage(_ytdist, 'single')
+        R = dendrogram(Z, no_plot=True)
+        leaves = R["leaves"]
+        self.assertEqual(leaves, [2, 5, 1, 0, 3, 4])
+
+
 def calculate_maximum_distances(Z):
     "Used for testing correctness of maxdists. Very slow."
     n = Z.shape[0] + 1
@@ -1389,33 +1377,35 @@ def calculate_maximum_distances(Z):
         left = Z[i, 0]
         right = Z[i, 1]
         if left >= n:
-            q[0] = B[left - n]
+            q[0] = B[int(left) - n]
         if right >= n:
-            q[1] = B[right - n]
+            q[1] = B[int(right) - n]
         q[2] = Z[i, 2]
         B[i] = q.max()
     return B
 
+
 def calculate_maximum_inconsistencies(Z, R, k=3):
     "Used for testing correctness of maxinconsts. Very slow."
     n = Z.shape[0] + 1
     B = np.zeros((n-1,))
     q = np.zeros((3,))
-    #print R.shape
     for i in xrange(0, n - 1):
         q[:] = 0.0
         left = Z[i, 0]
         right = Z[i, 1]
         if left >= n:
-            q[0] = B[left - n]
+            q[0] = B[int(left) - n]
         if right >= n:
-            q[1] = B[right - n]
+            q[1] = B[int(right) - n]
         q[2] = R[i, k]
         B[i] = q.max()
     return B
 
+
 def within_tol(a, b, tol):
     return np.abs(a - b).max() < tol
 
+
 if __name__ == "__main__":
     run_module_suite()
