@@ -37,6 +37,8 @@ f, y and Jacobians by transparently translating them into the equivalent
 real valued system. It supports the real valued solvers (i.e not zvode) and is
 an alternative to ode with the zvode solver, sometimes performing better.
 """
+from __future__ import division, print_function, absolute_import
+
 
 # XXX: Integrators must have:
 # ===========================
@@ -88,8 +90,9 @@ import warnings
 
 from numpy import asarray, array, zeros, int32, isscalar, real, imag
 
-import vode as _vode
-import _dop
+from . import vode as _vode
+from . import _dop
+from . import lsoda as _lsoda
 
 
 #------------------------------------------------------------------------------
@@ -108,6 +111,7 @@ class ode(object):
     f : callable ``f(t, y, *f_args)``
         Rhs of the equation. t is a scalar, ``y.shape == (n,)``.
         ``f_args`` is set by calling ``set_f_params(*args)``.
+        `f` should return a scalar, array or list (not a tuple).
     jac : callable ``jac(t, y, *jac_args)``
         Jacobian of the rhs, ``jac[i,j] = d f[i] / d y[j]``.
         ``jac_args`` is set by calling ``set_f_params(*args)``.
@@ -199,6 +203,52 @@ class ode(object):
             failures, and for this problem one should instead use DVODE on the
             equivalent real system (in the real and imaginary parts of y).
 
+    "lsoda"
+
+        Real-valued Variable-coefficient Ordinary Differential Equation
+        solver, with fixed-leading-coefficient implementation. It provides
+        automatic method switching between implicit Adams method (for non-stiff
+        problems) and a method based on backward differentiation formulas (BDF)
+        (for stiff problems).
+
+        Source: http://www.netlib.org/odepack
+
+        .. warning::
+
+           This integrator is not re-entrant. You cannot have two `ode`
+           instances using the "lsoda" integrator at the same time.
+
+        This integrator accepts the following parameters in `set_integrator`
+        method of the `ode` class:
+
+        - atol : float or sequence
+          absolute tolerance for solution
+        - rtol : float or sequence
+          relative tolerance for solution
+        - lband : None or int
+        - rband : None or int
+          Jacobian band width, jac[i,j] != 0 for i-lband <= j <= i+rband.
+          Setting these requires your jac routine to return the jacobian
+          in packed format, jac_packed[i-j+lband, j] = jac[i,j].
+        - with_jacobian : bool
+          Whether to use the jacobian
+        - nsteps : int
+          Maximum number of (internally defined) steps allowed during one
+          call to the solver.
+        - first_step : float
+        - min_step : float
+        - max_step : float
+          Limits for the step sizes used by the integrator.
+        - max_order_ns : int
+          Maximum order used in the nonstiff case (default 12).
+        - max_order_s : int
+          Maximum order used in the stiff case (default 5).
+        - max_hnil : int
+          Maximum number of messages reporting too small step size (t + h = t)
+          (default 0)
+        - ixpr : int
+          Whether to generate extra printing at method switches (default False).
+
     "dopri5"
 
         This is an explicit runge-kutta method of order (4)5 due to Dormand &
@@ -232,6 +282,8 @@ class ode(object):
           Maximum factor to increase/decrease step size by in one step
         - beta : float
           Beta parameter for stabilised step size control.
+        - verbosity : int
+          Switch for printing messages (< 0 for no messages).
 
     "dop853"
 
@@ -262,7 +314,7 @@ class ode(object):
     >>> dt = 1
     >>> while r.successful() and r.t < t1:
     >>>     r.integrate(r.t+dt)
-    >>>     print r.t, r.y
+    >>>     print("%g %g" % (r.t, r.y))
 
     References
     ----------
@@ -329,9 +381,15 @@ class ode(object):
             mth = self._integrator.run_relax
         else:
             mth = self._integrator.run
-        self._y, self.t = mth(self.f, self.jac or (lambda: None),
-                            self._y, self.t, t,
-                            self.f_params, self.jac_params)
+
+        try:
+            self._y, self.t = mth(self.f, self.jac or (lambda: None),
+                                self._y, self.t, t,
+                                self.f_params, self.jac_params)
+        except SystemError:
+            # f2py issue with tuple returns, see ticket 1187.
+            raise ValueError('Function to integrate must not return a tuple.')
+
         return self._y
 
     def successful(self):
@@ -352,6 +410,26 @@ class ode(object):
         self.jac_params = args
         return self
 
+    def set_solout(self, solout):
+        """
+        Set callable to be called at every successful integration step.
+
+        Parameters
+        ----------
+        solout : callable
+            ``solout(t, y)`` is called at each internal integrator step,
+            t is a scalar providing the current independent position
+            y is the current soloution ``y.shape == (n,)``
+            solout should return -1 to stop integration
+            otherwise it should return None or 0
+
+        """
+        if self._integrator.supports_solout:
+            self._integrator.set_solout(solout)
+        else:
+            raise ValueError("selected integrator does not support solout,"
+                            + " choose another one")
+
 
 class complex_ode(ode):
     """
@@ -437,6 +515,26 @@ class complex_ode(ode):
         y = ode.integrate(self, t, step, relax)
         return y[::2] + 1j * y[1::2]
 
+    def set_solout(self, solout):
+        """
+        Set callable to be called at every successful integration step.
+
+        Parameters
+        ----------
+        solout : callable
+            ``solout(t, y)`` is called at each internal integrator step,
+            t is a scalar providing the current independent position
+            y is the current soloution ``y.shape == (n,)``
+            solout should return -1 to stop integration
+            otherwise it should return None or 0
+
+        """
+        if self._integrator.supports_solout:
+            self._integrator.set_solout(solout, complex=True)
+        else:
+            raise TypeError("selected integrator does not support solouta,"
+                            + "choose another one")
+
 
 #------------------------------------------------------------------------------
 # ODE integrators
@@ -448,6 +546,7 @@ def find_integrator(name):
             return cl
     return None
 
+
 class IntegratorConcurrencyError(RuntimeError):
     """
     Failure due to concurrent usage of an integrator that can be used
@@ -461,12 +560,14 @@ class IntegratorConcurrencyError(RuntimeError):
                "(see `ode.set_integrator`)") % name
         RuntimeError.__init__(self, msg)
 
+
 class IntegratorBase(object):
 
     runner = None            # runner is None => integrator is not available
     success = None           # success==1 if integrator was called successfully
     supports_run_relax = None
     supports_step = None
+    supports_solout = False
     integrator_classes = []
     scalar = float
 
@@ -780,6 +881,7 @@ class dopri5(IntegratorBase):
 
     runner = getattr(_dop, 'dopri5', None)
     name = 'dopri5'
+    supports_solout = True
 
     messages = {1: 'computation successful',
                 2: 'comput. successful (interrupted by solout)',
@@ -787,7 +889,7 @@ class dopri5(IntegratorBase):
                 -2: 'larger nmax is needed',
                 -3: 'step size becomes too small',
                 -4: 'problem is probably stiff (interrupted)',
-               }
+                }
 
     def __init__(self,
                  rtol=1e-6, atol=1e-12,
@@ -798,7 +900,8 @@ class dopri5(IntegratorBase):
                  ifactor=10.0,
                  dfactor=0.2,
                  beta=0.0,
-                 method=None
+                 method=None,
+                 verbosity=-1,  # no messages if negative
                  ):
         self.rtol = rtol
         self.atol = atol
@@ -809,7 +912,17 @@ class dopri5(IntegratorBase):
         self.ifactor = ifactor
         self.dfactor = dfactor
         self.beta = beta
+        self.verbosity = verbosity
         self.success = 1
+        self.set_solout(None)
+
+    def set_solout(self, solout, complex=False):
+        self.solout = solout
+        self.solout_cmplx = complex
+        if solout is None:
+            self.iout = 0
+        else:
+            self.iout = 1
 
     def reset(self, n, has_jac):
         work = zeros((8 * n + 21,), float)
@@ -822,9 +935,10 @@ class dopri5(IntegratorBase):
         self.work = work
         iwork = zeros((21,), int32)
         iwork[0] = self.nsteps
+        iwork[2] = self.verbosity
         self.iwork = iwork
         self.call_args = [self.rtol, self.atol, self._solout,
-                          self.work, self.iwork]
+                          self.iout, self.work, self.iwork]
         self.success = 1
 
     def run(self, f, jac, y0, t0, t1, f_params, jac_params):
@@ -836,10 +950,13 @@ class dopri5(IntegratorBase):
             self.success = 0
         return y, x
 
-    def _solout(self, *args):
-        # dummy solout function
-        pass
-
+    def _solout(self, nr, xold, x, y, nd, icomp, con):
+        if self.solout is not None:
+            if self.solout_cmplx:
+                y = y[::2] + 1j * y[1::2]
+            return self.solout(x, y)
+        else:
+            return 1
 
 if dopri5.runner is not None:
     IntegratorBase.integrator_classes.append(dopri5)
@@ -859,7 +976,8 @@ class dop853(dopri5):
                  ifactor=6.0,
                  dfactor=0.3,
                  beta=0.0,
-                 method=None
+                 method=None,
+                 verbosity=-1,  # no messages if negative
                  ):
         self.rtol = rtol
         self.atol = atol
@@ -870,7 +988,9 @@ class dop853(dopri5):
         self.ifactor = ifactor
         self.dfactor = dfactor
         self.beta = beta
+        self.verbosity = verbosity
         self.success = 1
+        self.set_solout(None)
 
     def reset(self, n, has_jac):
         work = zeros((11 * n + 21,), float)
@@ -883,11 +1003,146 @@ class dop853(dopri5):
         self.work = work
         iwork = zeros((21,), int32)
         iwork[0] = self.nsteps
+        iwork[2] = self.verbosity
         self.iwork = iwork
         self.call_args = [self.rtol, self.atol, self._solout,
-                          self.work, self.iwork]
+                          self.iout, self.work, self.iwork]
         self.success = 1
 
-
 if dop853.runner is not None:
     IntegratorBase.integrator_classes.append(dop853)
+
+
+class lsoda(IntegratorBase):
+
+    runner = getattr(_lsoda, 'lsoda', None)
+    active_global_handle = 0
+
+    messages = {
+        2: "Integration successful.",
+        -1: "Excess work done on this call (perhaps wrong Dfun type).",
+        -2: "Excess accuracy requested (tolerances too small).",
+        -3: "Illegal input detected (internal error).",
+        -4: "Repeated error test failures (internal error).",
+        -5: "Repeated convergence failures (perhaps bad Jacobian or tolerances).",
+        -6: "Error weight became zero during problem.",
+        -7: "Internal workspace insufficient to finish (internal error)."
+    }
+
+    def __init__(self,
+                 with_jacobian=0,
+                 rtol=1e-6, atol=1e-12,
+                 lband=None, uband=None,
+                 nsteps=500,
+                 max_step=0.0,  # corresponds to infinite
+                 min_step=0.0,
+                 first_step=0.0,  # determined by solver
+                 ixpr=0,
+                 max_hnil=0,
+                 max_order_ns=12,
+                 max_order_s=5,
+                 method=None
+                 ):
+
+        self.with_jacobian = with_jacobian
+        self.rtol = rtol
+        self.atol = atol
+        self.mu = uband
+        self.ml = lband
+
+        self.max_order_ns = max_order_ns
+        self.max_order_s = max_order_s
+        self.nsteps = nsteps
+        self.max_step = max_step
+        self.min_step = min_step
+        self.first_step = first_step
+        self.ixpr = ixpr
+        self.max_hnil = max_hnil
+        self.success = 1
+
+        self.initialized = False
+
+    def reset(self, n, has_jac):
+        # Calculate parameters for Fortran subroutine dvode.
+        if has_jac:
+            if self.mu is None and self.ml is None:
+                jt = 1
+            else:
+                if self.mu is None:
+                    self.mu = 0
+                if self.ml is None:
+                    self.ml = 0
+                jt = 4
+        else:
+            if self.mu is None and self.ml is None:
+                jt = 2
+            else:
+                if self.mu is None:
+                    self.mu = 0
+                if self.ml is None:
+                    self.ml = 0
+                jt = 5
+        lrn = 20 + (self.max_order_ns + 4) * n
+        if jt in [1, 2]:
+            lrs = 22 + (self.max_order_s + 4) * n + n * n
+        elif jt in [4, 5]:
+            lrs = 22 + (self.max_order_s + 5 + 2 * self.ml + self.mu) * n
+        else:
+            raise ValueError('Unexpected jt=%s' % jt)
+        lrw = max(lrn, lrs)
+        liw = 20 + n
+        rwork = zeros((lrw,), float)
+        rwork[4] = self.first_step
+        rwork[5] = self.max_step
+        rwork[6] = self.min_step
+        self.rwork = rwork
+        iwork = zeros((liw,), int32)
+        if self.ml is not None:
+            iwork[0] = self.ml
+        if self.mu is not None:
+            iwork[1] = self.mu
+        iwork[4] = self.ixpr
+        iwork[5] = self.nsteps
+        iwork[6] = self.max_hnil
+        iwork[7] = self.max_order_ns
+        iwork[8] = self.max_order_s
+        self.iwork = iwork
+        self.call_args = [self.rtol, self.atol, 1, 1,
+                          self.rwork, self.iwork, jt]
+        self.success = 1
+        self.initialized = False
+
+    def run(self, f,jac,y0,t0,t1,f_params,jac_params):
+        if self.initialized:
+            self.check_handle()
+        else:
+            self.initialized = True
+            self.acquire_new_handle()
+        args = [f, y0, t0, t1] + self.call_args[:-1] + \
+               [jac, self.call_args[-1], f_params, 0, jac_params]
+        y1, t, istate = self.runner(*args)
+        if istate < 0:
+            warnings.warn('lsoda: ' +
+                          self.messages.get(istate,
+                                            'Unexpected istate=%s' % istate))
+            self.success = 0
+        else:
+            self.call_args[3] = 2  # upgrade istate from 1 to 2
+        return y1, t
+
+    def step(self, *args):
+        itask = self.call_args[2]
+        self.call_args[2] = 2
+        r = self.run(*args)
+        self.call_args[2] = itask
+        return r
+
+    def run_relax(self, *args):
+        itask = self.call_args[2]
+        self.call_args[2] = 3
+        r = self.run(*args)
+        self.call_args[2] = itask
+        return r
+
+if lsoda.runner:
+    IntegratorBase.integrator_classes.append(lsoda)
