@@ -1,27 +1,32 @@
-
-
 """ Classes for interpolating values.
 """
+from __future__ import division, print_function, absolute_import
 
 __all__ = ['interp1d', 'interp2d', 'spline', 'spleval', 'splmake', 'spltopp',
            'ppform', 'lagrange']
 
 from numpy import shape, sometrue, array, transpose, searchsorted, \
-                  ones, logical_or, atleast_1d, atleast_2d, meshgrid, ravel, \
+                  ones, logical_or, atleast_1d, atleast_2d, ravel, \
                   dot, poly1d, asarray, intp
 import numpy as np
 import scipy.special as spec
 import math
 
-import fitpack
-import _fitpack
+from scipy.lib.six.moves import xrange
+
+from . import fitpack
+from . import dfitpack
+from . import _fitpack
+from .polyint import _Interpolator1D
+
 
 def reduce_sometrue(a):
     all = a
     while len(shape(all)) > 1:
-        all = sometrue(all,axis=0)
+        all = sometrue(all, axis=0)
     return all
 
+
 def lagrange(x, w):
     """
     Return a Lagrange interpolating polynomial.
@@ -39,15 +44,21 @@ def lagrange(x, w):
     w : array_like
         `w` represents the y-coordinates of a set of datapoints, i.e. f(`x`).
 
+    Returns
+    -------
+    lagrange : numpy.poly1d instance
+        The Lagrange interpolating polynomial.
+
     """
     M = len(x)
     p = poly1d(0.0)
     for j in xrange(M):
         pt = poly1d(w[j])
         for k in xrange(M):
-            if k == j: continue
+            if k == j:
+                continue
             fac = x[j]-x[k]
-            pt *= poly1d([1.0,-x[k]])/fac
+            pt *= poly1d([1.0, -x[k]])/fac
         p += pt
     return p
 
@@ -66,13 +77,16 @@ class interp2d(object):
     f: ``z = f(x, y)``. This class returns a function whose call method uses
     spline interpolation to find the value of new points.
 
+    If `x` and `y` represent a regular grid, consider using
+    RectBivariateSpline.
+
     Methods
     -------
     __call__
 
     Parameters
     ----------
-    x, y : 1-D ndarrays
+    x, y : array_like
         Arrays defining the data point coordinates.
 
         If the points lie on a regular grid, `x` can specify the column
@@ -80,34 +94,41 @@ class interp2d(object):
 
           >>> x = [0,1,2];  y = [0,3]; z = [[1,2,3], [4,5,6]]
 
-        Otherwise, x and y must specify the full coordinates for each point,
-        for example::
+        Otherwise, `x` and `y` must specify the full coordinates for each
+        point, for example::
 
           >>> x = [0,1,2,0,1,2];  y = [0,0,0,3,3,3]; z = [1,2,3,4,5,6]
 
         If `x` and `y` are multi-dimensional, they are flattened before use.
-
-    z : 1-D ndarray
+    z : array_like
         The values of the function to interpolate at the data points. If
-        `z` is a multi-dimensional array, it is flattened before use.
+        `z` is a multi-dimensional array, it is flattened before use.  The
+        length of a flattened `z` array is either
+        len(`x`)*len(`y`) if `x` and `y` specify the column and row coordinates
+        or ``len(z) == len(x) == len(y)`` if `x` and `y` specify coordinates
+        for each point.
     kind : {'linear', 'cubic', 'quintic'}, optional
         The kind of spline interpolation to use. Default is 'linear'.
     copy : bool, optional
-        If True, then data is copied, otherwise only a reference is held.
+        If True, the class makes internal copies of x, y and z.
+        If False, references may be used. The default is to copy.
     bounds_error : bool, optional
         If True, when interpolated values are requested outside of the
-        domain of the input data, an error is raised.
+        domain of the input data (x,y), a ValueError is raised.
         If False, then `fill_value` is used.
     fill_value : number, optional
         If provided, the value to use for points outside of the
-        interpolation domain. Defaults to NaN.
+        interpolation domain. If omitted (None), values outside
+        the domain are extrapolated.
 
     See Also
     --------
-    bisplrep, bisplev
+    RectBivariateSpline :
+        Much faster 2D interpolation if your input data is on a grid
+    bisplrep, bisplev :
         Spline interpolation based on FITPACK
     BivariateSpline : a more recent wrapper of the FITPACK routines
-    interp1d
+    interp1d : one dimension version of this function
 
     Notes
     -----
@@ -141,27 +162,56 @@ class interp2d(object):
     """
 
     def __init__(self, x, y, z, kind='linear', copy=True, bounds_error=False,
-                 fill_value=np.nan):
-        self.x, self.y, self.z = map(ravel, map(asarray, [x, y, z]))
+                 fill_value=None):
+        x = ravel(x)
+        y = ravel(y)
+        z = asarray(z)
 
-        if len(self.z) == len(self.x) * len(self.y):
-            self.x, self.y = meshgrid(x,y)
-            self.x, self.y = map(ravel, [self.x, self.y])
-        if len(self.x) != len(self.y):
-            raise ValueError("x and y must have equal lengths")
-        if len(self.z) != len(self.x):
-            raise ValueError("Invalid length for input z")
+        rectangular_grid = (z.size == len(x) * len(y))
+        if rectangular_grid:
+            if not np.all(x[1:] >= x[:-1]):
+                j = np.argsort(x)
+                x = x[j]
+                z = z[:, j]
+            if not np.all(y[1:] >= y[:-1]):
+                j = np.argsort(y)
+                y = y[j]
+                z = z[j, :]
+            z = ravel(z.T)
+        else:
+            z = ravel(z)
+            if len(x) != len(y):
+                raise ValueError(
+                    "x and y must have equal lengths for non rectangular grid")
+            if len(z) != len(x):
+                raise ValueError(
+                    "Invalid length for input z for non rectangular grid")
 
         try:
-            kx = ky = {'linear' : 1,
-                       'cubic' : 3,
-                       'quintic' : 5}[kind]
+            kx = ky = {'linear': 1,
+                       'cubic': 3,
+                       'quintic': 5}[kind]
         except KeyError:
             raise ValueError("Unsupported interpolation type.")
 
-        self.tck = fitpack.bisplrep(self.x, self.y, self.z, kx=kx, ky=ky, s=0.)
+        if not rectangular_grid:
+            # TODO: surfit is really not meant for interpolation!
+            self.tck = fitpack.bisplrep(x, y, z, kx=kx, ky=ky, s=0.0)
+        else:
+            nx, tx, ny, ty, c, fp, ier = dfitpack.regrid_smth(
+                x, y, z, None, None, None, None,
+                kx=kx, ky=ky, s=0.0)
+            self.tck = (tx[:nx], ty[:ny], c[:(nx - kx - 1) * (ny - ky - 1)],
+                        kx, ky)
 
-    def __call__(self,x,y,dx=0,dy=0):
+        self.bounds_error = bounds_error
+        self.fill_value = fill_value
+        self.x, self.y, self.z = [array(a, copy=copy) for a in (x, y, z)]
+
+        self.x_min, self.x_max = np.amin(x), np.amax(x)
+        self.y_min, self.y_max = np.amin(y), np.amax(y)
+
+    def __call__(self, x, y, dx=0, dy=0):
         """Interpolate the function.
 
         Parameters
@@ -184,15 +234,38 @@ class interp2d(object):
 
         x = atleast_1d(x)
         y = atleast_1d(y)
+
+        if x.ndim != 1 or y.ndim != 1:
+            raise ValueError("x and y should both be 1-D arrays")
+
+        if self.bounds_error or self.fill_value is not None:
+            out_of_bounds_x = (x < self.x_min) | (x > self.x_max)
+            out_of_bounds_y = (y < self.y_min) | (y > self.y_max)
+
+            any_out_of_bounds_x = np.any(out_of_bounds_x)
+            any_out_of_bounds_y = np.any(out_of_bounds_y)
+
+        if self.bounds_error and (any_out_of_bounds_x or any_out_of_bounds_y):
+            raise ValueError("Values out of range; x must be in %r, y in %r"
+                             % ((self.x_min, self.x_max),
+                                (self.y_min, self.y_max)))
+
         z = fitpack.bisplev(x, y, self.tck, dx, dy)
         z = atleast_2d(z)
         z = transpose(z)
-        if len(z)==1:
+
+        if self.fill_value is not None:
+            if any_out_of_bounds_x:
+                z[:, out_of_bounds_x] = self.fill_value
+            if any_out_of_bounds_y:
+                z[out_of_bounds_y, :] = self.fill_value
+
+        if len(z) == 1:
             z = z[0]
         return array(z)
 
 
-class interp1d(object):
+class interp1d(_Interpolator1D):
     """
     interp1d(x, y, kind='linear', axis=-1, copy=True, bounds_error=True,
              fill_value=np.nan)
@@ -205,16 +278,18 @@ class interp1d(object):
 
     Parameters
     ----------
-    x : array_like
+    x : (N,) array_like
         A 1-D array of monotonically increasing real values.
-    y : array_like
+    y : (...,N,...) array_like
         A N-D array of real values. The length of `y` along the interpolation
         axis must be equal to the length of `x`.
     kind : str or int, optional
         Specifies the kind of interpolation as a string
-        ('linear','nearest', 'zero', 'slinear', 'quadratic, 'cubic')
-        or as an integer specifying the order of the spline interpolator
-        to use. Default is 'linear'.
+        ('linear', 'nearest', 'zero', 'slinear', 'quadratic, 'cubic'
+        where 'slinear', 'quadratic' and 'cubic' refer to a spline
+        interpolation of first, second or third order) or as an integer
+        specifying the order of the spline interpolator to use.
+        Default is 'linear'.
     axis : int, optional
         Specifies the axis of `y` along which to interpolate.
         Interpolation defaults to the last axis of `y`.
@@ -222,7 +297,7 @@ class interp1d(object):
         If True, the class makes internal copies of x and y.
         If False, references to `x` and `y` are used. The default is to copy.
     bounds_error : bool, optional
-        If True, an error is thrown any time interpolation is attempted on
+        If True, a ValueError is raised any time interpolation is attempted on
         a value outside of the range of x (where extrapolation is
         necessary). If False, out of bounds values are assigned `fill_value`.
         By default, an error is raised.
@@ -255,20 +330,21 @@ class interp1d(object):
     def __init__(self, x, y, kind='linear', axis=-1,
                  copy=True, bounds_error=True, fill_value=np.nan):
         """ Initialize a 1D linear interpolation class."""
+        _Interpolator1D.__init__(self, x, y, axis=axis)
 
         self.copy = copy
         self.bounds_error = bounds_error
         self.fill_value = fill_value
 
         if kind in ['zero', 'slinear', 'quadratic', 'cubic']:
-            order = {'nearest':0, 'zero':0,'slinear':1,
-                     'quadratic':2, 'cubic':3}[kind]
+            order = {'nearest': 0, 'zero': 0,'slinear': 1,
+                     'quadratic': 2, 'cubic': 3}[kind]
             kind = 'spline'
         elif isinstance(kind, int):
             order = kind
             kind = 'spline'
         elif kind not in ('linear', 'nearest'):
-            raise NotImplementedError("%s is unsupported: Use fitpack "\
+            raise NotImplementedError("%s is unsupported: Use fitpack "
                                       "routines for other types." % kind)
         x = array(x, copy=self.copy)
         y = array(y, copy=self.copy)
@@ -282,46 +358,40 @@ class interp1d(object):
         if not issubclass(y.dtype.type, np.inexact):
             y = y.astype(np.float_)
 
-        # Normalize the axis to ensure that it is positive.
-        self.axis = axis % len(y.shape)
-        self._kind = kind
+        # Backward compatibility
+        self.axis = axis % y.ndim
 
+        # Interpolation goes internally along the first axis
+        self.y = y
+        y = self._reshape_yi(y)
+
+        # Adjust to interpolation kind; store reference to *unbound*
+        # interpolation methods, in order to avoid circular references to self
+        # stored in the bound instance methods, and therefore delayed garbage
+        # collection.  See: http://docs.python.org/2/reference/datamodel.html
         if kind in ('linear', 'nearest'):
             # Make a "view" of the y array that is rotated to the interpolation
             # axis.
-            axes = range(y.ndim)
-            del axes[self.axis]
-            axes.append(self.axis)
-            oriented_y = y.transpose(axes)
             minval = 2
-            len_y = oriented_y.shape[-1]
-            if kind == 'linear':
-                self._call = self._call_linear
-            elif kind == 'nearest':
+            if kind == 'nearest':
                 self.x_bds = (x[1:] + x[:-1]) / 2.0
-                self._call = self._call_nearest
+                self._call = self.__class__._call_nearest
+            else:
+                self._call = self.__class__._call_linear
         else:
-            axes = range(y.ndim)
-            del axes[self.axis]
-            axes.insert(0, self.axis)
-            oriented_y = y.transpose(axes)
             minval = order + 1
-            len_y = oriented_y.shape[0]
-            self._call = self._call_spline
-            self._spline = splmake(x,oriented_y,order=order)
+            self._spline = splmake(x, y, order=order)
+            self._call = self.__class__._call_spline
 
-        len_x = len(x)
-        if len_x != len_y:
-            raise ValueError("x and y arrays must be equal in length along "
-                             "interpolation axis.")
-        if len_x < minval:
+        if len(x) < minval:
             raise ValueError("x and y arrays must have at "
                              "least %d entries" % minval)
+
+        self._kind = kind
         self.x = x
-        self.y = oriented_y
+        self._y = y
 
     def _call_linear(self, x_new):
-
         # 2. Find where in the orignal data, the values to interpolate
         #    would be inserted.
         #    Note: If x_new[n] == x[m], then m is returned by searchsorted.
@@ -338,15 +408,15 @@ class interp1d(object):
 
         x_lo = self.x[lo]
         x_hi = self.x[hi]
-        y_lo = self.y[..., lo]
-        y_hi = self.y[..., hi]
+        y_lo = self._y[lo]
+        y_hi = self._y[hi]
 
         # Note that the following two expressions rely on the specifics of the
         # broadcasting semantics.
-        slope = (y_hi-y_lo) / (x_hi-x_lo)
+        slope = (y_hi - y_lo) / (x_hi - x_lo)[:, None]
 
         # 5. Calculate the actual value for each entry in x_new.
-        y_new = slope*(x_new-x_lo) + y_lo
+        y_new = slope*(x_new - x_lo)[:, None] + y_lo
 
         return y_new
 
@@ -360,70 +430,26 @@ class interp1d(object):
         x_new_indices = searchsorted(self.x_bds, x_new, side='left')
 
         # 3. Clip x_new_indices so that they are within the range of x indices.
-        x_new_indices = x_new_indices.clip(0,  len(self.x)-1).astype(intp)
+        x_new_indices = x_new_indices.clip(0, len(self.x)-1).astype(intp)
 
         # 4. Calculate the actual value for each entry in x_new.
-        y_new = self.y[..., x_new_indices]
+        y_new = self._y[x_new_indices]
 
         return y_new
 
     def _call_spline(self, x_new):
-        x_new =np.asarray(x_new)
-        result = spleval(self._spline,x_new.ravel())
-        return result.reshape(x_new.shape+result.shape[1:])
-
-    def __call__(self, x_new):
-        """Find interpolated y_new = f(x_new).
-
-        Parameters
-        ----------
-        x_new : number or array
-            New independent variable(s).
-
-        Returns
-        -------
-        y_new : ndarray
-            Interpolated value(s) corresponding to x_new.
-
-        """
+        return spleval(self._spline, x_new)
 
+    def _evaluate(self, x_new):
         # 1. Handle values in x_new that are outside of x.  Throw error,
         #    or return a list of mask array indicating the outofbounds values.
         #    The behavior is set by the bounds_error variable.
         x_new = asarray(x_new)
         out_of_bounds = self._check_bounds(x_new)
-
-        y_new = self._call(x_new)
-
-        # Rotate the values of y_new back so that they correspond to the
-        # correct x_new values. For N-D x_new, take the last (for linear)
-        # or first (for other splines) N axes
-        # from y_new and insert them where self.axis was in the list of axes.
-        nx = x_new.ndim
-        ny = y_new.ndim
-
-        # 6. Fill any values that were out of bounds with fill_value.
-        # and
-        # 7. Rotate the values back to their proper place.
-
-        if nx == 0:
-            # special case: x is a scalar
-            if out_of_bounds:
-                if ny == 0:
-                    return asarray(self.fill_value)
-                else:
-                    y_new[...] = self.fill_value
-            return asarray(y_new)
-        elif self._kind in ('linear', 'nearest'):
-            y_new[..., out_of_bounds] = self.fill_value
-            axes = range(ny - nx)
-            axes[self.axis:self.axis] = range(ny - nx, ny)
-            return y_new.transpose(axes)
-        else:
+        y_new = self._call(self, x_new)
+        if len(y_new) > 0:
             y_new[out_of_bounds] = self.fill_value
-            axes = range(nx, ny)
-            axes[self.axis:self.axis] = range(nx)
-            return y_new.transpose(axes)
+        return y_new
 
     def _check_bounds(self, x_new):
         """Check the inputs for being in the bounds of the interpolated data.
@@ -457,6 +483,7 @@ class interp1d(object):
         out_of_bounds = logical_or(below_bounds, above_bounds)
         return out_of_bounds
 
+
 class ppform(object):
     """The ppform of the piecewise polynomials is given in terms of coefficients
     and breaks.  The polynomial in the ith interval is
@@ -484,24 +511,24 @@ class ppform(object):
         res[~mask] = self.fill
         xx = xnew.compress(mask)
         indxs = np.searchsorted(self.breaks, xx)-1
-        indxs = indxs.clip(0,len(self.breaks))
+        indxs = indxs.clip(0, len(self.breaks))
         pp = self.coeffs
         diff = xx - self.breaks.take(indxs)
-        V = np.vander(diff,N=self.K)
+        V = np.vander(diff, N=self.K)
         # values = np.diag(dot(V,pp[:,indxs]))
-        values = array([dot(V[k,:],pp[:,indxs[k]]) for k in xrange(len(xx))])
+        values = array([dot(V[k, :], pp[:, indxs[k]]) for k in xrange(len(xx))])
         res[mask] = values
         res.shape = saveshape
         return res
 
     def fromspline(cls, xk, cvals, order, fill=0.0):
         N = len(xk)-1
-        sivals = np.empty((order+1,N), dtype=float)
-        for m in xrange(order,-1,-1):
+        sivals = np.empty((order+1, N), dtype=float)
+        for m in xrange(order, -1, -1):
             fact = spec.gamma(m+1)
             res = _fitpack._bspleval(xk[:-1], xk, cvals, order, m)
             res /= fact
-            sivals[order-m,:] = res
+            sivals[order-m, :] = res
         return cls(sivals, xk, fill=fill)
     fromspline = classmethod(fromspline)
 
@@ -511,11 +538,12 @@ def _dot0(a, b):
     if b.ndim <= 2:
         return dot(a, b)
     else:
-        axes = range(b.ndim)
+        axes = list(range(b.ndim))
         axes.insert(-1, 0)
         axes.pop(0)
         return dot(a, b.transpose(axes))
 
+
 def _find_smoothest(xk, yk, order, conds=None, B=None):
     # construct Bmatrix, and Jmatrix
     # e = J*c
@@ -527,7 +555,7 @@ def _find_smoothest(xk, yk, order, conds=None, B=None):
     if B is None:
         B = _fitpack._bsplmat(order, xk)
     J = _fitpack._bspldismat(order, xk)
-    u,s,vh = np.dual.svd(B)
+    u, s, vh = np.dual.svd(B)
     ind = K-1
     V2 = vh[-ind:,:].T
     V1 = vh[:-ind,:].T
@@ -542,15 +570,16 @@ def _find_smoothest(xk, yk, order, conds=None, B=None):
     tmp = dot(tmp,u.T)
     return _dot0(tmp, yk)
 
+
 def _setdiag(a, k, v):
     if not a.ndim == 2:
         raise ValueError("Input array should be 2-D.")
     M,N = a.shape
     if k > 0:
         start = k
-        num = N-k
+        num = N - k
     else:
-        num = M+k
+        num = M + k
         start = abs(k)*N
     end = start + num*(N+1)-1
     a.flat[start:end:(N+1)] = v
@@ -558,23 +587,24 @@ def _setdiag(a, k, v):
 # Return the spline that minimizes the dis-continuity of the
 # "order-th" derivative; for order >= 2.
 
+
 def _find_smoothest2(xk, yk):
-    N = len(xk)-1
-    Np1 = N+1
+    N = len(xk) - 1
+    Np1 = N + 1
     # find pseudo-inverse of B directly.
-    Bd = np.empty((Np1,N))
+    Bd = np.empty((Np1, N))
     for k in range(-N,N):
-        if (k<0):
-            l = np.arange(-k,Np1)
+        if (k < 0):
+            l = np.arange(-k, Np1)
             v = (l+k+1)
             if ((k+1) % 2):
                 v = -v
         else:
             l = np.arange(k,N)
-            v = N-l
+            v = N - l
             if ((k % 2)):
                 v = -v
-        _setdiag(Bd,k,v)
+        _setdiag(Bd, k, v)
     Bd /= (Np1)
     V2 = np.ones((Np1,))
     V2[1::2] = -1
@@ -592,6 +622,7 @@ def _find_smoothest2(xk, yk):
     mk = dot(np.eye(Np1)-res1, _dot0(Bd,b))
     return mk
 
+
 def _get_spline2_Bb(xk, yk, kind, conds):
     Np1 = len(xk)
     dk = xk[1:]-xk[:-1]
@@ -612,6 +643,7 @@ def _get_spline2_Bb(xk, yk, kind, conds):
     else:
         raise NotImplementedError("quadratic %s is not available" % kind)
 
+
 def _get_spline3_Bb(xk, yk, kind, conds):
     # internal function to compute different tri-diagonal system
     # depending on the kind of spline requested.
@@ -653,7 +685,6 @@ def _get_spline3_Bb(xk, yk, kind, conds):
 
         return B, b, append_func, nlu
 
-
     elif kind in ['clamped', 'endslope', 'first', 'not-a-knot', 'runout',
                   'parabolic']:
         if kind == 'endslope':
@@ -722,74 +753,102 @@ def _get_spline3_Bb(xk, yk, kind, conds):
 # conds is a tuple of an array and a vector
 #  giving the left-hand and the right-hand side
 #  of the additional equations to add to B
+
+
 def _find_user(xk, yk, order, conds, B):
     lh = conds[0]
     rh = conds[1]
-    B = np.concatenate((B,lh),axis=0)
-    w = np.concatenate((yk,rh),axis=0)
-    M,N = B.shape
-    if (M>N):
+    B = np.concatenate((B, lh), axis=0)
+    w = np.concatenate((yk, rh), axis=0)
+    M, N = B.shape
+    if (M > N):
         raise ValueError("over-specification of conditions")
-    elif (M<N):
+    elif (M < N):
         return _find_smoothest(xk, yk, order, None, B)
     else:
         return np.dual.solve(B, w)
 
 # If conds is None, then use the not_a_knot condition
 #  at K-1 farthest separated points in the interval
+
+
 def _find_not_a_knot(xk, yk, order, conds, B):
     raise NotImplementedError
     return _find_user(xk, yk, order, conds, B)
 
 # If conds is None, then ensure zero-valued second
 #  derivative at K-1 farthest separated points
+
+
 def _find_natural(xk, yk, order, conds, B):
     raise NotImplementedError
     return _find_user(xk, yk, order, conds, B)
 
 # If conds is None, then ensure zero-valued first
 #  derivative at K-1 farthest separated points
+
+
 def _find_clamped(xk, yk, order, conds, B):
     raise NotImplementedError
     return _find_user(xk, yk, order, conds, B)
 
+
 def _find_fixed(xk, yk, order, conds, B):
     raise NotImplementedError
     return _find_user(xk, yk, order, conds, B)
 
 # If conds is None, then use coefficient periodicity
 # If conds is 'function' then use function periodicity
+
+
 def _find_periodic(xk, yk, order, conds, B):
     raise NotImplementedError
     return _find_user(xk, yk, order, conds, B)
 
 # Doesn't use conds
+
+
 def _find_symmetric(xk, yk, order, conds, B):
     raise NotImplementedError
     return _find_user(xk, yk, order, conds, B)
 
 # conds is a dictionary with multiple values
+
+
 def _find_mixed(xk, yk, order, conds, B):
     raise NotImplementedError
     return _find_user(xk, yk, order, conds, B)
 
 
-def splmake(xk,yk,order=3,kind='smoothest',conds=None):
-    """Return a (xk, cvals, k) representation of a spline given
-    data-points where the (internal) knots are at the data-points.
+def splmake(xk, yk, order=3, kind='smoothest', conds=None):
+    """
+    Return a representation of a spline given data-points at internal knots
 
-    yk can be an N-d array to represent more than one curve, through
-    the same xk points. The first dimension is assumed to be the
-    interpolating dimension.
+    Parameters
+    ----------
+    xk : array_like
+        The input array of x values of rank 1
+    yk : array_like
+        The input array of y values of rank N. `yk` can be an N-d array to
+        represent more than one curve, through the same `xk` points. The first
+        dimension is assumed to be the interpolating dimension and is the same
+        length of `xk`.
+    order : int, optional
+        Order of the spline
+    kind : str, optional
+        Can be 'smoothest', 'not_a_knot', 'fixed', 'clamped', 'natural',
+        'periodic', 'symmetric', 'user', 'mixed' and it is ignored if order < 2
+    conds : optional
+        Conds
 
-    kind can be 'smoothest', 'not_a_knot', 'fixed',
-                'clamped', 'natural', 'periodic', 'symmetric',
-                'user', 'mixed'
+    Returns
+    -------
+    splmake : tuple
+        Return a (`xk`, `cvals`, `k`) representation of a spline given
+        data-points where the (internal) knots are at the data-points.
 
-                it is ignored if order < 2
     """
     yk = np.asanyarray(yk)
-    N = yk.shape[0]-1
 
     order = int(order)
     if order < 0:
@@ -809,19 +868,44 @@ def splmake(xk,yk,order=3,kind='smoothest',conds=None):
     coefs = func(xk, yk, order, conds, B)
     return xk, coefs, order
 
-def spleval((xj,cvals,k),xnew,deriv=0):
-    """Evaluate a fixed spline represented by the given tuple at the new
-    x-values. The xj values are the interior knot points.  The approximation
-    region is xj[0] to xj[-1].  If N+1 is the length of xj, then cvals should
-    have length N+k where k is the order of the spline.
 
-    Internally, an additional k-1 knot points are added on either side of
+def spleval(xck, xnew, deriv=0):
+    """
+    Evaluate a fixed spline represented by the given tuple at the new x-values
+
+    The `xj` values are the interior knot points.  The approximation
+    region is `xj[0]` to `xj[-1]`.  If N+1 is the length of `xj`, then `cvals`
+    should have length N+k where `k` is the order of the spline.
+
+    Parameters
+    ----------
+    (xj, cvals, k) : tuple
+        Parameters that define the fixed spline
+    xj : array_like
+        Interior knot points
+    cvals : array_like
+        Curvature
+    k : int
+        Order of the spline
+    xnew : array_like
+        Locations to calculate spline
+    deriv : int
+        Deriv
+
+    Returns
+    -------
+    spleval : ndarray
+        If `cvals` represents more than one curve (`cvals.ndim` > 1) and/or
+        `xnew` is N-d, then the result is `xnew.shape` + `cvals.shape[1:]`
+        providing the interpolation of multiple curves.
+
+    Notes
+    -----
+    Internally, an additional `k`-1 knot points are added on either side of
     the spline.
 
-    If cvals represents more than one curve (cvals.ndim > 1) and/or xnew is
-    N-d, then the result is xnew.shape + cvals.shape[1:] providing the
-    interpolation of multiple curves.
     """
+    (xj,cvals,k) = xck
     oldshape = np.shape(xnew)
     xx = np.ravel(xnew)
     sh = cvals.shape[1:]
@@ -836,12 +920,34 @@ def spleval((xj,cvals,k),xnew,deriv=0):
     res.shape = oldshape + sh
     return res
 
-def spltopp(xk,cvals,k):
+
+def spltopp(xk, cvals, k):
     """Return a piece-wise polynomial object from a fixed-spline tuple.
     """
     return ppform.fromspline(xk, cvals, k)
 
-def spline(xk,yk,xnew,order=3,kind='smoothest',conds=None):
-    """Interpolate a curve (xk,yk) at points xnew using a spline fit.
+
+def spline(xk, yk, xnew, order=3, kind='smoothest', conds=None):
+    """
+    Interpolate a curve at new points using a spline fit
+
+    Parameters
+    ----------
+    xk, yk : array_like
+        The x and y values that define the curve.
+    xnew : array_like
+        The x values where spline should estimate the y values.
+    order : int
+        Default is 3.
+    kind : string
+        One of {'smoothest'}
+    conds : Don't know
+        Don't know
+
+    Returns
+    -------
+    spline : ndarray
+        An array of y values; the spline evaluated at the positions `xnew`.
+
     """
     return spleval(splmake(xk,yk,order=order,kind=kind,conds=conds),xnew)
