@@ -1,31 +1,37 @@
 #!/usr/bin/env python
+from __future__ import division, print_function, absolute_import
 
 from os.path import join
 
+
 def configuration(parent_package='',top_path=None):
     from numpy.distutils.misc_util import Configuration
 
     config = Configuration('interpolate', parent_package, top_path)
 
-    config.add_library('fitpack',
-                       sources=[join('fitpack', '*.f')],
-                      )
+    fitpack_src = [join('fitpack', '*.f')]
+    config.add_library('fitpack', sources=fitpack_src)
+
+    config.add_extension('interpnd',
+                         sources=['interpnd.c'])
 
     config.add_extension('_fitpack',
                          sources=['src/_fitpackmodule.c'],
                          libraries=['fitpack'],
-                         depends = ['src/__fitpack.h','src/multipack.h']
-                        )
+                         depends=(['src/__fitpack.h','src/multipack.h']
+                                  + fitpack_src)
+                         )
 
     config.add_extension('dfitpack',
                          sources=['src/fitpack.pyf'],
                          libraries=['fitpack'],
-                        )
+                         depends=fitpack_src,
+                         )
 
     config.add_extension('_interpolate',
                          sources=['src/_interpolate.cpp'],
-                         include_dirs = ['src'],
-                         depends = ['src/interpolate.h'])
+                         include_dirs=['src'],
+                         depends=['src/interpolate.h'])
 
     config.add_data_dir('tests')
 
