@@ -35,9 +35,10 @@ Dataset: foo
 \tcolor's type is nominal, range is ('red', 'green', 'blue', 'yellow', 'black')
 
 """
+from __future__ import division, print_function, absolute_import
 
-from arffread import *
-import arffread
+from .arffread import *
+from . import arffread
 
 __all__ = arffread.__all__
 
