@@ -1,11 +1,14 @@
 #! /usr/bin/env python
 # Last Change: Mon Aug 20 08:00 PM 2007 J
+from __future__ import division, print_function, absolute_import
+
 import re
 import itertools
+from functools import partial
 
 import numpy as np
 
-from scipy.io.arff.utils import partial
+from scipy.lib.six import next
 
 """A module to read arff files."""
 
@@ -47,9 +50,12 @@ r_wcomattrval = re.compile(r"(\S+)\s+(..+$)")
 #-------------------------
 # Module defined exception
 #-------------------------
+
+
 class ArffError(IOError):
     pass
 
+
 class ParseArffError(ArffError):
     pass
 
@@ -58,6 +64,8 @@ class ParseArffError(ArffError):
 #------------------
 
 # An attribute  is defined as @attribute name value
+
+
 def parse_type(attrtype):
     """Given an arff attribute value (meta data), returns its type.
 
@@ -86,7 +94,7 @@ def get_nominal(attribute):
 
 def read_data_list(ofile):
     """Read each line of the iterable and put it in a list."""
-    data = [ofile.next()]
+    data = [next(ofile)]
     if data[0].strip()[0] == '{':
         raise ValueError("This looks like a sparse ARFF: not supported yet")
     data.extend([i for i in ofile])
@@ -95,7 +103,7 @@ def read_data_list(ofile):
 
 def get_ndata(ofile):
     """Read the whole file to get number of data attributes."""
-    data = [ofile.next()]
+    data = [next(ofile)]
     loc = 1
     if data[0].strip()[0] == '{':
         raise ValueError("This looks like a sparse ARFF: not supported yet")
@@ -119,7 +127,7 @@ def maxnomlen(atrv):
     -------
     slen : int
        length of longest component
-    
+
     Examples
     --------
     maxnomlen("{floup, bouga, fl, ratata}") returns 6 (the size of
@@ -165,7 +173,7 @@ def go_data(ofile):
     """Skip header.
 
     the first next() call of the returned iterator will be the @data line"""
-    return itertools.dropwhile(lambda x : not r_datameta.match(x), ofile)
+    return itertools.dropwhile(lambda x: not r_datameta.match(x), ofile)
 
 
 #----------------
@@ -176,7 +184,7 @@ def tokenize_attribute(iterable, attribute):
 
     Given a raw string attribute, try to get the name and type of the
     attribute. Constraints:
-    
+
     * The first line must start with @attribute (case insensitive, and
       space like characters before @attribute are allowed)
     * Works also if the attribute is spread on multilines.
@@ -204,7 +212,7 @@ def tokenize_attribute(iterable, attribute):
     >>> iterable = iter([0] * 10) # dummy iterator
     >>> tokenize_attribute(iterable, r"@attribute floupi real")
     ('floupi', 'real', 0)
-     
+
     If attribute is r"'floupi 2' real", will return 'floupi 2' as name,
     and real as value.
 
@@ -219,10 +227,10 @@ def tokenize_attribute(iterable, attribute):
         atrv = mattr.group(1)
         if r_comattrval.match(atrv):
             name, type = tokenize_single_comma(atrv)
-            next_item = iterable.next()
+            next_item = next(iterable)
         elif r_wcomattrval.match(atrv):
             name, type = tokenize_single_wcomma(atrv)
-            next_item = iterable.next()
+            next_item = next(iterable)
         else:
             # Not sure we should support this, as it does not seem supported by
             # weka.
@@ -242,17 +250,17 @@ def tokenize_multilines(iterable, val):
     # line with meta character, and try to parse everything up to there.
     if not r_mcomattrval.match(val):
         all = [val]
-        i = iterable.next()
+        i = next(iterable)
         while not r_meta.match(i):
             all.append(i)
-            i = iterable.next()
+            i = next(iterable)
         if r_mend.search(i):
             raise ValueError("relational attribute not supported yet")
-        print "".join(all[:-1])
+        print("".join(all[:-1]))
         m = r_comattrval.match("".join(all[:-1]))
         return m.group(1), m.group(2), i
     else:
-        raise ValueError("Cannot parse attribute names spread over multi "\
+        raise ValueError("Cannot parse attribute names spread over multi "
                         "lines yet")
 
 
@@ -288,11 +296,11 @@ def tokenize_single_wcomma(val):
 
 def read_header(ofile):
     """Read the header of the iterable ofile."""
-    i = ofile.next()
+    i = next(ofile)
 
     # Pass first comments
     while r_comment.match(i):
-        i = ofile.next()
+        i = next(ofile)
 
     # Header is everything up to DATA attribute ?
     relation = None
@@ -310,9 +318,9 @@ def read_header(ofile):
                     relation = isrel.group(1)
                 else:
                     raise ValueError("Error parsing line %s" % i)
-                i = ofile.next()
+                i = next(ofile)
         else:
-            i = ofile.next()
+            i = next(ofile)
 
     return relation, attributes
 
@@ -397,8 +405,8 @@ class MetaData(object):
 
     Knows about attributes names and types.
 
-    Example
-    -------
+    Examples
+    --------
     data, meta = loadarff('iris.arff')
     # This will print the attributes names of the iris.arff dataset
     for i in meta:
@@ -454,7 +462,7 @@ class MetaData(object):
         return attr_types
 
 
-def loadarff(filename):
+def loadarff(f):
     """
     Read an arff file.
 
@@ -466,8 +474,8 @@ def loadarff(filename):
 
     Parameters
     ----------
-    filename : str
-       the name of the file
+    f : file-like or str
+       File-like object to read from, or filename to open.
 
     Returns
     -------
@@ -499,12 +507,22 @@ def loadarff(filename):
     points as NaNs.
 
     """
-    ofile = open(filename)
+    if hasattr(f, 'read'):
+        ofile = f
+    else:
+        ofile = open(f, 'rt')
+    try:
+        return _loadarff(ofile)
+    finally:
+        if ofile is not f:  # only close what we opened
+            ofile.close()
 
+
+def _loadarff(ofile):
     # Parse the header file
     try:
         rel, attr = read_header(ofile)
-    except ValueError, e:
+    except ValueError as e:
         msg = "Error while parsing header, error was: " + str(e)
         raise ParseArffError(msg)
 
@@ -524,8 +542,8 @@ def loadarff(filename):
 
     # This can be used once we want to support integer as integer values and
     # not as numeric anymore (using masked arrays ?).
-    acls2dtype = {'real' : np.float, 'integer' : np.float, 'numeric' : np.float}
-    acls2conv = {'real' : safe_float, 'integer' : safe_float, 'numeric' : safe_float}
+    acls2dtype = {'real': np.float, 'integer': np.float, 'numeric': np.float}
+    acls2conv = {'real': safe_float, 'integer': safe_float, 'numeric': safe_float}
     descr = []
     convertors = []
     if not hasstr:
@@ -537,7 +555,7 @@ def loadarff(filename):
                 n = maxnomlen(value)
                 descr.append((name, 'S%d' % n))
                 pvalue = get_nom_val(value)
-                convertors.append(partial(safe_nominal, pvalue = pvalue))
+                convertors.append(partial(safe_nominal, pvalue=pvalue))
             else:
                 descr.append((name, acls2dtype[type]))
                 convertors.append(safe_float)
@@ -553,26 +571,26 @@ def loadarff(filename):
     # Get the delimiter from the first line of data:
     def next_data_line(row_iter):
         """Assumes we are already in the data part (eg after @data)."""
-        raw = row_iter.next()
+        raw = next(row_iter)
         while r_empty.match(raw):
-            raw = row_iter.next()
+            raw = next(row_iter)
         while r_comment.match(raw):
-            raw = row_iter.next()
+            raw = next(row_iter)
         return raw
 
     try:
         try:
             dtline = next_data_line(ofile)
             delim = get_delim(dtline)
-        except ValueError, e:
+        except ValueError as e:
             raise ParseArffError("Error while parsing delimiter: " + str(e))
     finally:
         ofile.seek(0, 0)
         ofile = go_data(ofile)
         # skip the @data line
-        ofile.next()
+        next(ofile)
 
-    def generator(row_iter, delim = ','):
+    def generator(row_iter, delim=','):
         # TODO: this is where we are spending times (~80%). I think things
         # could be made more efficiently:
         #   - We could for example "compile" the function, because some values
@@ -586,28 +604,28 @@ def loadarff(filename):
 
         # We do not abstract skipping comments and empty lines for performances
         # reason.
-        raw = row_iter.next()
+        raw = next(row_iter)
         while r_empty.match(raw):
-            raw = row_iter.next()
+            raw = next(row_iter)
         while r_comment.match(raw):
-            raw = row_iter.next()
+            raw = next(row_iter)
 
         # 'compiling' the range since it does not change
         # Note, I have already tried zipping the converters and
         # row elements and got slightly worse performance.
-        elems = range(ni)
+        elems = list(range(ni))
 
         row = raw.split(delim)
         yield tuple([convertors[i](row[i]) for i in elems])
         for raw in row_iter:
             while r_comment.match(raw):
-                raw = row_iter.next()
+                raw = next(row_iter)
             while r_empty.match(raw):
-                raw = row_iter.next()
+                raw = next(row_iter)
             row = raw.split(delim)
             yield tuple([convertors[i](row[i]) for i in elems])
 
-    a = generator(ofile, delim = delim)
+    a = generator(ofile, delim=delim)
     # No error should happen here: it is a bug otherwise
     data = np.fromiter(a, descr)
     return data, meta
@@ -625,35 +643,35 @@ def print_attribute(name, tp, data):
     type = tp[0]
     if type == 'numeric' or type == 'real' or type == 'integer':
         min, max, mean, std = basic_stats(data)
-        print "%s,%s,%f,%f,%f,%f" % (name, type, min, max, mean, std)
+        print("%s,%s,%f,%f,%f,%f" % (name, type, min, max, mean, std))
     else:
         msg = name + ",{"
         for i in range(len(tp[1])-1):
             msg += tp[1][i] + ","
         msg += tp[1][-1]
         msg += "}"
-        print msg
+        print(msg)
 
 
 def test_weka(filename):
     data, meta = loadarff(filename)
-    print len(data.dtype)
-    print data.size
+    print(len(data.dtype))
+    print(data.size)
     for i in meta:
         print_attribute(i,meta[i],data[i])
 
 # make sure nose does not find this as a test
 test_weka.__test__ = False
-        
+
 
 def floupi(filename):
     data, meta = loadarff(filename)
     from attrselect import print_dataset_info
     print_dataset_info(data)
-    print "relation %s, has %d instances" % (meta.name, data.size)
+    print("relation %s, has %d instances" % (meta.name, data.size))
     itp = iter(types)
     for i in data.dtype.names:
-        print_attribute(i,itp.next(),data[i])
+        print_attribute(i,next(itp),data[i])
         #tp = itp.next()
         #if tp == 'numeric' or tp == 'real' or tp == 'integer':
         #    min, max, mean, std = basic_stats(data[i])
