@@ -1,19 +1,22 @@
 """LU decomposition functions."""
 
+from __future__ import division, print_function, absolute_import
+
 from warnings import warn
 
 from numpy import asarray, asarray_chkfinite
 
 # Local imports
-from misc import _datacopied
-from lapack import get_lapack_funcs
-from flinalg import get_flinalg_funcs
+from .misc import _datacopied
+from .lapack import get_lapack_funcs
+from .flinalg import get_flinalg_funcs
 
 __all__ = ['lu', 'lu_solve', 'lu_factor']
 
 
-def lu_factor(a, overwrite_a=False):
-    """Compute pivoted LU decomposition of a matrix.
+def lu_factor(a, overwrite_a=False, check_finite=True):
+    """
+    Compute pivoted LU decomposition of a matrix.
 
     The decomposition is::
 
@@ -24,17 +27,21 @@ def lu_factor(a, overwrite_a=False):
 
     Parameters
     ----------
-    a : array, shape (M, M)
+    a : (M, M) array_like
         Matrix to decompose
     overwrite_a : boolean
         Whether to overwrite data in A (may increase performance)
+    check_finite : boolean, optional
+        Whether to check that the input matrix contains only finite numbers.
+        Disabling may give a performance gain, but may result in problems
+        (crashes, non-termination) if the inputs do contain infinities or NaNs.
 
     Returns
     -------
-    lu : array, shape (N, N)
+    lu : (N, N) ndarray
         Matrix containing U in its upper triangle, and L in its lower triangle.
         The unit diagonal elements of L are not stored.
-    piv : array, shape (N,)
+    piv : (N,) ndarray
         Pivot indices representing the permutation matrix P:
         row i of matrix was interchanged with row piv[i].
 
@@ -47,7 +54,10 @@ def lu_factor(a, overwrite_a=False):
     This is a wrapper to the ``*GETRF`` routines from LAPACK.
 
     """
-    a1 = asarray(a)
+    if check_finite:
+        a1 = asarray_chkfinite(a)
+    else:
+        a1 = asarray(a)
     if len(a1.shape) != 2 or (a1.shape[0] != a1.shape[1]):
         raise ValueError('expected square matrix')
     overwrite_a = overwrite_a or (_datacopied(a1, a))
@@ -62,7 +72,7 @@ def lu_factor(a, overwrite_a=False):
     return lu, piv
 
 
-def lu_solve((lu, piv), b, trans=0, overwrite_b=False):
+def lu_solve(lu_and_piv, b, trans=0, overwrite_b=False, check_finite=True):
     """Solve an equation system, a x = b, given the LU factorization of a
 
     Parameters
@@ -81,6 +91,10 @@ def lu_solve((lu, piv), b, trans=0, overwrite_b=False):
         1      a^T x = b
         2      a^H x = b
         =====  =========
+    check_finite : boolean, optional
+        Whether to check that the input matrices contain only finite numbers.
+        Disabling may give a performance gain, but may result in problems
+        (crashes, non-termination) if the inputs do contain infinities or NaNs.
 
     Returns
     -------
@@ -92,7 +106,11 @@ def lu_solve((lu, piv), b, trans=0, overwrite_b=False):
     lu_factor : LU factorize a matrix
 
     """
-    b1 = asarray_chkfinite(b)
+    (lu, piv) = lu_and_piv
+    if check_finite:
+        b1 = asarray_chkfinite(b)
+    else:
+        b1 = asarray(b)
     overwrite_b = overwrite_b or _datacopied(b1, b)
     if lu.shape[0] != b1.shape[0]:
         raise ValueError("incompatible dimensions.")
@@ -105,8 +123,9 @@ def lu_solve((lu, piv), b, trans=0, overwrite_b=False):
                                                                     % -info)
 
 
-def lu(a, permute_l=False, overwrite_a=False):
-    """Compute pivoted LU decompostion of a matrix.
+def lu(a, permute_l=False, overwrite_a=False, check_finite=True):
+    """
+    Compute pivoted LU decompostion of a matrix.
 
     The decomposition is::
 
@@ -117,29 +136,35 @@ def lu(a, permute_l=False, overwrite_a=False):
 
     Parameters
     ----------
-    a : array, shape (M, N)
+    a : (M, N) array_like
         Array to decompose
-    permute_l : boolean
+    permute_l : bool
         Perform the multiplication P*L  (Default: do not permute)
-    overwrite_a : boolean
+    overwrite_a : bool
         Whether to overwrite data in a (may improve performance)
+    check_finite : boolean, optional
+        Whether to check that the input matrix contains only finite numbers.
+        Disabling may give a performance gain, but may result in problems
+        (crashes, non-termination) if the inputs do contain infinities or NaNs.
 
     Returns
     -------
-    (If permute_l == False)
-    p : array, shape (M, M)
+    **(If permute_l == False)**
+
+    p : (M, M) ndarray
         Permutation matrix
-    l : array, shape (M, K)
+    l : (M, K) ndarray
         Lower triangular or trapezoidal matrix with unit diagonal.
         K = min(M, N)
-    u : array, shape (K, N)
+    u : (K, N) ndarray
         Upper triangular or trapezoidal matrix
 
-    (If permute_l == True)
-    pl : array, shape (M, K)
+    **(If permute_l == True)**
+
+    pl : (M, K) ndarray
         Permuted L matrix.
         K = min(M, N)
-    u : array, shape (K, N)
+    u : (K, N) ndarray
         Upper triangular or trapezoidal matrix
 
     Notes
@@ -147,7 +172,10 @@ def lu(a, permute_l=False, overwrite_a=False):
     This is a LU factorization routine written for Scipy.
 
     """
-    a1 = asarray_chkfinite(a)
+    if check_finite:
+        a1 = asarray_chkfinite(a)
+    else:
+        a1 = asarray(a)
     if len(a1.shape) != 2:
         raise ValueError('expected matrix')
     overwrite_a = overwrite_a or (_datacopied(a1, a))
