@@ -41,234 +41,24 @@ typedef struct {
     void* next;
 } _index_pair;
 
-#define CASE_LABEL(_p, _pi, _type) \
-case t ## _type:                   \
-    *_p = *(_type*)_pi ? -1 : 0;     \
-    break
-
-int NI_Label(PyArrayObject* input, PyArrayObject* strct,
-                         maybelong *max_label, PyArrayObject* output)
-{
-    int kk;
-    maybelong jj, ll, ssize, size, filter_size, *offsets = NULL;
-    maybelong mask_value, *oo;
-    Bool *ps, *footprint = NULL;
-    char *pi, *po;
-    Int32 index = 0, *index_map = NULL;
-    NI_FilterIterator fi;
-    NI_Iterator ii, io;
-    _index_pair *pairs = NULL;
-
-    /* structure size */
-    ssize = 1;
-    for(kk = 0; kk < strct->nd; kk++)
-        ssize *= strct->dimensions[kk];
-    /* we only use the first half of the structure data, so we make a
-         temporary structure for use with the filter functions: */
-    footprint = (Bool*)malloc(ssize * sizeof(Bool));
-    if (!footprint) {
-        PyErr_NoMemory();
-        goto exit;
-    }
-    ps = (Bool*)PyArray_DATA(strct);
-    filter_size = 0;
-    for(jj = 0; jj < ssize / 2; jj++) {
-        footprint[jj] = ps[jj];
-        if (ps[jj])
-            ++filter_size;
-    }
-    for(jj = ssize / 2; jj < ssize; jj++)
-        footprint[jj] = 0;
-    /* get data and size */
-    pi = (void *)PyArray_DATA(input);
-    po = (void *)PyArray_DATA(output);
-    size = 1;
-    for(kk = 0; kk < output->nd; kk++)
-        size *= output->dimensions[kk];
-    if (!NI_InitPointIterator(input, &ii))
-        goto exit;
-    if (!NI_InitPointIterator(output, &io))
-        goto exit;
-    /* set all elements in the output corresponding to non-zero elements
-         in input to -1: */
-    for(jj = 0; jj < size; jj++) {
-        Int32 *p = (Int32*)po;
-        switch (input->descr->type_num) {
-        CASE_LABEL(p, pi, Bool);
-        CASE_LABEL(p, pi, UInt8);
-        CASE_LABEL(p, pi, UInt16);
-        CASE_LABEL(p, pi, UInt32);
-#if HAS_UINT64
-        CASE_LABEL(p, pi, UInt64);
-#endif
-        CASE_LABEL(p, pi, Int8);
-        CASE_LABEL(p, pi, Int16);
-        CASE_LABEL(p, pi, Int32);
-        CASE_LABEL(p, pi, Int64);
-        CASE_LABEL(p, pi, Float32);
-        CASE_LABEL(p, pi, Float64);
-        default:
-            PyErr_SetString(PyExc_RuntimeError, "data type not supported");
-            goto exit;
-        }
-        NI_ITERATOR_NEXT2(ii, io, pi, po);
-    }
-
-    /* calculate the filter offsets: */
-    if (!NI_InitFilterOffsets(output, footprint, strct->dimensions, NULL,
-                                                    NI_EXTEND_CONSTANT, &offsets, &mask_value, NULL))
-        goto exit;
-    /* initialize filter iterator: */
-    if (!NI_InitFilterIterator(input->nd, strct->dimensions, filter_size,
-                                                                                     input->dimensions, NULL, &fi))
-        goto exit;
-    /* reset output iterator: */
-    NI_ITERATOR_RESET(io);
-    po = (void *)PyArray_DATA(output);
-    /* iterator over the elements: */
-    oo = offsets;
-    for(jj = 0; jj < size; jj++) {
-        if (*(Int32*)po < 0) {
-            Int32 neighbor = 0;
-            /* iterate over structuring element: */
-            for(ll = 0; ll < filter_size; ll++) {
-                int offset = oo[ll];
-                if (offset != mask_value) {
-                    Int32 tt = *(Int32*)(po + offset);
-                    if (tt > 0) {
-                        /* this element is next to an already found object: */
-                        if (neighbor && neighbor != tt) {
-                            /* we have two objects that must be merged later: */
-                            _index_pair* tp = (_index_pair*)malloc(sizeof(_index_pair));
-                            if (!tp) {
-                                PyErr_NoMemory();
-                                goto exit;
-                            }
-                            tp->next = pairs;
-                            /* the pairs must be ordered: */
-                            if (neighbor < tt) {
-                                tp->index1 = neighbor;
-                                tp->index2 = tt;
-                            } else {
-                                tp->index1 = tt;
-                                tp->index2 = neighbor;
-                            }
-                            pairs = tp;
-                        } else {
-                            neighbor = tt;
-                        }
-                    }
-                }
-            }
-            if (neighbor) {
-                /* this point belongs to an existing object */
-                *(Int32*)po = neighbor;
-            } else {
-                /* this may be a new object: */
-                *(Int32*)po = ++index;
-            }
-        }
-        NI_FILTER_NEXT(fi, io, oo, po);
-    }
-    *max_label = index;
-    /* merge any touching objects: */
-    if (pairs) {
-        Int32 counter;
-        index_map = (Int32*)malloc(index * sizeof(Int32));
-        if (!index_map) {
-            PyErr_NoMemory();
-            goto exit;
-        }
-        for(jj = 0; jj < index; jj++)
-            index_map[jj] = (Int32)jj;
-        while (pairs) {
-            Int32 idx1 = pairs->index1 - 1;
-            Int32 idx2 = pairs->index2 - 1;
-            if (index_map[idx2] == idx1 || index_map[idx2] == idx2) {
-                /* if this pair was already processed, or if idx2 was not
-                     mapped yet, we delete this pair and map idx2 to idx1: */
-                _index_pair *tp = pairs;
-                pairs = tp->next;
-                free(tp);
-                index_map[idx2] = idx1;
-            } else {
-                /* idx2 was already mapped, therefore we find what it was
-                     mapped to and change the current pair to the result of that
-                     and idx1. Since the pair is not destroyed, it will be
-                     re-processed with the adapted values.  */
-                idx2 = index_map[idx2];
-                /* keep the pairs ordered: */
-                if (idx1 < idx2) {
-                    pairs->index1 = idx1 + 1;
-                    pairs->index2 = idx2 + 1;
-                } else {
-                    pairs->index1 = idx2 + 1;
-                    pairs->index2 = idx1 + 1;
-                }
-            }
-        }
-        for(jj = 0; jj < index; jj++) {
-            /* if the current index maps to a index that is also mapped,
-                 change it to map to that index. Since an index always maps to
-                 a lower index or to itself, this will make sure that at the
-                 end all indices map to an unmapped index. */
-            if (index_map[index_map[jj]] < index_map[jj])
-                index_map[jj] = index_map[index_map[jj]];
-        }
-        /* renumber the indices that are not mapped: */
-        counter = 0;
-        for(jj = 0; jj < index; jj++)
-            if (index_map[jj] == jj)
-                index_map[jj] = ++counter;
-            else
-                index_map[jj] = index_map[index_map[jj]];
-    }
-
-    /* relabel the output if we merged some objects: */
-    if (index_map) {
-        *max_label = 0;
-        NI_ITERATOR_RESET(io);
-        po = (void *)PyArray_DATA(output);
-        for(jj = 0; jj < size; jj++) {
-            Int32 p = *(Int32*)po;
-            if (p > 0 )
-                *(Int32*)po = index_map[p - 1];
-            if (*(Int32*)po > *max_label)
-                *max_label = *(Int32*)po;
-            NI_ITERATOR_NEXT(io, po);
-        }
-    }
- exit:
-    if (offsets) free(offsets);
-    if (index_map) free(index_map);
-    while (pairs) {
-        _index_pair *tp = pairs;
-        pairs = (_index_pair*)pairs->next;
-        free(tp);
-    }
-    if (footprint)
-        free(footprint);
-    return PyErr_Occurred() ? 0 : 1;
-}
-
 #define CASE_FIND_OBJECT_POINT(_pi, _regions, _rank, _dimensions, \
                                                              _max_label, _ii, _type)            \
 case t ## _type:                                                  \
 {                                                                 \
     int _kk;                                                        \
-    maybelong _sindex = *(_type*)_pi - 1;                           \
+    npy_intp _sindex = *(_type*)_pi - 1;                            \
     if (_sindex >= 0 && _sindex < _max_label) {                     \
         if (_rank > 0) {                                              \
             _sindex *= 2 * _rank;                                       \
             if (_regions[_sindex] < 0) {                                \
                 for(_kk = 0; _kk < _rank; _kk++) {                        \
-                    maybelong _cc = _ii.coordinates[_kk];                   \
+                    npy_intp _cc = _ii.coordinates[_kk];            \
                     _regions[_sindex + _kk] = _cc;                          \
                     _regions[_sindex + _kk + _rank] = _cc + 1;              \
                 }                                                         \
             } else {                                                    \
                 for(_kk = 0; _kk < _rank; _kk++) {                        \
-                    maybelong _cc = _ii.coordinates[_kk];                   \
+                    npy_intp _cc = _ii.coordinates[_kk];            \
                     if (_cc < _regions[_sindex + _kk])                      \
                         _regions[_sindex + _kk] = _cc;                        \
                     if (_cc + 1 > _regions[_sindex + _kk + _rank])          \
@@ -282,11 +72,11 @@ case t ## _type:                                                  \
 }                                                                 \
 break
 
-int NI_FindObjects(PyArrayObject* input, maybelong max_label,
-                                     maybelong* regions)
+int NI_FindObjects(PyArrayObject* input, npy_intp max_label,
+                                     npy_intp* regions)
 {
     int kk;
-    maybelong size, jj;
+    npy_intp size, jj;
     NI_Iterator ii;
     char *pi;
 
@@ -306,7 +96,7 @@ int NI_FindObjects(PyArrayObject* input, maybelong max_label,
     }
     /* iterate over all points: */
     for(jj = 0 ; jj < size; jj++) {
-        switch (input->descr->type_num) {
+        switch (NI_NormalizeType(input->descr->type_num)) {
         CASE_FIND_OBJECT_POINT(pi, regions, input->nd, input->dimensions,
                                                      max_label, ii,  Bool);
         CASE_FIND_OBJECT_POINT(pi, regions, input->nd, input->dimensions,
@@ -378,7 +168,8 @@ int NI_FindObjects(PyArrayObject* input, maybelong max_label,
         _v = *(Float64*)_pi;                                              \
         break;                                                            \
     default:                                                            \
-            PyErr_SetString(PyExc_RuntimeError, "data type not supported"); \
+            PyErr_SetString(PyExc_RuntimeError,                       \
+                                         "data type not supported");  \
             return 0;                                                       \
     }                                                                   \
 }
@@ -417,7 +208,8 @@ int NI_FindObjects(PyArrayObject* input, maybelong max_label,
         _v = *(Float64*)_pi;                                              \
         break;                                                            \
     default:                                                            \
-            PyErr_SetString(PyExc_RuntimeError, "data type not supported"); \
+            PyErr_SetString(PyExc_RuntimeError,                       \
+                                        "data type not supported");   \
             return 0;                                                       \
     }                                                                   \
 }
@@ -463,7 +255,8 @@ int NI_FindObjects(PyArrayObject* input, maybelong max_label,
             _label = *(Float64*)_pm;                                        \
             break;                                                          \
         default:                                                          \
-            PyErr_SetString(PyExc_RuntimeError, "data type not supported"); \
+            PyErr_SetString(PyExc_RuntimeError,                       \
+                                        "data type not supported");   \
             return 0;                                                       \
         }                                                                 \
     }                                                                   \
@@ -504,7 +297,8 @@ int NI_FindObjects(PyArrayObject* input, maybelong max_label,
             _label = *(Float64*)_pm;                                        \
             break;                                                          \
         default:                                                          \
-            PyErr_SetString(PyExc_RuntimeError, "data type not supported"); \
+            PyErr_SetString(PyExc_RuntimeError,                       \
+                                        "data type not supported");   \
             return 0;                                                       \
         }                                                                 \
     }                                                                   \
@@ -512,13 +306,13 @@ int NI_FindObjects(PyArrayObject* input, maybelong max_label,
 #endif
 
 int NI_Statistics(PyArrayObject *input, PyArrayObject *labels,
-    maybelong min_label, maybelong max_label, maybelong *indices,
-    maybelong n_results, double *sum, maybelong *total, double *variance,
-    double *minimum, double *maximum, maybelong* min_pos, maybelong* max_pos)
+    npy_intp min_label, npy_intp max_label, npy_intp *indices,
+    npy_intp n_results, double *sum, npy_intp *total, double *variance,
+    double *minimum, double *maximum, npy_intp* min_pos, npy_intp* max_pos)
 {
     char *pi = NULL, *pm = NULL;
     NI_Iterator ii, mi;
-    maybelong jj, size, idx = 0, label = 1, doit = 1;
+    npy_intp jj, size, idx = 0, label = 1, doit = 1;
     int qq;
 
     /* input iterator: */
@@ -553,7 +347,7 @@ int NI_Statistics(PyArrayObject *input, PyArrayObject *labels,
     }
     /* iterate over array: */
     for(jj = 0; jj < size; jj++) {
-        NI_GET_LABEL(pm, label, labels->descr->type_num);
+        NI_GET_LABEL(pm, label, NI_NormalizeType(labels->descr->type_num));
         if (min_label >= 0) {
             if (label >= min_label && label <= max_label) {
                 idx = indices[label - min_label];
@@ -566,7 +360,7 @@ int NI_Statistics(PyArrayObject *input, PyArrayObject *labels,
         }
         if (doit) {
             double val;
-            NI_GET_VALUE(pi, val, input->descr->type_num);
+            NI_GET_VALUE(pi, val, NI_NormalizeType(input->descr->type_num));
             if (sum)
                 sum[idx] += val;
             if (total)
@@ -617,7 +411,7 @@ int NI_Statistics(PyArrayObject *input, PyArrayObject *labels,
                 pm = (void *)PyArray_DATA(labels);
             }
             for(jj = 0; jj < size; jj++) {
-                NI_GET_LABEL(pm, label, labels->descr->type_num);
+                NI_GET_LABEL(pm, label, NI_NormalizeType(labels->descr->type_num));
                 if (min_label >= 0) {
                     if (label >= min_label && label <= max_label) {
                         idx = indices[label - min_label];
@@ -630,7 +424,7 @@ int NI_Statistics(PyArrayObject *input, PyArrayObject *labels,
                 }
                 if (doit) {
                     double val;
-                    NI_GET_VALUE(pi, val, input->descr->type_num);
+                    NI_GET_VALUE(pi, val, NI_NormalizeType(input->descr->type_num));
                     val = val - sum[idx] / total[idx];
                     variance[idx] += val * val;
                 }
@@ -650,12 +444,12 @@ int NI_Statistics(PyArrayObject *input, PyArrayObject *labels,
 
 
 int NI_CenterOfMass(PyArrayObject *input, PyArrayObject *labels,
-                            maybelong min_label, maybelong max_label, maybelong *indices,
-                            maybelong n_results, double *center_of_mass)
+                    npy_intp min_label, npy_intp max_label, npy_intp *indices,
+                    npy_intp n_results, double *center_of_mass)
 {
     char *pi = NULL, *pm = NULL;
     NI_Iterator ii, mi;
-    maybelong jj, kk, size, idx = 0, label = 1, doit = 1;
+    npy_intp jj, kk, size, idx = 0, label = 1, doit = 1;
     double *sum = NULL;
     int qq;
 
@@ -685,7 +479,7 @@ int NI_CenterOfMass(PyArrayObject *input, PyArrayObject *labels,
     }
     /* iterate over array: */
     for(jj = 0; jj < size; jj++) {
-        NI_GET_LABEL(pm, label, labels->descr->type_num);
+        NI_GET_LABEL(pm, label, NI_NormalizeType(labels->descr->type_num));
         if (min_label >= 0) {
             if (label >= min_label && label <= max_label) {
                 idx = indices[label - min_label];
@@ -698,7 +492,7 @@ int NI_CenterOfMass(PyArrayObject *input, PyArrayObject *labels,
         }
         if (doit) {
             double val;
-            NI_GET_VALUE(pi, val, input->descr->type_num);
+            NI_GET_VALUE(pi, val, NI_NormalizeType(input->descr->type_num));
             sum[idx] += val;
             for(kk = 0; kk < input->nd; kk++)
                 center_of_mass[idx * input->nd + kk] += val * ii.coordinates[kk];
@@ -720,13 +514,13 @@ int NI_CenterOfMass(PyArrayObject *input, PyArrayObject *labels,
 
 
 int NI_Histogram(PyArrayObject *input, PyArrayObject *labels,
-                            maybelong min_label, maybelong max_label, maybelong *indices,
-                            maybelong n_results, PyArrayObject **histograms,
-                            double min, double max, maybelong nbins)
+                 npy_intp min_label, npy_intp max_label, npy_intp *indices,
+                 npy_intp n_results, PyArrayObject **histograms,
+                 double min, double max, npy_intp nbins)
 {
     char *pi = NULL, *pm = NULL;
     NI_Iterator ii, mi;
-    maybelong jj, kk, size, idx = 0, label = 1, doit = 1;
+    npy_intp jj, kk, size, idx = 0, label = 1, doit = 1;
     Int32 **ph = NULL;
     double bsize;
     int qq;
@@ -758,7 +552,7 @@ int NI_Histogram(PyArrayObject *input, PyArrayObject *labels,
         size *= input->dimensions[qq];
     /* iterate over array: */
     for(jj = 0; jj < size; jj++) {
-        NI_GET_LABEL(pm, label, labels->descr->type_num);
+        NI_GET_LABEL(pm, label, NI_NormalizeType(labels->descr->type_num));
         if (min_label >= 0) {
             if (label >= min_label && label <= max_label) {
                 idx = indices[label - min_label];
@@ -772,7 +566,7 @@ int NI_Histogram(PyArrayObject *input, PyArrayObject *labels,
         if (doit) {
             int bin;
             double val;
-            NI_GET_VALUE(pi, val, input->descr->type_num);
+            NI_GET_VALUE(pi, val, NI_NormalizeType(input->descr->type_num));
             if (val >= min && val < max) {
                 bin = (int)((val - min) / bsize);
                 ++(ph[idx][bin]);
@@ -797,7 +591,7 @@ do {                                                              \
         _out = _index * sizeof(_type);                                \
     } else {                                                        \
         int _qq;                                                      \
-        maybelong _cc, _idx = _index;                                 \
+        npy_intp _cc, _idx = _index;                              \
         _out = 0;                                                     \
         for (_qq = 0; _qq < _rank; _qq++) {                           \
             _cc = _idx / _c_strides[_qq];                               \
@@ -854,7 +648,7 @@ break
 #define WS_MAXDIM 7
 
 typedef struct {
-    maybelong index;
+    npy_intp index;
     COST_TYPE cost;
     void *next, *prev;
     DONE_TYPE done;
@@ -865,9 +659,9 @@ int NI_WatershedIFT(PyArrayObject* input, PyArrayObject* markers,
 {
     char *pl, *pm, *pi;
     int ll;
-    maybelong size, jj, hh, kk, maxval;
-    maybelong strides[WS_MAXDIM], coordinates[WS_MAXDIM];
-    maybelong *nstrides = NULL, nneigh, ssize;
+    npy_intp size, jj, hh, kk, maxval;
+    npy_intp strides[WS_MAXDIM], coordinates[WS_MAXDIM];
+    npy_intp *nstrides = NULL, nneigh, ssize;
     int i_contiguous, o_contiguous;
     NI_WatershedElement *temp = NULL, **first = NULL, **last = NULL;
     Bool *ps = NULL;
@@ -898,7 +692,7 @@ int NI_WatershedIFT(PyArrayObject* input, PyArrayObject* markers,
     maxval = 0;
     for(jj = 0; jj < size; jj++) {
         int ival = 0;
-        switch(input->descr->type_num) {
+        switch(NI_NormalizeType(input->descr->type_num)) {
         CASE_GET_INPUT(ival, pi, UInt8);
         CASE_GET_INPUT(ival, pi, UInt16);
         default:
@@ -937,7 +731,7 @@ int NI_WatershedIFT(PyArrayObject* input, PyArrayObject* markers,
     for(jj = 0; jj < size; jj++) {
         /* get marker */
         int label = 0;
-        switch(markers->descr->type_num) {
+        switch(NI_NormalizeType(markers->descr->type_num)) {
         CASE_GET_LABEL(label, pm, UInt8);
         CASE_GET_LABEL(label, pm, UInt16);
         CASE_GET_LABEL(label, pm, UInt32);
@@ -952,7 +746,7 @@ int NI_WatershedIFT(PyArrayObject* input, PyArrayObject* markers,
             PyErr_SetString(PyExc_RuntimeError, "data type not supported");
             goto exit;
         }
-        switch(output->descr->type_num) {
+        switch(NI_NormalizeType(output->descr->type_num)) {
         CASE_PUT_LABEL(label, pl, UInt8);
         CASE_PUT_LABEL(label, pl, UInt16);
         CASE_PUT_LABEL(label, pl, UInt32);
@@ -978,8 +772,8 @@ int NI_WatershedIFT(PyArrayObject* input, PyArrayObject* markers,
                 last[0] = first[0];
             } else {
                 if (label > 0) {
-                    /* object markers are enqueued at the beginning, so they are
-                         processed first. */
+                    /* object markers are enqueued at the beginning, so they
+                       are processed first. */
                     temp[jj].next = first[0];
                     temp[jj].prev = NULL;
                     first[0]->prev = &(temp[jj]);
@@ -1014,7 +808,7 @@ int NI_WatershedIFT(PyArrayObject* input, PyArrayObject* markers,
     for (kk = 0; kk < ssize; kk++)
         if (ps[kk] && kk != (ssize / 2))
             ++nneigh;
-    nstrides = (maybelong*)malloc(nneigh * sizeof(maybelong));
+    nstrides = (npy_intp*)malloc(nneigh * sizeof(npy_intp));
     if (!nstrides) {
         PyErr_NoMemory();
         goto exit;
@@ -1057,7 +851,7 @@ int NI_WatershedIFT(PyArrayObject* input, PyArrayObject* markers,
             v->done = 1;
             /* Iterate over the neighbors of the element: */
             for(hh = 0; hh < nneigh; hh++) {
-                maybelong v_index = v->index, p_index = v->index, idx, cc;
+                npy_intp v_index = v->index, p_index = v->index, idx, cc;
                 int qq, outside = 0;
                 p_index += nstrides[hh];
                 /* check if the neighbor is within the extent of the array: */
@@ -1075,7 +869,7 @@ int NI_WatershedIFT(PyArrayObject* input, PyArrayObject* markers,
                     if (!(p->done)) {
                         /* If the neighbor was not processed yet: */
                         int max, pval, vval, wvp, pcost, label, p_idx, v_idx;
-                        switch(input->descr->type_num) {
+                        switch(NI_NormalizeType(input->descr->type_num)) {
                         CASE_WINDEX1(v_index, p_index, strides, input->strides,
                                                  input->nd, i_contiguous, p_idx, v_idx, pi,
                                                  vval, pval, UInt8);
@@ -1100,7 +894,7 @@ int NI_WatershedIFT(PyArrayObject* input, PyArrayObject* markers,
                                  adapt the cost and the label of the neighbor: */
                             int idx;
                             p->cost = max;
-                            switch(output->descr->type_num) {
+                            switch(NI_NormalizeType(output->descr->type_num)) {
                             CASE_WINDEX2(v_index, strides, output->strides, input->nd,
                                                      idx, o_contiguous, label, pl, UInt8);
                             CASE_WINDEX2(v_index, strides, output->strides, input->nd,
@@ -1124,7 +918,7 @@ int NI_WatershedIFT(PyArrayObject* input, PyArrayObject* markers,
                                                                 "data type not supported");
                                 goto exit;
                             }
-                            switch(output->descr->type_num) {
+                            switch(NI_NormalizeType(output->descr->type_num)) {
                             CASE_WINDEX3(p_index, strides, output->strides, input->nd,
                                                      idx, o_contiguous, label, pl, UInt8);
                             CASE_WINDEX3(p_index, strides, output->strides, input->nd,
