@@ -7,6 +7,8 @@ import numpy as np
 
 import scipy.ndimage as ndimage
 
+import os.path
+
 types = [np.int8, np.uint8, np.int16,
          np.uint16, np.int32, np.uint32,
          np.int64, np.uint64,
@@ -15,6 +17,7 @@ types = [np.int8, np.uint8, np.int16,
 
 np.mod(1., 1)  # Silence fmod bug on win-amd64. See #1408 and #1238.
 
+
 class Test_measurements_stats(TestCase):
     """ndimage.measurements._stats() is a utility function used by other functions."""
 
@@ -118,36 +121,48 @@ def test_label01():
     data = np.ones([])
     out, n = ndimage.label(data)
     assert_array_almost_equal(out, 1)
+    assert_equal(n, 1)
+
 
 def test_label02():
     "label 2"
     data = np.zeros([])
     out, n = ndimage.label(data)
     assert_array_almost_equal(out, 0)
+    assert_equal(n, 0)
+
 
 def test_label03():
     "label 3"
     data = np.ones([1])
     out, n = ndimage.label(data)
     assert_array_almost_equal(out, [1])
+    assert_equal(n, 1)
+
 
 def test_label04():
     "label 4"
     data = np.zeros([1])
     out, n = ndimage.label(data)
     assert_array_almost_equal(out, [0])
+    assert_equal(n, 0)
+
 
 def test_label05():
     "label 5"
     data = np.ones([5])
     out, n = ndimage.label(data)
     assert_array_almost_equal(out, [1, 1, 1, 1, 1])
+    assert_equal(n, 1)
+
 
 def test_label06():
     "label 6"
     data = np.array([1, 0, 1, 1, 0, 1])
     out, n = ndimage.label(data)
     assert_array_almost_equal(out, [1, 0, 2, 2, 0, 3])
+    assert_equal(n, 3)
+
 
 def test_label07():
     "label 7"
@@ -164,6 +179,8 @@ def test_label07():
                                [0, 0, 0, 0, 0, 0],
                                [0, 0, 0, 0, 0, 0],
                                [0, 0, 0, 0, 0, 0]])
+    assert_equal(n, 0)
+
 
 def test_label08():
     "label 8"
@@ -180,6 +197,8 @@ def test_label08():
                                [3, 3, 0, 0, 0, 0],
                                [3, 3, 0, 0, 0, 0],
                                [0, 0, 0, 4, 4, 0]])
+    assert_equal(n, 4)
+
 
 def test_label09():
     "label 9"
@@ -197,6 +216,8 @@ def test_label09():
                                [2, 2, 0, 0, 0, 0],
                                [2, 2, 0, 0, 0, 0],
                                [0, 0, 0, 3, 3, 0]])
+    assert_equal(n, 3)
+
 
 def test_label10():
     "label 10"
@@ -210,6 +231,8 @@ def test_label10():
                                [0, 1, 1, 0, 1, 0],
                                [0, 1, 1, 1, 1, 0],
                                [0, 0, 0, 0, 0, 0]])
+    assert_equal(n, 1)
+
 
 def test_label11():
     "label 11"
@@ -230,6 +253,27 @@ def test_label11():
         assert_array_almost_equal(out, expected)
         assert_equal(n, 4)
 
+
+def test_label11_inplace():
+    "label 11 in place"
+    for type in types:
+        data = np.array([[1, 0, 0, 0, 0, 0],
+                               [0, 0, 1, 1, 0, 0],
+                               [0, 0, 1, 1, 1, 0],
+                               [1, 1, 0, 0, 0, 0],
+                               [1, 1, 0, 0, 0, 0],
+                               [0, 0, 0, 1, 1, 0]], type)
+        n = ndimage.label(data, output=data)
+        expected = [[1, 0, 0, 0, 0, 0],
+                    [0, 0, 2, 2, 0, 0],
+                    [0, 0, 2, 2, 2, 0],
+                    [3, 3, 0, 0, 0, 0],
+                    [3, 3, 0, 0, 0, 0],
+                    [0, 0, 0, 4, 4, 0]]
+        assert_array_almost_equal(data, expected)
+        assert_equal(n, 4)
+
+
 def test_label12():
     "label 12"
     for type in types:
@@ -247,6 +291,7 @@ def test_label12():
         assert_array_almost_equal(out, expected)
         assert_equal(n, 1)
 
+
 def test_label13():
     "label 13"
     for type in types:
@@ -263,6 +308,7 @@ def test_label13():
         assert_array_almost_equal(out, expected)
         assert_equal(n, 1)
 
+
 def test_label_output_typed():
     "test label with specified output with type"
     data = np.ones([5])
@@ -272,6 +318,7 @@ def test_label_output_typed():
         assert_array_almost_equal(output, 1)
         assert_equal(n, 1)
 
+
 def test_label_output_dtype():
     "test label with specified output dtype"
     data = np.ones([5])
@@ -280,12 +327,39 @@ def test_label_output_dtype():
         assert_array_almost_equal(output, 1)
         assert output.dtype == t
 
+
 def test_label_output_wrong_size():
     "test label with output of wrong size"
     data = np.ones([5])
     for t in types:
         output = np.zeros([10], t)
-        assert_raises(RuntimeError, ndimage.label, data, output=output)
+        assert_raises((RuntimeError, ValueError), ndimage.label, data, output=output)
+
+
+def test_label_structuring_elements():
+    "test label with different structuring element neighborhoods"
+    data = np.loadtxt(os.path.join(os.path.dirname(__file__), "data", "label_inputs.txt"))
+    strels = np.loadtxt(os.path.join(os.path.dirname(__file__), "data", "label_strels.txt"))
+    results = np.loadtxt(os.path.join(os.path.dirname(__file__), "data", "label_results.txt"))
+    data = data.reshape((-1, 7, 7))
+    strels = strels.reshape((-1, 3, 3))
+    results = results.reshape((-1, 7, 7))
+    r = 0
+    for i in range(data.shape[0]):
+        d = data[i, :, :]
+        for j in range(strels.shape[0]):
+            s = strels[j, :, :]
+            assert_equal(ndimage.label(d, s)[0], results[r, :, :])
+            r += 1
+
+
+def test_label_default_dtype():
+    test_array = np.random.rand(10, 10)
+    label, no_features = ndimage.label(test_array > 0.5)
+    assert_(label.dtype in (np.int32, np.int64))
+    # Shouldn't raise an exception
+    ndimage.find_objects(label)
+
 
 def test_find_objects01():
     "find_objects 1"
@@ -293,30 +367,35 @@ def test_find_objects01():
     out = ndimage.find_objects(data)
     assert_(out == [()])
 
+
 def test_find_objects02():
     "find_objects 2"
     data = np.zeros([], dtype=int)
     out = ndimage.find_objects(data)
     assert_(out == [])
 
+
 def test_find_objects03():
     "find_objects 3"
     data = np.ones([1], dtype=int)
     out = ndimage.find_objects(data)
     assert_equal(out, [(slice(0, 1, None),)])
 
+
 def test_find_objects04():
     "find_objects 4"
     data = np.zeros([1], dtype=int)
     out = ndimage.find_objects(data)
     assert_equal(out, [])
 
+
 def test_find_objects05():
     "find_objects 5"
     data = np.ones([5], dtype=int)
     out = ndimage.find_objects(data)
     assert_equal(out, [(slice(0, 5, None),)])
 
+
 def test_find_objects06():
     "find_objects 6"
     data = np.array([1, 0, 2, 2, 0, 3])
@@ -325,6 +404,7 @@ def test_find_objects06():
                        (slice(2, 4, None),),
                        (slice(5, 6, None),)])
 
+
 def test_find_objects07():
     "find_objects 7"
     data = np.array([[0, 0, 0, 0, 0, 0],
@@ -336,6 +416,7 @@ def test_find_objects07():
     out = ndimage.find_objects(data)
     assert_equal(out, [])
 
+
 def test_find_objects08():
     "find_objects 8"
     data = np.array([[1, 0, 0, 0, 0, 0],
@@ -350,6 +431,7 @@ def test_find_objects08():
                        (slice(3, 5, None), slice(0, 2, None)),
                        (slice(5, 6, None), slice(3, 5, None))])
 
+
 def test_find_objects09():
     "find_objects 9"
     data = np.array([[1, 0, 0, 0, 0, 0],
@@ -364,6 +446,7 @@ def test_find_objects09():
                        None,
                        (slice(5, 6, None), slice(3, 5, None))])
 
+
 def test_sum01():
     "sum 1"
     for type in types:
@@ -371,6 +454,7 @@ def test_sum01():
         output = ndimage.sum(input)
         assert_equal(output, 0.0)
 
+
 def test_sum02():
     "sum 2"
     for type in types:
@@ -378,6 +462,7 @@ def test_sum02():
         output = ndimage.sum(input)
         assert_equal(output, 0.0)
 
+
 def test_sum03():
     "sum 3"
     for type in types:
@@ -385,6 +470,7 @@ def test_sum03():
         output = ndimage.sum(input)
         assert_almost_equal(output, 1.0)
 
+
 def test_sum04():
     "sum 4"
     for type in types:
@@ -392,6 +478,7 @@ def test_sum04():
         output = ndimage.sum(input)
         assert_almost_equal(output, 3.0)
 
+
 def test_sum05():
     "sum 5"
     for type in types:
@@ -399,6 +486,7 @@ def test_sum05():
         output = ndimage.sum(input)
         assert_almost_equal(output, 10.0)
 
+
 def test_sum06():
     "sum 6"
     labels = np.array([], bool)
@@ -407,6 +495,7 @@ def test_sum06():
         output = ndimage.sum(input, labels=labels)
         assert_equal(output, 0.0)
 
+
 def test_sum07():
     "sum 7"
     labels = np.ones([0, 4], bool)
@@ -415,6 +504,7 @@ def test_sum07():
         output = ndimage.sum(input, labels=labels)
         assert_equal(output, 0.0)
 
+
 def test_sum08():
     "sum 8"
     labels = np.array([1, 0], bool)
@@ -423,6 +513,7 @@ def test_sum08():
         output = ndimage.sum(input, labels=labels)
         assert_equal(output, 1.0)
 
+
 def test_sum09():
     "sum 9"
     labels = np.array([1, 0], bool)
@@ -431,6 +522,7 @@ def test_sum09():
         output = ndimage.sum(input, labels=labels)
         assert_almost_equal(output, 4.0)
 
+
 def test_sum10():
     "sum 10"
     labels = np.array([1, 0], bool)
@@ -438,6 +530,7 @@ def test_sum10():
     output = ndimage.sum(input, labels=labels)
     assert_almost_equal(output, 2.0)
 
+
 def test_sum11():
     "sum 11"
     labels = np.array([1, 2], np.int8)
@@ -447,6 +540,7 @@ def test_sum11():
                                        index=2)
         assert_almost_equal(output, 6.0)
 
+
 def test_sum12():
     "sum 12"
     labels = np.array([[1, 2], [2, 4]], np.int8)
@@ -456,6 +550,7 @@ def test_sum12():
                                         index=[4, 8, 2])
         assert_array_almost_equal(output, [4.0, 0.0, 5.0])
 
+
 def test_mean01():
     "mean 1"
     labels = np.array([1, 0], bool)
@@ -464,6 +559,7 @@ def test_mean01():
         output = ndimage.mean(input, labels=labels)
         assert_almost_equal(output, 2.0)
 
+
 def test_mean02():
     "mean 2"
     labels = np.array([1, 0], bool)
@@ -471,6 +567,7 @@ def test_mean02():
     output = ndimage.mean(input, labels=labels)
     assert_almost_equal(output, 1.0)
 
+
 def test_mean03():
     "mean 3"
     labels = np.array([1, 2])
@@ -480,6 +577,7 @@ def test_mean03():
                                         index=2)
         assert_almost_equal(output, 3.0)
 
+
 def test_mean04():
     "mean 4"
     labels = np.array([[1, 2], [2, 4]], np.int8)
@@ -494,6 +592,7 @@ def test_mean04():
     finally:
         np.seterr(**olderr)
 
+
 def test_minimum01():
     "minimum 1"
     labels = np.array([1, 0], bool)
@@ -502,6 +601,7 @@ def test_minimum01():
         output = ndimage.minimum(input, labels=labels)
         assert_almost_equal(output, 1.0)
 
+
 def test_minimum02():
     "minimum 2"
     labels = np.array([1, 0], bool)
@@ -509,6 +609,7 @@ def test_minimum02():
     output = ndimage.minimum(input, labels=labels)
     assert_almost_equal(output, 1.0)
 
+
 def test_minimum03():
     "minimum 3"
     labels = np.array([1, 2])
@@ -518,6 +619,7 @@ def test_minimum03():
                                            index=2)
         assert_almost_equal(output, 2.0)
 
+
 def test_minimum04():
     "minimum 4"
     labels = np.array([[1, 2], [2, 3]])
@@ -527,6 +629,7 @@ def test_minimum04():
                                            index=[2, 3, 8])
         assert_array_almost_equal(output, [2.0, 4.0, 0.0])
 
+
 def test_maximum01():
     "maximum 1"
     labels = np.array([1, 0], bool)
@@ -535,6 +638,7 @@ def test_maximum01():
         output = ndimage.maximum(input, labels=labels)
         assert_almost_equal(output, 3.0)
 
+
 def test_maximum02():
     "maximum 2"
     labels = np.array([1, 0], bool)
@@ -542,6 +646,7 @@ def test_maximum02():
     output = ndimage.maximum(input, labels=labels)
     assert_almost_equal(output, 1.0)
 
+
 def test_maximum03():
     "maximum 3"
     labels = np.array([1, 2])
@@ -551,6 +656,7 @@ def test_maximum03():
                                            index=2)
         assert_almost_equal(output, 4.0)
 
+
 def test_maximum04():
     "maximum 4"
     labels = np.array([[1, 2], [2, 3]])
@@ -560,11 +666,13 @@ def test_maximum04():
                                            index=[2, 3, 8])
         assert_array_almost_equal(output, [3.0, 4.0, 0.0])
 
+
 def test_maximum05():
     "Ticket #501"
     x = np.array([-3,-2,-1])
     assert_equal(ndimage.maximum(x),-1)
 
+
 def test_median01():
     "median 1"
     a = np.array([[1, 2, 0, 1],
@@ -578,6 +686,7 @@ def test_median01():
     output = ndimage.median(a, labels=labels, index=[1, 2, 3])
     assert_array_almost_equal(output, [2.5, 4.0, 6.0])
 
+
 def test_median02():
     "median 2"
     a = np.array([[1, 2, 0, 1],
@@ -587,6 +696,7 @@ def test_median02():
     output = ndimage.median(a)
     assert_almost_equal(output, 1.0)
 
+
 def test_median03():
     "median 3"
     a = np.array([[1, 2, 0, 1],
@@ -600,6 +710,7 @@ def test_median03():
     output = ndimage.median(a, labels=labels)
     assert_almost_equal(output, 3.0)
 
+
 def test_variance01():
     "variance 1"
     olderr = np.seterr(all='ignore')
@@ -611,6 +722,7 @@ def test_variance01():
     finally:
         np.seterr(**olderr)
 
+
 def test_variance02():
     "variance 2"
     for type in types:
@@ -618,6 +730,7 @@ def test_variance02():
         output = ndimage.variance(input)
         assert_almost_equal(output, 0.0)
 
+
 def test_variance03():
     "variance 3"
     for type in types:
@@ -625,12 +738,14 @@ def test_variance03():
         output = ndimage.variance(input)
         assert_almost_equal(output, 1.0)
 
+
 def test_variance04():
     "variance 4"
     input = np.array([1, 0], bool)
     output = ndimage.variance(input)
     assert_almost_equal(output, 0.25)
 
+
 def test_variance05():
     "variance 5"
     labels = [2, 2, 3]
@@ -639,6 +754,7 @@ def test_variance05():
         output = ndimage.variance(input, labels, 2)
         assert_almost_equal(output, 1.0)
 
+
 def test_variance06():
     "variance 6"
     labels = [2, 2, 3, 3, 4]
@@ -651,6 +767,7 @@ def test_variance06():
     finally:
         np.seterr(**olderr)
 
+
 def test_standard_deviation01():
     "standard deviation 1"
     olderr = np.seterr(all='ignore')
@@ -662,6 +779,7 @@ def test_standard_deviation01():
     finally:
         np.seterr(**olderr)
 
+
 def test_standard_deviation02():
     "standard deviation 2"
     for type in types:
@@ -669,6 +787,7 @@ def test_standard_deviation02():
         output = ndimage.standard_deviation(input)
         assert_almost_equal(output, 0.0)
 
+
 def test_standard_deviation03():
     "standard deviation 3"
     for type in types:
@@ -676,12 +795,14 @@ def test_standard_deviation03():
         output = ndimage.standard_deviation(input)
         assert_almost_equal(output, np.sqrt(1.0))
 
+
 def test_standard_deviation04():
     "standard deviation 4"
     input = np.array([1, 0], bool)
     output = ndimage.standard_deviation(input)
     assert_almost_equal(output, 0.5)
 
+
 def test_standard_deviation05():
     "standard deviation 5"
     labels = [2, 2, 3]
@@ -690,6 +811,7 @@ def test_standard_deviation05():
         output = ndimage.standard_deviation(input, labels, 2)
         assert_almost_equal(output, 1.0)
 
+
 def test_standard_deviation06():
     "standard deviation 6"
     labels = [2, 2, 3, 3, 4]
@@ -702,6 +824,7 @@ def test_standard_deviation06():
     finally:
         np.seterr(**olderr)
 
+
 def test_standard_deviation07():
     "standard deviation 7"
     labels = [1]
@@ -714,6 +837,7 @@ def test_standard_deviation07():
     finally:
         np.seterr(**olderr)
 
+
 def test_minimum_position01():
     "minimum position 1"
     labels = np.array([1, 0], bool)
@@ -722,6 +846,7 @@ def test_minimum_position01():
         output = ndimage.minimum_position(input, labels=labels)
         assert_equal(output, (0, 0))
 
+
 def test_minimum_position02():
     "minimum position 2"
     for type in types:
@@ -731,6 +856,7 @@ def test_minimum_position02():
         output = ndimage.minimum_position(input)
         assert_equal(output, (1, 2))
 
+
 def test_minimum_position03():
     "minimum position 3"
     input = np.array([[5, 4, 2, 5],
@@ -739,6 +865,7 @@ def test_minimum_position03():
     output = ndimage.minimum_position(input)
     assert_equal(output, (1, 2))
 
+
 def test_minimum_position04():
     "minimum position 4"
     input = np.array([[5, 4, 2, 5],
@@ -747,6 +874,7 @@ def test_minimum_position04():
     output = ndimage.minimum_position(input)
     assert_equal(output, (0, 0))
 
+
 def test_minimum_position05():
     "minimum position 5"
     labels = [1, 2, 0, 4]
@@ -757,6 +885,7 @@ def test_minimum_position05():
         output = ndimage.minimum_position(input, labels)
         assert_equal(output, (2, 0))
 
+
 def test_minimum_position06():
     "minimum position 6"
     labels = [1, 2, 3, 4]
@@ -767,6 +896,7 @@ def test_minimum_position06():
         output = ndimage.minimum_position(input, labels, 2)
         assert_equal(output, (0, 1))
 
+
 def test_minimum_position07():
     "minimum position 7"
     labels = [1, 2, 3, 4]
@@ -779,6 +909,7 @@ def test_minimum_position07():
         assert_equal(output[0], (0, 1))
         assert_equal(output[1], (1, 2))
 
+
 def test_maximum_position01():
     "maximum position 1"
     labels = np.array([1, 0], bool)
@@ -788,6 +919,7 @@ def test_maximum_position01():
                                                     labels=labels)
         assert_equal(output, (1, 0))
 
+
 def test_maximum_position02():
     "maximum position 2"
     for type in types:
@@ -797,6 +929,7 @@ def test_maximum_position02():
         output = ndimage.maximum_position(input)
         assert_equal(output, (1, 2))
 
+
 def test_maximum_position03():
     "maximum position 3"
     input = np.array([[5, 4, 2, 5],
@@ -805,6 +938,7 @@ def test_maximum_position03():
     output = ndimage.maximum_position(input)
     assert_equal(output, (0, 0))
 
+
 def test_maximum_position04():
     "maximum position 4"
     labels = [1, 2, 0, 4]
@@ -815,6 +949,7 @@ def test_maximum_position04():
         output = ndimage.maximum_position(input, labels)
         assert_equal(output, (1, 1))
 
+
 def test_maximum_position05():
     "maximum position 5"
     labels = [1, 2, 0, 4]
@@ -825,6 +960,7 @@ def test_maximum_position05():
         output = ndimage.maximum_position(input, labels, 1)
         assert_equal(output, (0, 0))
 
+
 def test_maximum_position06():
     "maximum position 6"
     labels = [1, 2, 0, 4]
@@ -837,6 +973,7 @@ def test_maximum_position06():
         assert_equal(output[0], (0, 0))
         assert_equal(output[1], (1, 1))
 
+
 def test_maximum_position07():
     "maximum position 7 - float labels"
     labels = np.array([1.0, 2.5, 0.0, 4.5])
@@ -849,6 +986,7 @@ def test_maximum_position07():
         assert_equal(output[0], (0, 0))
         assert_equal(output[1], (0, 3))
 
+
 def test_extrema01():
     "extrema 1"
     labels = np.array([1, 0], bool)
@@ -863,6 +1001,7 @@ def test_extrema01():
                                                      labels=labels)
         assert_equal(output1, (output2, output3, output4, output5))
 
+
 def test_extrema02():
     "extrema 2"
     labels = np.array([1, 2])
@@ -880,6 +1019,7 @@ def test_extrema02():
                                             labels=labels, index=2)
         assert_equal(output1, (output2, output3, output4, output5))
 
+
 def test_extrema03():
     "extrema 3"
     labels = np.array([[1, 2], [2, 3]])
@@ -900,6 +1040,7 @@ def test_extrema03():
         assert_array_almost_equal(output1[2], output4)
         assert_array_almost_equal(output1[3], output5)
 
+
 def test_extrema04():
     "extrema 4"
     labels = [1, 2, 0, 4]
@@ -919,6 +1060,7 @@ def test_extrema04():
         assert_array_almost_equal(output1[2], output4)
         assert_array_almost_equal(output1[3], output5)
 
+
 def test_center_of_mass01():
     "center of mass 1"
     expected = [0.0, 0.0]
@@ -927,6 +1069,7 @@ def test_center_of_mass01():
         output = ndimage.center_of_mass(input)
         assert_array_almost_equal(output, expected)
 
+
 def test_center_of_mass02():
     "center of mass 2"
     expected = [1, 0]
@@ -935,6 +1078,7 @@ def test_center_of_mass02():
         output = ndimage.center_of_mass(input)
         assert_array_almost_equal(output, expected)
 
+
 def test_center_of_mass03():
     "center of mass 3"
     expected = [0, 1]
@@ -943,6 +1087,7 @@ def test_center_of_mass03():
         output = ndimage.center_of_mass(input)
         assert_array_almost_equal(output, expected)
 
+
 def test_center_of_mass04():
     "center of mass 4"
     expected = [1, 1]
@@ -951,6 +1096,7 @@ def test_center_of_mass04():
         output = ndimage.center_of_mass(input)
         assert_array_almost_equal(output, expected)
 
+
 def test_center_of_mass05():
     "center of mass 5"
     expected = [0.5, 0.5]
@@ -959,6 +1105,7 @@ def test_center_of_mass05():
         output = ndimage.center_of_mass(input)
         assert_array_almost_equal(output, expected)
 
+
 def test_center_of_mass06():
     "center of mass 6"
     expected = [0.5, 0.5]
@@ -966,6 +1113,7 @@ def test_center_of_mass06():
     output = ndimage.center_of_mass(input)
     assert_array_almost_equal(output, expected)
 
+
 def test_center_of_mass07():
     "center of mass 7"
     labels = [1, 0]
@@ -974,6 +1122,7 @@ def test_center_of_mass07():
     output = ndimage.center_of_mass(input, labels)
     assert_array_almost_equal(output, expected)
 
+
 def test_center_of_mass08():
     "center of mass 8"
     labels = [1, 2]
@@ -982,6 +1131,7 @@ def test_center_of_mass08():
     output = ndimage.center_of_mass(input, labels, 2)
     assert_array_almost_equal(output, expected)
 
+
 def test_center_of_mass09():
     "center of mass 9"
     labels = [1, 2]
@@ -990,6 +1140,7 @@ def test_center_of_mass09():
     output = ndimage.center_of_mass(input, labels, [1, 2])
     assert_array_almost_equal(output, expected)
 
+
 def test_histogram01():
     "histogram 1"
     expected = np.ones(10)
@@ -997,6 +1148,7 @@ def test_histogram01():
     output = ndimage.histogram(input, 0, 10, 10)
     assert_array_almost_equal(output, expected)
 
+
 def test_histogram02():
     "histogram 2"
     labels = [1, 1, 1, 1, 2, 2, 2, 2]
@@ -1005,6 +1157,7 @@ def test_histogram02():
     output = ndimage.histogram(input, 0, 4, 5, labels, 1)
     assert_array_almost_equal(output, expected)
 
+
 def test_histogram03():
     "histogram 3"
     labels = [1, 0, 1, 1, 2, 2, 2, 2]
@@ -1019,10 +1172,10 @@ def test_histogram03():
 
 def test_stat_funcs_2d():
     """Apply the stat funcs to a 2-d array."""
-    a   = np.array([[5,6,0,0,0], [8,9,0,0,0], [0,0,0,3,5]])
+    a = np.array([[5,6,0,0,0], [8,9,0,0,0], [0,0,0,3,5]])
     lbl = np.array([[1,1,0,0,0], [1,1,0,0,0], [0,0,0,2,2]])
 
-    mean= ndimage.mean(a, labels=lbl, index=[1, 2])
+    mean = ndimage.mean(a, labels=lbl, index=[1, 2])
     assert_array_equal(mean, [7.0, 4.0])
 
     var = ndimage.variance(a, labels=lbl, index=[1, 2])
