@@ -1,15 +1,20 @@
 """Dictionary Of Keys based matrix"""
 
+from __future__ import division, print_function, absolute_import
+
 __docformat__ = "restructuredtext en"
 
 __all__ = ['dok_matrix', 'isspmatrix_dok']
 
-from itertools import izip
 
 import numpy as np
 
-from base import spmatrix, isspmatrix
-from sputils import isdense, getdtype, isshape, isintlike, isscalarlike, upcast
+from scipy.lib.six.moves import zip as izip, xrange
+from scipy.lib.six import iteritems
+
+from .base import spmatrix, isspmatrix
+from .sputils import (isdense, getdtype, isshape, isintlike, isscalarlike,
+                      upcast, upcast_scalar)
 
 try:
     from operator import isSequenceType as _is_sequence
@@ -18,6 +23,7 @@ except ImportError:
         return (hasattr(x, '__len__') or hasattr(x, '__next__')
                 or hasattr(x, 'next'))
 
+
 class dok_matrix(spmatrix, dict):
     """
     Dictionary Of Keys based sparse matrix.
@@ -73,10 +79,10 @@ class dok_matrix(spmatrix, dict):
         spmatrix.__init__(self)
 
         self.dtype = getdtype(dtype, default=float)
-        if isinstance(arg1, tuple) and isshape(arg1): # (M,N)
+        if isinstance(arg1, tuple) and isshape(arg1):  # (M,N)
             M, N = arg1
             self.shape = (M, N)
-        elif isspmatrix(arg1): # Sparse ctor
+        elif isspmatrix(arg1):  # Sparse ctor
             if isspmatrix_dok(arg1) and copy:
                 arg1 = arg1.copy()
             else:
@@ -88,19 +94,20 @@ class dok_matrix(spmatrix, dict):
             self.update(arg1)
             self.shape = arg1.shape
             self.dtype = arg1.dtype
-        else: # Dense ctor
+        else:  # Dense ctor
             try:
                 arg1 = np.asarray(arg1)
             except:
                 raise TypeError('invalid input format')
 
-            if len(arg1.shape)!=2:
+            if len(arg1.shape) != 2:
                 raise TypeError('expected rank <=2 dense array or matrix')
 
-            from coo import coo_matrix
-            self.update( coo_matrix(arg1, dtype=dtype).todok() )
+            from .coo import coo_matrix
+            d = coo_matrix(arg1, dtype=dtype).todok()
+            self.update(d)
             self.shape = arg1.shape
-            self.dtype = arg1.dtype
+            self.dtype = d.dtype
 
     def getnnz(self):
         return dict.__len__(self)
@@ -122,7 +129,7 @@ class dok_matrix(spmatrix, dict):
             raise IndexError('index out of bounds')
         return dict.get(self, key, default)
 
-    def  __getitem__(self, key):
+    def __getitem__(self, key):
         """If key=(i,j) is a pair of integers, return the corresponding
         element.  If either i or j is a slice or sequence, return a new sparse
         matrix with just these elements.
@@ -132,7 +139,6 @@ class dok_matrix(spmatrix, dict):
         except (ValueError, TypeError):
             raise TypeError('index must be a pair of integers or slices')
 
-
         # Bounds checking
         if isintlike(i):
             if i < 0:
@@ -181,7 +187,7 @@ class dok_matrix(spmatrix, dict):
                     # ** linear time in the number of non-zeros:
                     for (ii, jj) in self.keys():
                         if jj == j and ii >= first and ii <= last:
-                            dict.__setitem__(new, (ii-first, 0), \
+                            dict.__setitem__(new, (ii-first, 0),
                                              dict.__getitem__(self, (ii,jj)))
                 else:
                     ###################################
@@ -216,11 +222,10 @@ class dok_matrix(spmatrix, dict):
             # ** if there are many non-zeros
             for (ii, jj) in self.keys():
                 if ii == i and jj >= first and jj <= last:
-                    dict.__setitem__(new, (0, jj-first), \
+                    dict.__setitem__(new, (0, jj-first),
                                      dict.__getitem__(self, (ii,jj)))
             return new
 
-
     def __setitem__(self, key, value):
         try:
             i, j = key
@@ -239,7 +244,7 @@ class dok_matrix(spmatrix, dict):
 
             if np.isscalar(value):
                 if value == 0:
-                    if self.has_key((i,j)):
+                    if (i,j) in self:
                         del self[(i,j)]
                 else:
                     dict.__setitem__(self, (i,j), self.dtype.type(value))
@@ -324,7 +329,6 @@ class dok_matrix(spmatrix, dict):
                         for element, val in izip(seq, value):
                             self[i, element] = val
 
-
     def __add__(self, other):
         # First check if argument is a scalar
         if isscalarlike(other):
@@ -336,7 +340,7 @@ class dok_matrix(spmatrix, dict):
                     aij = self.get((i, j), 0) + other
                     if aij != 0:
                         new[i, j] = aij
-            #new.dtype.char = self.dtype.char
+            # new.dtype.char = self.dtype.char
         elif isinstance(other, dok_matrix):
             if other.shape != self.shape:
                 raise ValueError("matrix dimensions are not equal")
@@ -389,55 +393,54 @@ class dok_matrix(spmatrix, dict):
         return new
 
     def _mul_scalar(self, other):
+        res_dtype = upcast_scalar(self.dtype, other)
         # Multiply this scalar by every element.
-        new = dok_matrix(self.shape, dtype=self.dtype)
-        for (key, val) in self.iteritems():
+        new = dok_matrix(self.shape, dtype=res_dtype)
+        for (key, val) in iteritems(self):
             new[key] = val * other
         return new
 
     def _mul_vector(self, other):
-        #matrix * vector
-        result = np.zeros( self.shape[0], dtype=upcast(self.dtype,other.dtype) )
-        for (i,j),v in self.iteritems():
+        # matrix * vector
+        result = np.zeros(self.shape[0], dtype=upcast(self.dtype,other.dtype))
+        for (i,j),v in iteritems(self):
             result[i] += v * other[j]
         return result
 
     def _mul_multivector(self, other):
-        #matrix * multivector
+        # matrix * multivector
         M,N = self.shape
-        n_vecs = other.shape[1] #number of column vectors
-        result = np.zeros( (M,n_vecs), dtype=upcast(self.dtype,other.dtype) )
-        for (i,j),v in self.iteritems():
+        n_vecs = other.shape[1]  # number of column vectors
+        result = np.zeros((M,n_vecs), dtype=upcast(self.dtype,other.dtype))
+        for (i,j),v in iteritems(self):
             result[i,:] += v * other[j,:]
         return result
 
     def __imul__(self, other):
         if isscalarlike(other):
             # Multiply this scalar by every element.
-            for (key, val) in self.iteritems():
+            for (key, val) in iteritems(self):
                 self[key] = val * other
-            #new.dtype.char = self.dtype.char
+            # new.dtype.char = self.dtype.char
             return self
         else:
             return NotImplementedError
 
-
     def __truediv__(self, other):
         if isscalarlike(other):
             new = dok_matrix(self.shape, dtype=self.dtype)
             # Multiply this scalar by every element.
-            for (key, val) in self.iteritems():
+            for (key, val) in iteritems(self):
                 new[key] = val / other
-            #new.dtype.char = self.dtype.char
+            # new.dtype.char = self.dtype.char
             return new
         else:
             return self.tocsr() / other
 
-
     def __itruediv__(self, other):
         if isscalarlike(other):
             # Multiply this scalar by every element.
-            for (key, val) in self.iteritems():
+            for (key, val) in iteritems(self):
                 self[key] = val / other
             return self
         else:
@@ -452,7 +455,7 @@ class dok_matrix(spmatrix, dict):
         """
         M, N = self.shape
         new = dok_matrix((N, M), dtype=self.dtype)
-        for key, value in self.iteritems():
+        for key, value in iteritems(self):
             new[key[1], key[0]] = value
         return new
 
@@ -461,7 +464,7 @@ class dok_matrix(spmatrix, dict):
         """
         M, N = self.shape
         new = dok_matrix((N, M), dtype=self.dtype)
-        for key, value in self.iteritems():
+        for key, value in iteritems(self):
             new[key[1], key[0]] = np.conj(value)
         return new
 
@@ -470,61 +473,34 @@ class dok_matrix(spmatrix, dict):
         new.update(self)
         return new
 
-    def take(self, cols_or_rows, columns=1):
-        # Extract columns or rows as indictated from matrix
-        # assume cols_or_rows is sorted
-        new = dok_matrix(dtype=self.dtype)    # what should the dimensions be ?!
-        indx = int((columns == 1))
-        N = len(cols_or_rows)
-        if indx: # columns
-            for key in self.keys():
-                num = np.searchsorted(cols_or_rows, key[1])
-                if num < N:
-                    newkey = (key[0], num)
-                    new[newkey] = self[key]
-        else:
-            for key in self.keys():
-                num = np.searchsorted(cols_or_rows, key[0])
-                if num < N:
-                    newkey = (num, key[1])
-                    new[newkey] = self[key]
-        return new
+    def getrow(self, i):
+        """Returns a copy of row i of the matrix as a (1 x n)
+        DOK matrix.
+        """
+        out = self.__class__((1, self.shape[1]), dtype=self.dtype)
+        for j in range(self.shape[1]):
+            out[0, j] = self[i, j]
+        return out
 
-    def split(self, cols_or_rows, columns=1):
-        # Similar to take but returns two arrays, the extracted columns plus
-        # the resulting array.  Assumes cols_or_rows is sorted
-        base = dok_matrix()
-        ext = dok_matrix()
-        indx = int((columns == 1))
-        if indx:
-            for key in self.keys():
-                num = np.searchsorted(cols_or_rows, key[1])
-                if cols_or_rows[num] == key[1]:
-                    newkey = (key[0], num)
-                    ext[newkey] = self[key]
-                else:
-                    newkey = (key[0], key[1]-num)
-                    base[newkey] = self[key]
-        else:
-            for key in self.keys():
-                num = np.searchsorted(cols_or_rows, key[0])
-                if cols_or_rows[num] == key[0]:
-                    newkey = (num, key[1])
-                    ext[newkey] = self[key]
-                else:
-                    newkey = (key[0]-num, key[1])
-                    base[newkey] = self[key]
-        return base, ext
+    def getcol(self, j):
+        """Returns a copy of column j of the matrix as a (m x 1)
+        DOK matrix.
+        """
+        out = self.__class__((self.shape[0], 1), dtype=self.dtype)
+        for i in range(self.shape[0]):
+            out[i, 0] = self[i, j]
+        return out
 
     def tocoo(self):
         """ Return a copy of this matrix in COOrdinate format"""
-        from coo import coo_matrix
+        from .coo import coo_matrix
         if self.nnz == 0:
             return coo_matrix(self.shape, dtype=self.dtype)
         else:
-            data    = np.asarray(self.values(), dtype=self.dtype)
-            indices = np.asarray(self.keys(), dtype=np.intc).T
-            return coo_matrix((data,indices), shape=self.shape, dtype=self.dtype)
+            data = np.asarray(_list(self.values()), dtype=self.dtype)
+            indices = np.asarray(_list(self.keys()), dtype=np.intc).T
+            return coo_matrix((data,indices), shape=self.shape,
+                              dtype=self.dtype)
 
     def todok(self,copy=False):
         if copy:
@@ -540,8 +516,9 @@ class dok_matrix(spmatrix, dict):
         """ Return a copy of this matrix in Compressed Sparse Column format"""
         return self.tocoo().tocsc()
 
-    def toarray(self):
-        return self.tocoo().toarray()
+    def toarray(self, order=None, out=None):
+        """See the docstring for `spmatrix.toarray`."""
+        return self.tocoo().toarray(order=order, out=out)
 
     def resize(self, shape):
         """ Resize the matrix in-place to dimensions given by 'shape'.
@@ -555,14 +532,18 @@ class dok_matrix(spmatrix, dict):
         M, N = self.shape
         if newM < M or newN < N:
             # Remove all elements outside new dimensions
-            for (i, j) in self.keys():
+            for (i, j) in list(self.keys()):
                 if i >= newM or j >= newN:
                     del self[i, j]
         self._shape = shape
 
 
+def _list(x):
+    """Force x to a list."""
+    if not isinstance(x, list):
+        x = list(x)
+    return x
 
-from sputils import _isinstance
 
 def isspmatrix_dok(x):
-    return _isinstance(x, dok_matrix)
+    return isinstance(x, dok_matrix)
