@@ -92,7 +92,7 @@ cgsequ(SuperMatrix *A, float *r, float *c, float *rowcnd,
     int i, j, irow;
     float rcmin, rcmax;
     float bignum, smlnum;
-    extern double slamch_(char *);
+    extern float slamch_(char *);
     
     /* Test the input parameters. */
     *info = 0;
@@ -127,7 +127,7 @@ cgsequ(SuperMatrix *A, float *r, float *c, float *rowcnd,
     for (j = 0; j < A->ncol; ++j)
 	for (i = Astore->colptr[j]; i < Astore->colptr[j+1]; ++i) {
 	    irow = Astore->rowind[i];
-	    r[irow] = SUPERLU_MAX( r[irow], slu_c_abs1(&Aval[i]) );
+	    r[irow] = SUPERLU_MAX( r[irow], c_abs1(&Aval[i]) );
 	}
 
     /* Find the maximum and minimum scale factors. */
@@ -162,7 +162,7 @@ cgsequ(SuperMatrix *A, float *r, float *c, float *rowcnd,
     for (j = 0; j < A->ncol; ++j)
 	for (i = Astore->colptr[j]; i < Astore->colptr[j+1]; ++i) {
 	    irow = Astore->rowind[i];
-	    c[j] = SUPERLU_MAX( c[j], slu_c_abs1(&Aval[i]) * r[irow] );
+	    c[j] = SUPERLU_MAX( c[j], c_abs1(&Aval[i]) * r[irow] );
 	}
 
     /* Find the maximum and minimum scale factors. */
