@@ -40,7 +40,7 @@ trans    'N': solve A   * x == b\n\
 static PyObject *
 SciPyLU_solve(SciPyLUObject *self, PyObject *args, PyObject *kwds) {
   PyArrayObject *b, *x=NULL;
-  SuperMatrix B;
+  SuperMatrix B = {0};
 #ifndef NPY_PY3K
   char itrans = 'N';
 #else
@@ -48,7 +48,7 @@ SciPyLU_solve(SciPyLUObject *self, PyObject *args, PyObject *kwds) {
 #endif
   int info;
   trans_t trans;
-  SuperLUStat_t stat;
+  SuperLUStat_t stat = {0};
 
   static char *kwlist[] = {"rhs","trans",NULL};
 
@@ -108,8 +108,8 @@ SciPyLU_solve(SciPyLUObject *self, PyObject *args, PyObject *kwds) {
   return (PyObject *)x;
 
 fail:
-  Destroy_SuperMatrix_Store(&B);  
-  StatFree(&stat);
+  XDestroy_SuperMatrix_Store(&B);
+  XStatFree(&stat);
   Py_XDECREF(x);
   return NULL;
 }
@@ -131,12 +131,10 @@ SciPyLU_dealloc(SciPyLUObject *self)
 {
   SUPERLU_FREE(self->perm_r);
   SUPERLU_FREE(self->perm_c);
-  if (self->L.Store != NULL) {
-      Destroy_SuperNode_Matrix(&self->L);
-  }
-  if (self->U.Store != NULL) {
-      Destroy_CompCol_Matrix(&self->U);
-  }
+  self->perm_r = NULL;
+  self->perm_c = NULL;
+  XDestroy_SuperNode_Matrix(&self->L);
+  XDestroy_CompCol_Matrix(&self->U);
   PyObject_Del(self);
 }
 
@@ -382,15 +380,14 @@ newSciPyLUObject(SuperMatrix *A, PyObject *option_dict, int intype, int ilu)
 
    /* A must be in SLU_NC format used by the factorization routine. */
   SciPyLUObject *self;
-  SuperMatrix AC;     /* Matrix postmultiplied by Pc */
+  SuperMatrix AC = {0};     /* Matrix postmultiplied by Pc */
   int lwork = 0;
   int *etree=NULL;
   int info;
   int n;
   superlu_options_t options;
-  SuperLUStat_t stat;
+  SuperLUStat_t stat = {0};
   int panel_size, relax;
-  int trf_finished = 0;
 
   n = A->ncol;
 
@@ -407,6 +404,8 @@ newSciPyLUObject(SuperMatrix *A, PyObject *option_dict, int intype, int ilu)
   self->n = n;
   self->perm_r = NULL;
   self->perm_c = NULL;
+  self->L.Store = NULL;
+  self->U.Store = NULL;
   self->type = intype;
 
   if (setjmp(_superlu_py_jmpbuf)) goto fail;
@@ -438,7 +437,6 @@ newSciPyLUObject(SuperMatrix *A, PyObject *option_dict, int intype, int ilu)
             etree, NULL, lwork, self->perm_c, self->perm_r,
             &self->L, &self->U, &stat, &info);
   }
-  trf_finished = 1;
 
   if (info) {
     if (info < 0)
@@ -461,16 +459,10 @@ newSciPyLUObject(SuperMatrix *A, PyObject *option_dict, int intype, int ilu)
   return (PyObject *)self;
 
 fail:
-  if (!trf_finished) {
-      /* Avoid trying to free partially initialized matrices;
-         might leak some memory, but avoids a crash */
-      self->L.Store = NULL;
-      self->U.Store = NULL;
-  }
   SUPERLU_FREE(etree);
-  Destroy_CompCol_Permuted(&AC);
-  StatFree(&stat);
-  SciPyLU_dealloc(self);
+  XDestroy_CompCol_Permuted(&AC);
+  XStatFree(&stat);
+  Py_DECREF(self);
   return NULL;
 }
 
@@ -501,13 +493,15 @@ fail:
     PyErr_SetString(PyExc_ValueError, message); \
     return 0;
 
-#define ENUM_CHECK(name)                                \
-    if (my_strxcmp(s, #name) == 0 || i == (long)name) { \
+#define ENUM_CHECK_NAME(name, sname)                    \
+    if (my_strxcmp(s, sname) == 0 || i == (long)name) { \
         *value = name;                                  \
         Py_XDECREF(tmpobj);                             \
         return 1;                                       \
     }
 
+#define ENUM_CHECK(name) ENUM_CHECK_NAME(name, #name)
+
 /*
  * Compare strings ignoring case, underscores and whitespace
  */
@@ -589,9 +583,12 @@ static int iterrefine_cvt(PyObject *input, IterRefine_t *value)
 {
     ENUM_CHECK_INIT;
     ENUM_CHECK(NOREFINE);
-    ENUM_CHECK(SINGLE);
-    ENUM_CHECK(DOUBLE);
-    ENUM_CHECK(EXTRA);
+    ENUM_CHECK(SLU_SINGLE);
+    ENUM_CHECK_NAME(SLU_SINGLE, "SINGLE");
+    ENUM_CHECK(SLU_DOUBLE);
+    ENUM_CHECK_NAME(SLU_DOUBLE, "DOUBLE");
+    ENUM_CHECK(SLU_EXTRA);
+    ENUM_CHECK_NAME(SLU_EXTRA, "EXTRA");
     ENUM_CHECK_FINISH("invalid value for 'IterRefine' parameter");
 }
 
