@@ -5,21 +5,30 @@ Interface to the UMFPACK library.
 Author: Robert Cimrman
 """
 
+from __future__ import division, print_function, absolute_import
+
+import re
+import warnings
+
+from scipy.lib.six import iteritems
 
-#from base import Struct, pause
 import numpy as np
 import scipy.sparse as sp
-import re
-try: # Silence import error.
-    import _umfpack as _um
+try:  # Silence import error.
+    from . import _umfpack as _um
 except:
     _um = None
 
 assumeSortedIndices = False
 
+
+class UmfpackWarning(UserWarning):
+    pass
+
+
 ##
 # 10.01.2006, c
-def configure( **kwargs ):
+def configure(**kwargs):
     """
     Valid keyword arguments with defaults (other ignored):
       assumeSortedIndices = False
@@ -34,13 +43,13 @@ def configure( **kwargs ):
 
 ##
 # 30.11.2005, c
-def updateDictWithVars( adict, module, pattern, group = None ):
-    match = re.compile( pattern ).match
+def updateDictWithVars(adict, module, pattern, group=None):
+    match = re.compile(pattern).match
 
-    for name in [ii for ii in vars( module ).keys()
-                 if match( ii )]:
+    for name in [ii for ii in vars(module)
+                 if match(ii)]:
         if group is not None:
-            outName = match( name ).group( group )
+            outName = match(name).group(group)
         else:
             outName = name
 
@@ -176,26 +185,25 @@ umfInfo = [
 if _um:
     ##
     # Export UMFPACK constants from _um.
-    umfDefines = updateDictWithVars( {}, _um, 'UMFPACK_.*' )
-    locals().update( umfDefines )
-
+    umfDefines = updateDictWithVars({}, _um, 'UMFPACK_.*')
+    locals().update(umfDefines)
 
     umfStatus = {
-        UMFPACK_OK : 'UMFPACK_OK',
-        UMFPACK_WARNING_singular_matrix : 'UMFPACK_WARNING_singular_matrix',
-        UMFPACK_WARNING_determinant_underflow : 'UMFPACK_WARNING_determinant_underflow',
-        UMFPACK_WARNING_determinant_overflow : 'UMFPACK_WARNING_determinant_overflow',
-        UMFPACK_ERROR_out_of_memory : 'UMFPACK_ERROR_out_of_memory',
-        UMFPACK_ERROR_invalid_Numeric_object : 'UMFPACK_ERROR_invalid_Numeric_object',
-        UMFPACK_ERROR_invalid_Symbolic_object : 'UMFPACK_ERROR_invalid_Symbolic_object',
-        UMFPACK_ERROR_argument_missing : 'UMFPACK_ERROR_argument_missing',
-        UMFPACK_ERROR_n_nonpositive : 'UMFPACK_ERROR_n_nonpositive',
-        UMFPACK_ERROR_invalid_matrix : 'UMFPACK_ERROR_invalid_matrix',
-        UMFPACK_ERROR_different_pattern : 'UMFPACK_ERROR_different_pattern',
-        UMFPACK_ERROR_invalid_system : 'UMFPACK_ERROR_invalid_system',
-        UMFPACK_ERROR_invalid_permutation : 'UMFPACK_ERROR_invalid_permutation',
-        UMFPACK_ERROR_internal_error : 'UMFPACK_ERROR_internal_error',
-        UMFPACK_ERROR_file_IO : 'UMFPACK_ERROR_file_IO',
+        UMFPACK_OK: 'UMFPACK_OK',
+        UMFPACK_WARNING_singular_matrix: 'UMFPACK_WARNING_singular_matrix',
+        UMFPACK_WARNING_determinant_underflow: 'UMFPACK_WARNING_determinant_underflow',
+        UMFPACK_WARNING_determinant_overflow: 'UMFPACK_WARNING_determinant_overflow',
+        UMFPACK_ERROR_out_of_memory: 'UMFPACK_ERROR_out_of_memory',
+        UMFPACK_ERROR_invalid_Numeric_object: 'UMFPACK_ERROR_invalid_Numeric_object',
+        UMFPACK_ERROR_invalid_Symbolic_object: 'UMFPACK_ERROR_invalid_Symbolic_object',
+        UMFPACK_ERROR_argument_missing: 'UMFPACK_ERROR_argument_missing',
+        UMFPACK_ERROR_n_nonpositive: 'UMFPACK_ERROR_n_nonpositive',
+        UMFPACK_ERROR_invalid_matrix: 'UMFPACK_ERROR_invalid_matrix',
+        UMFPACK_ERROR_different_pattern: 'UMFPACK_ERROR_different_pattern',
+        UMFPACK_ERROR_invalid_system: 'UMFPACK_ERROR_invalid_system',
+        UMFPACK_ERROR_invalid_permutation: 'UMFPACK_ERROR_invalid_permutation',
+        UMFPACK_ERROR_internal_error: 'UMFPACK_ERROR_internal_error',
+        UMFPACK_ERROR_file_IO: 'UMFPACK_ERROR_file_IO',
     }
 
     umfSys = [
@@ -217,48 +225,52 @@ if _um:
 
     # Real, complex.
     umfSys_transposeMap = [
-        {UMFPACK_A : UMFPACK_At,
-         UMFPACK_At : UMFPACK_A,
-         UMFPACK_Aat : UMFPACK_A},
-        {UMFPACK_A : UMFPACK_Aat,
-         UMFPACK_Aat : UMFPACK_A}
+        {UMFPACK_A: UMFPACK_At,
+         UMFPACK_At: UMFPACK_A,
+         UMFPACK_Aat: UMFPACK_A},
+        {UMFPACK_A: UMFPACK_Aat,
+         UMFPACK_Aat: UMFPACK_A}
     ]
 
-umfFamilyTypes = {'di' : int, 'dl' : long, 'zi' : int, 'zl' : long}
+umfFamilyTypes = {'di': int, 'dl': int, 'zi': int, 'zl': int}
 umfRealTypes = ('di', 'dl')
 umfComplexTypes = ('zi', 'zl')
 
 ##
 # 02.01.2005
-class Struct( object ):
+
+
+class Struct(object):
     # 03.10.2005, c
     # 26.10.2005
-    def __init__( self, **kwargs ):
+    def __init__(self, **kwargs):
         if kwargs:
-            self.__dict__.update( kwargs )
+            self.__dict__.update(kwargs)
 
     # 08.03.2005
-    def __str__( self ):
+    def __str__(self):
         ss = "%s\n" % self.__class__
-        for key, val in self.__dict__.iteritems():
-            if (issubclass( self.__dict__[key].__class__, Struct )):
+        for key, val in iteritems(self.__dict__):
+            if (issubclass(self.__dict__[key].__class__, Struct)):
                 ss += "  %s:\n    %s\n" % (key, self.__dict__[key].__class__)
             else:
-                aux = "\n" + str( val )
-                aux = aux.replace( "\n", "\n    " );
+                aux = "\n" + str(val)
+                aux = aux.replace("\n", "\n    ")
                 ss += "  %s:\n%s\n" % (key, aux[1:])
-        return( ss.rstrip() )
+        return(ss.rstrip())
 
 ##
 # 30.11.2005, c
-class UmfpackContext( Struct ):
+
+
+class UmfpackContext(Struct):
 
     ##
     # 30.11.2005, c
     # 01.12.2005
     # 21.12.2005
     # 01.03.2006
-    def __init__( self, family = 'di', **kwargs ):
+    def __init__(self, family='di', **kwargs):
         """
         Arguments:
 
@@ -267,21 +279,21 @@ class UmfpackContext( Struct ):
         Keyword arguments:
 
         maxCond .. if extimated condition number is greater than maxCond,
-                   a warning is printed (default: 1e12)"""
+                   a warning is issued (default: 1e12)"""
         if _um is None:
-            raise ImportError('Scipy was built without UMFPACK support. ' 
-                              'You need to install the UMFPACK library and ' 
+            raise ImportError('Scipy was built without UMFPACK support. '
+                              'You need to install the UMFPACK library and '
                               'header files before building scipy.')
 
         self.maxCond = 1e12
-        Struct.__init__( self, **kwargs )
+        Struct.__init__(self, **kwargs)
 
-        if family not in umfFamilyTypes.keys():
+        if family not in umfFamilyTypes:
             raise TypeError('wrong family: %s' % family)
 
         self.family = family
-        self.control = np.zeros( (UMFPACK_CONTROL, ), dtype = np.double )
-        self.info = np.zeros( (UMFPACK_INFO, ), dtype = np.double )
+        self.control = np.zeros((UMFPACK_CONTROL,), dtype=np.double)
+        self.info = np.zeros((UMFPACK_INFO,), dtype=np.double)
         self._symbolic = None
         self._numeric = None
         self.mtx = None
@@ -290,41 +302,45 @@ class UmfpackContext( Struct ):
         ##
         # Functions corresponding to <family> are stored in self.funs.
         pattern = 'umfpack_' + family + '_(.*)'
-        fn = updateDictWithVars( {}, _um, pattern, group = 1 )
-        self.funs = Struct( **fn )
+        fn = updateDictWithVars({}, _um, pattern, group=1)
+        self.funs = Struct(**fn)
 
-        self.funs.defaults( self.control )
+        self.funs.defaults(self.control)
         self.control[UMFPACK_PRL] = 3
 
+    def __del__(self):
+        if _um is not None:
+            self.free()
+
     ##
     # 30.11.2005, c
-    def strControl( self ):
-        maxLen = max( [len( name ) for name in umfControls] )
+    def strControl(self):
+        maxLen = max([len(name) for name in umfControls])
         format = '%%-%ds : %%d' % maxLen
         aux = [format % (name, self.control[umfDefines[name]])
                for name in umfControls if name in umfDefines]
-        return '\n'.join( aux )
+        return '\n'.join(aux)
 
     ##
     # 01.12.2005, c
-    def strInfo( self ):
-        maxLen = max( [len( name ) for name in umfInfo] )
+    def strInfo(self):
+        maxLen = max([len(name) for name in umfInfo])
         format = '%%-%ds : %%d' % maxLen
         aux = [format % (name, self.info[umfDefines[name]])
                for name in umfInfo if name in umfDefines]
-        return '\n'.join( aux )
+        return '\n'.join(aux)
 
     ##
     # 30.11.2005, c
     # 01.12.2005
     # 14.12.2005
     # 01.03.2006
-    def _getIndx( self, mtx ):
+    def _getIndx(self, mtx):
 
-        if sp.isspmatrix_csc( mtx ):
+        if sp.isspmatrix_csc(mtx):
             indx = mtx.indices
             self.isCSR = 0
-        elif sp.isspmatrix_csr( mtx ):
+        elif sp.isspmatrix_csr(mtx):
             indx = mtx.indices
             self.isCSR = 1
         else:
@@ -342,10 +358,10 @@ class UmfpackContext( Struct ):
                 raise ValueError('matrix must have long indices')
 
         if self.isReal:
-            if mtx.data.dtype != np.dtype('<f8'):
+            if mtx.data.dtype != np.dtype('f8'):
                 raise ValueError('matrix must have float64 values')
         else:
-            if mtx.data.dtype != np.dtype('<c16'):
+            if mtx.data.dtype != np.dtype('c16'):
                 raise ValueError('matrix must have complex128 values')
 
         return indx
@@ -353,12 +369,12 @@ class UmfpackContext( Struct ):
     ##
     # 30.11.2005, c
     # last revision: 10.01.2007
-    def symbolic( self, mtx ):
+    def symbolic(self, mtx):
         """Symbolic object (symbolic LU decomposition) computation for a given
         sparsity pattern."""
         self.free_symbolic()
 
-        indx = self._getIndx( mtx )
+        indx = self._getIndx(mtx)
 
         if not assumeSortedIndices:
             # row/column indices cannot be assumed to be sorted
@@ -366,17 +382,16 @@ class UmfpackContext( Struct ):
 
         if self.isReal:
             status, self._symbolic\
-                    = self.funs.symbolic( mtx.shape[0], mtx.shape[1],
+                    = self.funs.symbolic(mtx.shape[0], mtx.shape[1],
                                           mtx.indptr, indx, mtx.data,
-                                          self.control, self.info )
+                                          self.control, self.info)
         else:
             real, imag = mtx.data.real.copy(), mtx.data.imag.copy()
             status, self._symbolic\
-                    = self.funs.symbolic( mtx.shape[0], mtx.shape[1],
+                    = self.funs.symbolic(mtx.shape[0], mtx.shape[1],
                                           mtx.indptr, indx,
                                           real, imag,
-                                          self.control, self.info )
-##         print status, self._symbolic
+                                          self.control, self.info)
 
         if status != UMFPACK_OK:
             raise RuntimeError('%s failed with %s' % (self.funs.symbolic,
@@ -389,7 +404,7 @@ class UmfpackContext( Struct ):
     # 01.12.2005
     # 02.12.2005
     # 01.03.2006
-    def numeric( self, mtx ):
+    def numeric(self, mtx):
         """Numeric object (LU decomposition) computation using the
         symbolic decomposition. The symbolic decomposition is (re)computed
         if necessary."""
@@ -397,35 +412,34 @@ class UmfpackContext( Struct ):
         self.free_numeric()
 
         if self._symbolic is None:
-            self.symbolic( mtx )
+            self.symbolic(mtx)
 
-        indx = self._getIndx( mtx )
+        indx = self._getIndx(mtx)
 
         failCount = 0
         while 1:
             if self.isReal:
                 status, self._numeric\
-                        = self.funs.numeric( mtx.indptr, indx, mtx.data,
+                        = self.funs.numeric(mtx.indptr, indx, mtx.data,
                                              self._symbolic,
-                                             self.control, self.info )
+                                             self.control, self.info)
             else:
                 real, imag = mtx.data.real.copy(), mtx.data.imag.copy()
                 status, self._numeric\
-                        = self.funs.numeric( mtx.indptr, indx,
+                        = self.funs.numeric(mtx.indptr, indx,
                                              real, imag,
                                              self._symbolic,
-                                             self.control, self.info )
-##             print status, self._numeric
+                                             self.control, self.info)
 
             if status != UMFPACK_OK:
                 if status == UMFPACK_WARNING_singular_matrix:
-                    print 'warning: singular matrix'
+                    warnings.warn('Singular matrix', UmfpackWarning)
                     break
                 elif status in (UMFPACK_ERROR_different_pattern,
                                 UMFPACK_ERROR_invalid_Symbolic_object):
                     # Try again.
-                    print 'warning: recomputing symbolic'
-                    self.symbolic( mtx )
+                    warnings.warn('Recomputing symbolic', UmfpackWarning)
+                    self.symbolic(mtx)
                     failCount += 1
                 else:
                     failCount += 100
@@ -437,52 +451,52 @@ class UmfpackContext( Struct ):
 
     ##
     # 14.12.2005, c
-    def report_symbolic( self ):
+    def report_symbolic(self):
         """Print information about the symbolic object. Output depends on
         self.control[UMFPACK_PRL]."""
-        self.funs.report_symbolic( self._symbolic, self.control )
+        self.funs.report_symbolic(self._symbolic, self.control)
 
     ##
     # 14.12.2005, c
-    def report_numeric( self ):
+    def report_numeric(self):
         """Print information about the numeric object. Output depends on
         self.control[UMFPACK_PRL]."""
-        self.funs.report_numeric( self._numeric, self.control )
+        self.funs.report_numeric(self._numeric, self.control)
 
     ##
     # 14.12.2005, c
-    def report_control( self ):
+    def report_control(self):
         """Print control values."""
-        self.funs.report_control( self.control )
+        self.funs.report_control(self.control)
 
     ##
     # 14.12.2005, c
-    def report_info( self ):
+    def report_info(self):
         """Print all status information. Output depends on
         self.control[UMFPACK_PRL]."""
-        self.funs.report_info( self.control, self.info )
+        self.funs.report_info(self.control, self.info)
 
     ##
     # 30.11.2005, c
     # 01.12.2005
-    def free_symbolic( self ):
+    def free_symbolic(self):
         if self._symbolic is not None:
-            self.funs.free_symbolic( self._symbolic )
+            self.funs.free_symbolic(self._symbolic)
             self._symbolic = None
             self.mtx = None
 
     ##
     # 30.11.2005, c
     # 01.12.2005
-    def free_numeric( self ):
+    def free_numeric(self):
         if self._numeric is not None:
-            self.funs.free_numeric( self._numeric )
+            self.funs.free_numeric(self._numeric)
             self._numeric = None
             self.free_symbolic()
 
     ##
     # 30.11.2005, c
-    def free( self ):
+    def free(self):
         self.free_symbolic()
         self.free_numeric()
 
@@ -492,7 +506,7 @@ class UmfpackContext( Struct ):
     # 02.12.2005
     # 21.12.2005
     # 01.03.2006
-    def solve( self, sys, mtx, rhs, autoTranspose = False ):
+    def solve(self, sys, mtx, rhs, autoTranspose=False):
         """
         Solution of system of linear equation using the Numeric object.
 
@@ -512,8 +526,10 @@ class UmfpackContext( Struct ):
         if autoTranspose and self.isCSR:
             ##
             # UMFPACK uses CSC internally...
-            if self.family in umfRealTypes: ii = 0
-            else: ii = 1
+            if self.family in umfRealTypes:
+                ii = 0
+            else:
+                ii = 1
             if sys in umfSys_transposeMap[ii]:
                 sys = umfSys_transposeMap[ii][sys]
             else:
@@ -525,45 +541,46 @@ class UmfpackContext( Struct ):
         else:
             raise RuntimeError('numeric() not called')
 
-        indx = self._getIndx( mtx )
+        indx = self._getIndx(mtx)
 
         if self.isReal:
-            rhs = rhs.astype( np.float64 )
-            sol = np.zeros( (mtx.shape[1],), dtype = np.float64 )
-            status = self.funs.solve( sys, mtx.indptr, indx, mtx.data, sol, rhs,
-                                      self._numeric, self.control, self.info )
+            rhs = rhs.astype(np.float64)
+            sol = np.zeros((mtx.shape[1],), dtype=np.float64)
+            status = self.funs.solve(sys, mtx.indptr, indx, mtx.data, sol, rhs,
+                                      self._numeric, self.control, self.info)
         else:
-            rhs = rhs.astype( np.complex128 )
-            sol = np.zeros( (mtx.shape[1],), dtype = np.complex128 )
+            rhs = rhs.astype(np.complex128)
+            sol = np.zeros((mtx.shape[1],), dtype=np.complex128)
             mreal, mimag = mtx.data.real.copy(), mtx.data.imag.copy()
             sreal, simag = sol.real.copy(), sol.imag.copy()
             rreal, rimag = rhs.real.copy(), rhs.imag.copy()
-            status = self.funs.solve( sys, mtx.indptr, indx,
+            status = self.funs.solve(sys, mtx.indptr, indx,
                                       mreal, mimag, sreal, simag, rreal, rimag,
-                                      self._numeric, self.control, self.info )
+                                      self._numeric, self.control, self.info)
             sol.real, sol.imag = sreal, simag
 
-        #self.funs.report_info( self.control, self.info )
-        #pause()
+        # self.funs.report_info( self.control, self.info )
+        # pause()
         if status != UMFPACK_OK:
             if status == UMFPACK_WARNING_singular_matrix:
                 ## Change inf, nan to zeros.
-                print 'zeroing nan and inf entries...'
-                sol[~np.isfinite( sol )] = 0.0
+                warnings.warn('Zeroing nan and inf entries...', UmfpackWarning)
+                sol[~np.isfinite(sol)] = 0.0
             else:
                 raise RuntimeError('%s failed with %s' % (self.funs.solve,
                                                            umfStatus[status]))
         econd = 1.0 / self.info[UMFPACK_RCOND]
         if econd > self.maxCond:
-            print 'warning: (almost) singular matrix! '\
+            msg = '(almost) singular matrix! '\
                   + '(estimated cond. number: %.2e)' % econd
+            warnings.warn(msg, UmfpackWarning)
 
         return sol
 
     ##
     # 30.11.2005, c
     # 01.12.2005
-    def linsolve( self, sys, mtx, rhs, autoTranspose = False ):
+    def linsolve(self, sys, mtx, rhs, autoTranspose=False):
         """
         One-shot solution of system of linear equation. Reuses Numeric object
         if possible.
@@ -579,17 +596,16 @@ class UmfpackContext( Struct ):
                       assumes CSC internally
         """
 
-#        print self.family
         if sys not in umfSys:
             raise ValueError('sys must be in' % umfSys)
 
         if self._numeric is None:
-            self.numeric( mtx )
+            self.numeric(mtx)
         else:
             if self.mtx is not mtx:
-                self.numeric( mtx )
+                self.numeric(mtx)
 
-        sol = self.solve( sys, mtx, rhs, autoTranspose )
+        sol = self.solve(sys, mtx, rhs, autoTranspose)
         self.free_numeric()
 
         return sol
@@ -597,7 +613,7 @@ class UmfpackContext( Struct ):
     ##
     # 30.11.2005, c
     # 01.12.2005
-    def __call__( self, sys, mtx, rhs, autoTranspose = False ):
+    def __call__(self, sys, mtx, rhs, autoTranspose=False):
         """
         Uses solve() or linsolve() depending on the presence of the Numeric
         object.
@@ -614,13 +630,13 @@ class UmfpackContext( Struct ):
         """
 
         if self._numeric is not None:
-            return self.solve( sys, mtx, rhs, autoTranspose )
+            return self.solve(sys, mtx, rhs, autoTranspose)
         else:
-            return self.linsolve( sys, mtx, rhs, autoTranspose )
+            return self.linsolve(sys, mtx, rhs, autoTranspose)
 
     ##
     # 21.09.2006, added by Nathan Bell
-    def lu( self, mtx ):
+    def lu(self, mtx):
         """
         Returns an LU decomposition of an m-by-n matrix in the form
         (L, U, P, Q, R, do_recip):
@@ -637,40 +653,40 @@ class UmfpackContext( Struct ):
                 LU = P(R^-1)AQ   when do_recip is false
         """
 
-        #this should probably be changed
+        # this should probably be changed
         mtx = mtx.tocsc()
-        self.numeric( mtx )
+        self.numeric(mtx)
 
-        #first find out how much space to reserve
+        # first find out how much space to reserve
         (status, lnz, unz, n_row, n_col, nz_udiag)\
-                 = self.funs.get_lunz( self._numeric )
+                 = self.funs.get_lunz(self._numeric)
 
         if status != UMFPACK_OK:
             raise RuntimeError('%s failed with %s' % (self.funs.get_lunz,
                                                        umfStatus[status]))
 
-        #allocate storage for decomposition data
+        # allocate storage for decomposition data
         i_type = mtx.indptr.dtype
 
-        Lp = np.zeros( (n_row+1,), dtype = i_type )
-        Lj = np.zeros( (lnz,), dtype = i_type )
-        Lx = np.zeros( (lnz,), dtype = np.double )
+        Lp = np.zeros((n_row+1,), dtype=i_type)
+        Lj = np.zeros((lnz,), dtype=i_type)
+        Lx = np.zeros((lnz,), dtype=np.double)
 
-        Up = np.zeros( (n_col+1,), dtype = i_type )
-        Ui = np.zeros( (unz,), dtype = i_type )
-        Ux = np.zeros( (unz,), dtype = np.double )
+        Up = np.zeros((n_col+1,), dtype=i_type)
+        Ui = np.zeros((unz,), dtype=i_type)
+        Ux = np.zeros((unz,), dtype=np.double)
 
-        P  = np.zeros( (n_row,), dtype = i_type )
-        Q  = np.zeros( (n_col,), dtype = i_type )
+        P = np.zeros((n_row,), dtype=i_type)
+        Q = np.zeros((n_col,), dtype=i_type)
 
-        Dx = np.zeros( (min(n_row,n_col),), dtype = np.double )
+        Dx = np.zeros((min(n_row,n_col),), dtype=np.double)
 
-        Rs = np.zeros( (n_row,), dtype = np.double )
+        Rs = np.zeros((n_row,), dtype=np.double)
 
         if self.isReal:
-            (status,do_recip) = self.funs.get_numeric( Lp,Lj,Lx,Up,Ui,Ux,
+            (status,do_recip) = self.funs.get_numeric(Lp,Lj,Lx,Up,Ui,Ux,
                                                        P,Q,Dx,Rs,
-                                                       self._numeric )
+                                                       self._numeric)
 
             if status != UMFPACK_OK:
                 raise RuntimeError('%s failed with %s'
@@ -683,10 +699,10 @@ class UmfpackContext( Struct ):
             return (L,U,P,Q,R,bool(do_recip))
 
         else:
-            #allocate additional storage for imaginary parts
-            Lz = np.zeros( (lnz,), dtype = np.double )
-            Uz = np.zeros( (unz,), dtype = np.double )
-            Dz = np.zeros( (min(n_row,n_col),), dtype = np.double )
+            # allocate additional storage for imaginary parts
+            Lz = np.zeros((lnz,), dtype=np.double)
+            Uz = np.zeros((unz,), dtype=np.double)
+            Dz = np.zeros((min(n_row,n_col),), dtype=np.double)
 
             (status,do_recip) = self.funs.get_numeric(Lp,Lj,Lx,Lz,Up,Ui,Ux,Uz,
                                                       P,Q,Dx,Dz,Rs,
@@ -696,10 +712,9 @@ class UmfpackContext( Struct ):
                 raise RuntimeError('%s failed with %s'
                         % (self.funs.get_numeric, umfStatus[status]))
 
-
-            Lxz = np.zeros( (lnz,), dtype = np.complex128 )
-            Uxz = np.zeros( (unz,), dtype = np.complex128 )
-            Dxz = np.zeros( (min(n_row,n_col),), dtype = np.complex128 )
+            Lxz = np.zeros((lnz,), dtype=np.complex128)
+            Uxz = np.zeros((unz,), dtype=np.complex128)
+            Dxz = np.zeros((min(n_row,n_col),), dtype=np.complex128)
 
             Lxz.real,Lxz.imag = Lx,Lz
             Uxz.real,Uxz.imag = Ux,Uz
