@@ -83,6 +83,7 @@ void expandptr(const I n_row,
     }
 }
 
+
 /*
  * Scale the rows of a CSR matrix *in place*
  *
@@ -104,6 +105,7 @@ void csr_scale_rows(const I n_row,
     }
 }
 
+
 /*
  * Scale the columns of a CSR matrix *in place*
  *
@@ -189,7 +191,6 @@ I csr_count_blocks(const I n_row,
  *
  * 
  */
-
 template <class I, class T>
 void csr_tobsr(const I n_row,
 	           const I n_col, 
@@ -243,15 +244,13 @@ void csr_tobsr(const I n_row,
 }
 
 
-
 /*
- * Sort CSR column indices inplace
+ * Determine whether the CSR column indices are in sorted order.
  *
  * Input Arguments:
  *   I  n_row           - number of rows in A
  *   I  Ap[n_row+1]     - row pointer
  *   I  Aj[nnz(A)]      - column indices
- *   T  Ax[nnz(A)]      - nonzeros 
  *
  */
 template <class I>
@@ -268,11 +267,54 @@ bool csr_has_sorted_indices(const I n_row,
   }
   return true;
 }
+
+
+
+/*
+ * Determine whether the matrix structure is canonical CSR.
+ * Canonical CSR implies that column indices within each row
+ * are (1) sorted and (2) unique.  Matrices that meet these 
+ * conditions facilitate faster matrix computations.
+ *
+ * Input Arguments:
+ *   I  n_row           - number of rows in A
+ *   I  Ap[n_row+1]     - row pointer
+ *   I  Aj[nnz(A)]      - column indices
+ *
+ */
+template <class I>
+bool csr_has_canonical_format(const I n_row, 
+                              const I Ap[],
+                              const I Aj[])
+{
+    for(I i = 0; i < n_row; i++){
+        if (Ap[i] > Ap[i+1])
+            return false;
+        for(I jj = Ap[i] + 1; jj < Ap[i+1]; jj++){
+            if( !(Aj[jj-1] < Aj[jj]) ){
+                return false;
+            }
+        }
+    }
+    return true;
+}
+
+
 template< class T1, class T2 >
 bool kv_pair_less(const std::pair<T1,T2>& x, const std::pair<T1,T2>& y){
     return x.first < y.first;
 }
 
+/*
+ * Sort CSR column indices inplace
+ *
+ * Input Arguments:
+ *   I  n_row           - number of rows in A
+ *   I  Ap[n_row+1]     - row pointer
+ *   I  Aj[nnz(A)]      - column indices
+ *   T  Ax[nnz(A)]      - nonzeros 
+ *
+ */
 template<class I, class T>
 void csr_sort_indices(const I n_row,
                       const I Ap[], 
@@ -486,52 +528,8 @@ void csr_matmat_pass1(const I n_row,
                       const I Bj[],
                             I Cp[])
 {
-//    // method that uses O(1) temp storage
-//    const I hash_size = 1 << 5;
-//    I vals[hash_size];
-//    I mask[hash_size];
-//
-//    std::set<I> spill;    
-//    
-//    for(I i = 0; i < hash_size; i++){
-//        vals[i] = -1;
-//        mask[i] = -1;
-//    }
-//
-//    Cp[0] = 0;
-//
-//    I slow_inserts = 0;
-//    I total_inserts = 0;
-//    I nnz = 0;
-//    for(I i = 0; i < n_row; i++){
-//        spill.clear();
-//        for(I jj = Ap[i]; jj < Ap[i+1]; jj++){
-//            I j = Aj[jj];
-//            for(I kk = Bp[j]; kk < Bp[j+1]; kk++){
-//                I k = Bj[kk];
-//                // I hash = k & (hash_size - 1);
-//                I hash = ((I)2654435761 * k) & (hash_size -1 );
-//                total_inserts++;
-//                if(mask[hash] != i){
-//                    mask[hash] = i;                        
-//                    vals[hash] = k;
-//                    nnz++;
-//                } else {
-//                    if (vals[hash] != k){
-//                        slow_inserts++;
-//                        spill.insert(k);
-//                    }
-//                }
-//            }
-//        }       
-//        nnz += spill.size();
-//        Cp[i+1] = nnz;
-//    }
-//
-//    std::cout << "slow fraction " << ((float) slow_inserts)/ ((float) total_inserts) << std::endl;
-
     // method that uses O(n) temp storage
-    std::vector<I> mask(n_col,-1);
+    std::vector<I> mask(n_col, -1);
     Cp[0] = 0;
 
     I nnz = 0;
@@ -594,7 +592,7 @@ void csr_matmat_pass2(const I n_row,
 
                 if(next[k] == -1){
                     next[k] = head;                        
-                    head = k;
+                    head  = k;
                     length++;
                 }
             }
@@ -620,28 +618,13 @@ void csr_matmat_pass2(const I n_row,
 }
 
 
-
-
-
 /*
- * Compute C = A (bin_op) B for CSR matrices A,B
+ * Compute C = A (binary_op) B for CSR matrices that are not
+ * necessarily canonical CSR format.  Specifically, this method
+ * works even when the input matrices have duplicate and/or
+ * unsorted column indices within a given row.
  *
- *   bin_op(x,y) - binary operator to apply elementwise
- *
- *   
- * Input Arguments:
- *   I    n_row       - number of rows in A (and B)
- *   I    n_col       - number of columns in A (and B)
- *   I    Ap[n_row+1] - row pointer
- *   I    Aj[nnz(A)]  - column indices
- *   T    Ax[nnz(A)]  - nonzeros
- *   I    Bp[?]       - row pointer
- *   I    Bj[nnz(B)]  - column indices
- *   T    Bx[nnz(B)]  - nonzeros
- * Output Arguments:
- *   I    Cp[n_row+1] - row pointer
- *   I    Cj[nnz(C)]  - column indices
- *   T    Cx[nnz(C)]  - nonzeros
+ * Refer to csr_binop_csr() for additional information
  *   
  * Note:
  *   Output arrays Cp, Cj, and Cx must be preallocated
@@ -649,28 +632,109 @@ void csr_matmat_pass2(const I n_row,
  *          nnz(C) <= nnz(A) + nnz(B)
  *
  * Note: 
+ *   Input:  A and B column indices are not assumed to be in sorted order 
+ *   Output: C column indices are not generally in sorted order
+ *           C will not contain any duplicate entries or explicit zeros.
+ *
+ */
+template <class I, class T, class T2, class binary_op>
+void csr_binop_csr_general(const I n_row, const I n_col, 
+                           const I Ap[], const I Aj[], const T Ax[],
+                           const I Bp[], const I Bj[], const T Bx[],
+                                 I Cp[],       I Cj[],       T2 Cx[],
+                           const binary_op& op)
+{
+    //Method that works for duplicate and/or unsorted indices
+
+    std::vector<I>  next(n_col,-1);
+    std::vector<T> A_row(n_col, 0);
+    std::vector<T> B_row(n_col, 0);
+
+    I nnz = 0;
+    Cp[0] = 0;
+    
+    for(I i = 0; i < n_row; i++){
+        I head   = -2;
+        I length =  0;
+    
+        //add a row of A to A_row
+        I i_start = Ap[i];
+        I i_end   = Ap[i+1];
+        for(I jj = i_start; jj < i_end; jj++){
+            I j = Aj[jj];
+    
+            A_row[j] += Ax[jj];
+    
+            if(next[j] == -1){
+                next[j] = head;                       
+                head = j;
+                length++;
+            }
+        }
+    
+        //add a row of B to B_row
+        i_start = Bp[i];
+        i_end   = Bp[i+1];
+        for(I jj = i_start; jj < i_end; jj++){
+            I j = Bj[jj];
+    
+            B_row[j] += Bx[jj];
+    
+            if(next[j] == -1){
+                next[j] = head;                       
+                head = j;
+                length++;
+            }
+        }
+    
+   
+        // scan through columns where A or B has 
+        // contributed a non-zero entry
+        for(I jj = 0; jj < length; jj++){
+            T result = op(A_row[head], B_row[head]);
+    
+            if(result != 0){
+                Cj[nnz] = head;
+                Cx[nnz] = result;
+                nnz++;
+            }
+    
+            I temp = head;               
+            head = next[head];
+    
+            next[temp]  = -1;
+            A_row[temp] =  0;                             
+            B_row[temp] =  0;
+        }
+
+        Cp[i + 1] = nnz;
+    }
+}
+
+
+
+/*
+ * Compute C = A (binary_op) B for CSR matrices that are in the 
+ * canonical CSR format.  Specifically, this method requires that
+ * the rows of the input matrices are free of duplicate column indices
+ * and that the column indices are in sorted order.
+ *
+ * Refer to csr_binop_csr() for additional information
+ *
+ * Note: 
  *   Input:  A and B column indices are assumed to be in sorted order 
- *   Output: C column indices are assumed to be in sorted order
+ *   Output: C column indices will be in sorted order
  *           Cx will not contain any zero entries
  *
  */
-template <class I, class T, class bin_op>
-void csr_binop_csr(const I n_row,
-                   const I n_col, 
-                   const I Ap[], 
-                   const I Aj[], 
-                   const T Ax[],
-                   const I Bp[],
-                   const I Bj[],
-                   const T Bx[],
-                         I Cp[],
-                         I Cj[],
-                         T Cx[],
-                   const bin_op& op)
+template <class I, class T, class T2, class binary_op>
+void csr_binop_csr_canonical(const I n_row, const I n_col, 
+                             const I Ap[], const I Aj[], const T Ax[],
+                             const I Bp[], const I Bj[], const T Bx[],
+                                   I Cp[],       I Cj[],       T2 Cx[],
+                             const binary_op& op)
 {
-    //Method that works for sorted indices
-    // assert( csr_has_sorted_indices(n_row,Ap,Aj) );
-    // assert( csr_has_sorted_indices(n_row,Bp,Bj) );
+    //Method that works for canonical CSR matrices 
 
     Cp[0] = 0;
     I nnz = 0;
@@ -734,11 +798,109 @@ void csr_binop_csr(const I n_row,
             }
             B_pos++;
         }
+
         Cp[i+1] = nnz;
     }
 }
 
+
+/*
+ * Compute C = A (binary_op) B for CSR matrices A,B where the column 
+ * indices with the rows of A and B are known to be sorted.
+ *
+ *   binary_op(x,y) - binary operator to apply elementwise
+ *
+ * Input Arguments:
+ *   I    n_row       - number of rows in A (and B)
+ *   I    n_col       - number of columns in A (and B)
+ *   I    Ap[n_row+1] - row pointer
+ *   I    Aj[nnz(A)]  - column indices
+ *   T    Ax[nnz(A)]  - nonzeros
+ *   I    Bp[n_row+1] - row pointer
+ *   I    Bj[nnz(B)]  - column indices
+ *   T    Bx[nnz(B)]  - nonzeros
+ * Output Arguments:
+ *   I    Cp[n_row+1] - row pointer
+ *   I    Cj[nnz(C)]  - column indices
+ *   T    Cx[nnz(C)]  - nonzeros
+ *   
+ * Note:
+ *   Output arrays Cp, Cj, and Cx must be preallocated
+ *   If nnz(C) is not known a priori, a conservative bound is:
+ *          nnz(C) <= nnz(A) + nnz(B)
+ *
+ * Note: 
+ *   Input:  A and B column indices are not assumed to be in sorted order.
+ *   Output: C column indices will be in sorted if both A and B have sorted indices.
+ *           Cx will not contain any zero entries
+ *
+ */
+template <class I, class T, class T2, class binary_op>
+void csr_binop_csr(const I n_row,
+                   const I n_col, 
+                   const I Ap[], 
+                   const I Aj[], 
+                   const T Ax[],
+                   const I Bp[],
+                   const I Bj[],
+                   const T Bx[],
+                         I Cp[],
+                         I Cj[],
+                         T2 Cx[],
+                   const binary_op& op)
+{
+    if (csr_has_canonical_format(n_row,Ap,Aj) && csr_has_canonical_format(n_row,Bp,Bj))
+        csr_binop_csr_canonical(n_row, n_col, Ap, Aj, Ax, Bp, Bj, Bx, Cp, Cj, Cx, op);
+    else
+        csr_binop_csr_general(n_row, n_col, Ap, Aj, Ax, Bp, Bj, Bx, Cp, Cj, Cx, op);
+}
+
 /* element-wise binary operations*/
+template <class I, class T, class T2>
+void csr_ne_csr(const I n_row, const I n_col, 
+                   const I Ap[], const I Aj[], const T Ax[],
+                   const I Bp[], const I Bj[], const T Bx[],
+                         I Cp[],       I Cj[],      T2 Cx[])
+{
+    csr_binop_csr(n_row,n_col,Ap,Aj,Ax,Bp,Bj,Bx,Cp,Cj,Cx,std::not_equal_to<T>());
+}
+
+template <class I, class T, class T2>
+void csr_lt_csr(const I n_row, const I n_col, 
+                   const I Ap[], const I Aj[], const T Ax[],
+                   const I Bp[], const I Bj[], const T Bx[],
+                         I Cp[],       I Cj[],      T2 Cx[])
+{
+    csr_binop_csr(n_row,n_col,Ap,Aj,Ax,Bp,Bj,Bx,Cp,Cj,Cx,std::less<T>());
+}
+
+template <class I, class T, class T2>
+void csr_gt_csr(const I n_row, const I n_col, 
+                   const I Ap[], const I Aj[], const T Ax[],
+                   const I Bp[], const I Bj[], const T Bx[],
+                         I Cp[],       I Cj[],      T2 Cx[])
+{
+    csr_binop_csr(n_row,n_col,Ap,Aj,Ax,Bp,Bj,Bx,Cp,Cj,Cx,std::greater<T>());
+}
+
+template <class I, class T, class T2>
+void csr_le_csr(const I n_row, const I n_col, 
+                   const I Ap[], const I Aj[], const T Ax[],
+                   const I Bp[], const I Bj[], const T Bx[],
+                         I Cp[],       I Cj[],      T2 Cx[])
+{
+    csr_binop_csr(n_row,n_col,Ap,Aj,Ax,Bp,Bj,Bx,Cp,Cj,Cx,std::less_equal<T>());
+}
+
+template <class I, class T, class T2>
+void csr_ge_csr(const I n_row, const I n_col, 
+                   const I Ap[], const I Aj[], const T Ax[],
+                   const I Bp[], const I Bj[], const T Bx[],
+                         I Cp[],       I Cj[],      T2 Cx[])
+{
+    csr_binop_csr(n_row,n_col,Ap,Aj,Ax,Bp,Bj,Bx,Cp,Cj,Cx,std::greater_equal<T>());
+}
+
 template <class I, class T>
 void csr_elmul_csr(const I n_row, const I n_col, 
                    const I Ap[], const I Aj[], const T Ax[],
@@ -1026,4 +1188,108 @@ I csr_count_diagonals(const I n_row,
 }
 
 
+/*
+ * Sample the matrix at specific locations
+ * 
+ * Determine the matrix value for each row,col pair
+ *    Bx[n] = A(Bi[n],Bj[n])
+ *
+ * Input Arguments:
+ *   I  n_row         - number of rows in A
+ *   I  n_col         - number of columns in A
+ *   I  Ap[n_row+1]   - row pointer
+ *   I  Aj[nnz(A)]    - column indices
+ *   T  Ax[nnz(A)]    - nonzeros
+ *   I  n_samples     - number of samples
+ *   I  Bi[N]         - sample rows
+ *   I  Bj[N]         - sample columns
+ *
+ * Output Arguments:
+ *   T  Bx[N]         - sample values
+ *
+ * Note:
+ *   Output array Yx must be preallocated
+ *
+ *   Complexity: varies 
+ *
+ *   TODO handle other cases with asymptotically optimal method
+ *
+ */
+template <class I, class T>
+void csr_sample_values(const I n_row,
+                       const I n_col,
+                       const I Ap[],
+                       const I Aj[],
+                       const T Ax[],
+                       const I n_samples,
+                       const I Bi[],
+                       const I Bj[],
+                             T Bx[])
+{
+    // ideally we'd do the following
+    // Case 1: A is canonical and B is sorted by row and column
+    //   -> special purpose csr_binop_csr() (optimized form)
+    // Case 2: A is canonical and B is unsorted and max(log(Ap[i+1] - Ap[i])) > log(num_samples)
+    //   -> do binary searches for each sample
+    // Case 3: A is canonical and B is unsorted and max(log(Ap[i+1] - Ap[i])) < log(num_samples)
+    //   -> sort B by row and column and use Case 1
+    // Case 4: A is not canonical and num_samples ~ nnz
+    //   -> special purpose csr_binop_csr() (general form)
+    // Case 5: A is not canonical and num_samples << nnz
+    //   -> do linear searches for each sample
+
+    const I nnz = Ap[n_row];
+
+    const I threshold = nnz / 10; // constant is arbitrary
+
+    if (n_samples > threshold && csr_has_canonical_format(n_row, Ap, Aj))
+    {
+        for(I n = 0; n < n_samples; n++)
+        {
+            const I i = Bi[n] < 0 ? Bi[n] + n_row : Bi[n]; // sample row
+            const I j = Bj[n] < 0 ? Bj[n] + n_col : Bj[n]; // sample column
+   
+            const I row_start = Ap[i];
+            const I row_end   = Ap[i+1];
+            
+            if (row_start < row_end)
+            {
+                const I offset = std::lower_bound(Aj + row_start, Aj + row_end, j) - Aj;
+                
+                if (offset < row_end && Aj[offset] == j)
+                    Bx[n] = Ax[offset];
+                else
+                    Bx[n] = 0;
+            }
+            else
+            {
+                Bx[n] = 0;
+            }
+    
+        }
+    }
+    else
+    {
+        for(I n = 0; n < n_samples; n++)
+        {
+            const I i = Bi[n] < 0 ? Bi[n] + n_row : Bi[n]; // sample row
+            const I j = Bj[n] < 0 ? Bj[n] + n_col : Bj[n]; // sample column
+    
+            const I row_start = Ap[i];
+            const I row_end   = Ap[i+1];
+           
+            T x = 0;
+            
+            for(I jj = row_start; jj < row_end; jj++)
+            {
+                if (Aj[jj] == j)
+                    x += Ax[jj];
+            }
+    
+            Bx[n] = x;
+        }
+
+    }
+}
+
 #endif
