@@ -1,4 +1,6 @@
 #!/usr/bin/env python
+from __future__ import division, print_function, absolute_import
+
 
 def configuration(parent_package='',top_path=None):
     import numpy
@@ -6,10 +8,12 @@ def configuration(parent_package='',top_path=None):
 
     config = Configuration('sparsetools',parent_package,top_path)
 
-    for fmt in ['csr','csc','coo','bsr','dia']:
-        sources = [ fmt + '_wrap.cxx' ]
-        depends = [ fmt + '.h' ]
-        config.add_extension('_' + fmt, sources=sources, depends=depends)
+    for fmt in ['csr','csc','coo','bsr','dia','csgraph']:
+        sources = [fmt + '_wrap.cxx']
+        depends = [fmt + '.h']
+        config.add_extension('_' + fmt, sources=sources,
+            define_macros=[('__STDC_FORMAT_MACROS', 1)],
+            depends=depends)
 
     return config
 
