@@ -1,5 +1,7 @@
 # Copyright Anne M. Archibald 2008
 # Released under the scipy license
+from __future__ import division, print_function, absolute_import
+
 import sys
 import numpy as np
 from heapq import heappush, heappop
@@ -12,18 +14,18 @@ __all__ = ['minkowski_distance_p', 'minkowski_distance',
 
 def minkowski_distance_p(x, y, p=2):
     """
-    Compute the p-th power of the L**p distance between x and y.
+    Compute the p-th power of the L**p distance between two arrays.
 
     For efficiency, this function computes the L**p distance but does
-    not extract the pth root. If p is 1 or infinity, this is equal to
+    not extract the pth root. If `p` is 1 or infinity, this is equal to
     the actual L**p distance.
 
     Parameters
     ----------
-    x : array_like, M by K
-
-    y : array_like, N by K
-
+    x : (M, K) array_like
+        Input array.
+    y : (N, K) array_like
+        Input array.
     p : float, 1 <= p <= infinity
         Which Minkowski p-norm to use.
 
@@ -42,16 +44,17 @@ def minkowski_distance_p(x, y, p=2):
     else:
         return np.sum(np.abs(y-x)**p, axis=-1)
 
+
 def minkowski_distance(x, y, p=2):
     """
-    Compute the L**p distance between x and y.
+    Compute the L**p distance between two arrays.
 
     Parameters
     ----------
-    x : array_like, M by K
-
-    y : array_like, N by K
-
+    x : (M, K) array_like
+        Input array.
+    y : (N, K) array_like
+        Input array.
     p : float, 1 <= p <= infinity
         Which Minkowski p-norm to use.
 
@@ -68,6 +71,7 @@ def minkowski_distance(x, y, p=2):
     else:
         return minkowski_distance_p(x, y, p)**(1./p)
 
+
 class Rectangle(object):
     """Hyperrectangle class.
 
@@ -80,19 +84,28 @@ class Rectangle(object):
         self.m, = self.maxes.shape
 
     def __repr__(self):
-        return "<Rectangle %s>" % zip(self.mins, self.maxes)
+        return "<Rectangle %s>" % list(zip(self.mins, self.maxes))
 
     def volume(self):
         """Total volume."""
         return np.prod(self.maxes-self.mins)
 
     def split(self, d, split):
-        """Produce two hyperrectangles by splitting along axis d.
+        """
+        Produce two hyperrectangles by splitting.
 
         In general, if you need to compute maximum and minimum
         distances to the children, it can be done more efficiently
         by updating the maximum and minimum distances to the parent.
-        """ # FIXME: do this
+
+        Parameters
+        ----------
+        d : int
+            Axis to split hyperrectangle along.
+        split :
+            Input.
+
+        """
         mid = np.copy(self.maxes)
         mid[d] = split
         less = Rectangle(self.mins, mid)
@@ -102,19 +115,59 @@ class Rectangle(object):
         return less, greater
 
     def min_distance_point(self, x, p=2.):
-        """Compute the minimum distance between x and a point in the hyperrectangle."""
+        """
+        Return the minimum distance between input and points in the hyperrectangle.
+
+        Parameters
+        ----------
+        x : array_like
+            Input.
+        p : float, optional
+            Input.
+
+        """
         return minkowski_distance(0, np.maximum(0,np.maximum(self.mins-x,x-self.maxes)),p)
 
     def max_distance_point(self, x, p=2.):
-        """Compute the maximum distance between x and a point in the hyperrectangle."""
+        """
+        Return the maximum distance between input and points in the hyperrectangle.
+
+        Parameters
+        ----------
+        x : array_like
+            Input array.
+        p : float, optional
+            Input.
+
+        """
         return minkowski_distance(0, np.maximum(self.maxes-x,x-self.mins),p)
 
     def min_distance_rectangle(self, other, p=2.):
-        """Compute the minimum distance between points in the two hyperrectangles."""
+        """
+        Compute the minimum distance between points in the two hyperrectangles.
+
+        Parameters
+        ----------
+        other : hyperrectangle
+            Input.
+        p : float
+            Input.
+
+        """
         return minkowski_distance(0, np.maximum(0,np.maximum(self.mins-other.maxes,other.mins-self.maxes)),p)
 
     def max_distance_rectangle(self, other, p=2.):
-        """Compute the maximum distance between points in the two hyperrectangles."""
+        """
+        Compute the maximum distance between points in the two hyperrectangles.
+
+        Parameters
+        ----------
+        other : hyperrectangle
+            Input.
+        p : float, optional
+            Input.
+
+        """
         return minkowski_distance(0, np.maximum(self.maxes-other.mins,other.maxes-self.mins),p)
 
 
@@ -122,10 +175,30 @@ class KDTree(object):
     """
     kd-tree for quick nearest-neighbor lookup
 
-    This class provides an index into a set of k-dimensional points
-    which can be used to rapidly look up the nearest neighbors of any
-    point.
+    This class provides an index into a set of k-dimensional points which
+    can be used to rapidly look up the nearest neighbors of any point.
 
+    Parameters
+    ----------
+    data : (N,K) array_like
+        The data points to be indexed. This array is not copied, and
+        so modifying this data will result in bogus results.
+    leafsize : int, optional
+        The number of points at which the algorithm switches over to
+        brute-force.  Has to be positive.
+
+    Raises
+    ------
+    RuntimeError
+        The maximum recursion limit can be exceeded for large data
+        sets.  If this happens, either increase the value for the `leafsize`
+        parameter or increase the recursion limit by::
+
+            >>> import sys
+            >>> sys.setrecursionlimit(10000)
+
+    Notes
+    -----
     The algorithm used is described in Maneewongvatana and Mount 1999.
     The general idea is that the kd-tree is a binary tree, each of whose
     nodes represents an axis-aligned hyperrectangle. Each node specifies
@@ -152,21 +225,10 @@ class KDTree(object):
 
     """
     def __init__(self, data, leafsize=10):
-        """Construct a kd-tree.
-
-        Parameters
-        ----------
-        data : array_like, shape (n,k)
-            The data points to be indexed. This array is not copied, and
-            so modifying this data will result in bogus results.
-        leafsize : positive int
-            The number of points at which the algorithm switches over to
-            brute-force.
-        """
         self.data = np.asarray(data)
         self.n, self.m = np.shape(self.data)
         self.leafsize = int(leafsize)
-        if self.leafsize<1:
+        if self.leafsize < 1:
             raise ValueError("leafsize must be at least 1")
         self.maxes = np.amax(self.data,axis=0)
         self.mins = np.amin(self.data,axis=0)
@@ -175,11 +237,20 @@ class KDTree(object):
 
     class node(object):
         if sys.version_info[0] >= 3:
-            def __lt__(self, other): id(self) < id(other)
-            def __gt__(self, other): id(self) > id(other)
-            def __le__(self, other): id(self) <= id(other)
-            def __ge__(self, other): id(self) >= id(other)
-            def __eq__(self, other): id(self) == id(other)
+            def __lt__(self, other):
+                id(self) < id(other)
+
+            def __gt__(self, other):
+                id(self) > id(other)
+
+            def __le__(self, other):
+                id(self) <= id(other)
+
+            def __ge__(self, other):
+                id(self) >= id(other)
+
+            def __eq__(self, other):
+                id(self) == id(other)
 
     class leafnode(node):
         def __init__(self, idx):
@@ -195,16 +266,16 @@ class KDTree(object):
             self.children = less.children+greater.children
 
     def __build(self, idx, maxes, mins):
-        if len(idx)<=self.leafsize:
+        if len(idx) <= self.leafsize:
             return KDTree.leafnode(idx)
         else:
             data = self.data[idx]
-            #maxes = np.amax(data,axis=0)
-            #mins = np.amin(data,axis=0)
+            # maxes = np.amax(data,axis=0)
+            # mins = np.amin(data,axis=0)
             d = np.argmax(maxes-mins)
             maxval = maxes[d]
             minval = mins[d]
-            if maxval==minval:
+            if maxval == minval:
                 # all points are identical; warn user?
                 return KDTree.leafnode(idx)
             data = data[:,d]
@@ -212,19 +283,19 @@ class KDTree(object):
             # sliding midpoint rule; see Maneewongvatana and Mount 1999
             # for arguments that this is a good idea.
             split = (maxval+minval)/2
-            less_idx = np.nonzero(data<=split)[0]
-            greater_idx = np.nonzero(data>split)[0]
-            if len(less_idx)==0:
+            less_idx = np.nonzero(data <= split)[0]
+            greater_idx = np.nonzero(data > split)[0]
+            if len(less_idx) == 0:
                 split = np.amin(data)
-                less_idx = np.nonzero(data<=split)[0]
-                greater_idx = np.nonzero(data>split)[0]
-            if len(greater_idx)==0:
+                less_idx = np.nonzero(data <= split)[0]
+                greater_idx = np.nonzero(data > split)[0]
+            if len(greater_idx) == 0:
                 split = np.amax(data)
-                less_idx = np.nonzero(data<split)[0]
-                greater_idx = np.nonzero(data>=split)[0]
-            if len(less_idx)==0:
+                less_idx = np.nonzero(data < split)[0]
+                greater_idx = np.nonzero(data >= split)[0]
+            if len(less_idx) == 0:
                 # _still_ zero? all must have the same value
-                if not np.all(data==data[0]):
+                if not np.all(data == data[0]):
                     raise ValueError("Troublesome data array: %s" % data)
                 split = data[0]
                 less_idx = np.arange(len(data)-1)
@@ -241,8 +312,8 @@ class KDTree(object):
     def __query(self, x, k=1, eps=0, p=2, distance_upper_bound=np.inf):
 
         side_distances = np.maximum(0,np.maximum(x-self.maxes,self.mins-x))
-        if p!=np.inf:
-            side_distances**=p
+        if p != np.inf:
+            side_distances **= p
             min_distance = np.sum(side_distances)
         else:
             min_distance = np.amax(side_distances)
@@ -260,14 +331,14 @@ class KDTree(object):
         # entries are (-distance**p, i)
         neighbors = []
 
-        if eps==0:
-            epsfac=1
-        elif p==np.inf:
+        if eps == 0:
+            epsfac = 1
+        elif p == np.inf:
             epsfac = 1/(1+eps)
         else:
             epsfac = 1/(1+eps)**p
 
-        if p!=np.inf and distance_upper_bound!=np.inf:
+        if p != np.inf and distance_upper_bound != np.inf:
             distance_upper_bound = distance_upper_bound**p
 
         while q:
@@ -277,21 +348,21 @@ class KDTree(object):
                 data = self.data[node.idx]
                 ds = minkowski_distance_p(data,x[np.newaxis,:],p)
                 for i in range(len(ds)):
-                    if ds[i]<distance_upper_bound:
-                        if len(neighbors)==k:
+                    if ds[i] < distance_upper_bound:
+                        if len(neighbors) == k:
                             heappop(neighbors)
                         heappush(neighbors, (-ds[i], node.idx[i]))
-                        if len(neighbors)==k:
+                        if len(neighbors) == k:
                             distance_upper_bound = -neighbors[0][0]
             else:
                 # we don't push cells that are too far onto the queue at all,
                 # but since the distance_upper_bound decreases, we might get
                 # here even if the cell's too far
-                if min_distance>distance_upper_bound*epsfac:
+                if min_distance > distance_upper_bound*epsfac:
                     # since this is the nearest cell, we're done, bail out
                     break
                 # compute minimum distances to the children and push them on
-                if x[node.split_dim]<node.split:
+                if x[node.split_dim] < node.split:
                     near, far = node.less, node.greater
                 else:
                     near, far = node.greater, node.less
@@ -312,10 +383,10 @@ class KDTree(object):
                     min_distance = min_distance - side_distances[node.split_dim] + sd[node.split_dim]
 
                 # far child might be too far, if so, don't bother pushing it
-                if min_distance<=distance_upper_bound*epsfac:
+                if min_distance <= distance_upper_bound*epsfac:
                     heappush(q,(min_distance, tuple(sd), far))
 
-        if p==np.inf:
+        if p == np.inf:
             return sorted([(-d,i) for (d,i) in neighbors])
         else:
             return sorted([((-d)**(1./p),i) for (d,i) in neighbors])
@@ -361,9 +432,9 @@ class KDTree(object):
 
         Examples
         --------
-        >>> from scipy.spatial import KDTree
+        >>> from scipy import spatial
         >>> x, y = np.mgrid[0:5, 2:8]
-        >>> tree = KDTree(zip(x.ravel(), y.ravel()))
+        >>> tree = spatial.KDTree(zip(x.ravel(), y.ravel()))
         >>> tree.data
         array([[0, 2],
                [0, 3],
@@ -403,19 +474,19 @@ class KDTree(object):
         x = np.asarray(x)
         if np.shape(x)[-1] != self.m:
             raise ValueError("x must consist of vectors of length %d but has shape %s" % (self.m, np.shape(x)))
-        if p<1:
+        if p < 1:
             raise ValueError("Only p-norms with 1<=p<=infinity permitted")
         retshape = np.shape(x)[:-1]
-        if retshape!=():
+        if retshape != ():
             if k is None:
                 dd = np.empty(retshape,dtype=np.object)
                 ii = np.empty(retshape,dtype=np.object)
-            elif k>1:
+            elif k > 1:
                 dd = np.empty(retshape+(k,),dtype=np.float)
                 dd.fill(np.inf)
                 ii = np.empty(retshape+(k,),dtype=np.int)
                 ii.fill(self.n)
-            elif k==1:
+            elif k == 1:
                 dd = np.empty(retshape,dtype=np.float)
                 dd.fill(np.inf)
                 ii = np.empty(retshape,dtype=np.int)
@@ -423,30 +494,30 @@ class KDTree(object):
             else:
                 raise ValueError("Requested %s nearest neighbors; acceptable numbers are integers greater than or equal to one, or None")
             for c in np.ndindex(retshape):
-                hits = self.__query(x[c], k=k, p=p, distance_upper_bound=distance_upper_bound)
+                hits = self.__query(x[c], k=k, eps=eps, p=p, distance_upper_bound=distance_upper_bound)
                 if k is None:
                     dd[c] = [d for (d,i) in hits]
                     ii[c] = [i for (d,i) in hits]
-                elif k>1:
+                elif k > 1:
                     for j in range(len(hits)):
                         dd[c+(j,)], ii[c+(j,)] = hits[j]
-                elif k==1:
-                    if len(hits)>0:
+                elif k == 1:
+                    if len(hits) > 0:
                         dd[c], ii[c] = hits[0]
                     else:
                         dd[c] = np.inf
                         ii[c] = self.n
             return dd, ii
         else:
-            hits = self.__query(x, k=k, p=p, distance_upper_bound=distance_upper_bound)
+            hits = self.__query(x, k=k, eps=eps, p=p, distance_upper_bound=distance_upper_bound)
             if k is None:
                 return [d for (d,i) in hits], [i for (d,i) in hits]
-            elif k==1:
-                if len(hits)>0:
+            elif k == 1:
+                if len(hits) > 0:
                     return hits[0]
                 else:
                     return np.inf, self.n
-            elif k>1:
+            elif k > 1:
                 dd = np.empty(k,dtype=np.float)
                 dd.fill(np.inf)
                 ii = np.empty(k,dtype=np.int)
@@ -457,7 +528,6 @@ class KDTree(object):
             else:
                 raise ValueError("Requested %s nearest neighbors; acceptable numbers are integers greater than or equal to one, or None")
 
-
     def __query_ball_point(self, x, r, p=2., eps=0):
         R = Rectangle(self.maxes, self.mins)
 
@@ -525,7 +595,7 @@ class KDTree(object):
         """
         x = np.asarray(x)
         if x.shape[-1] != self.m:
-            raise ValueError("Searching for a %d-dimensional point in a " \
+            raise ValueError("Searching for a %d-dimensional point in a "
                              "%d-dimensional KDTree" % (x.shape[-1], self.m))
         if len(x.shape) == 1:
             return self.__query_ball_point(x, r, p, eps)
@@ -540,37 +610,39 @@ class KDTree(object):
         """Find all pairs of points whose distance is at most r
 
         Parameters
-        ==========
-
-        other : KDTree
-            The tree containing points to search against
-        r : positive float
-            The maximum distance
-        p : float 1<=p<=infinity
-            Which Minkowski norm to use
-        eps : nonnegative float
-            Approximate search. Branches of the tree are not explored
-            if their nearest points are further than r/(1+eps), and branches
-            are added in bulk if their furthest points are nearer than r*(1+eps).
+        ----------
+        other : KDTree instance
+            The tree containing points to search against.
+        r : float
+            The maximum distance, has to be positive.
+        p : float, optional
+            Which Minkowski norm to use.  `p` has to meet the condition
+            ``1 <= p <= infinity``.
+        eps : float, optional
+            Approximate search.  Branches of the tree are not explored
+            if their nearest points are further than ``r/(1+eps)``, and
+            branches are added in bulk if their furthest points are nearer
+            than ``r * (1+eps)``.  `eps` has to be non-negative.
 
         Returns
-        =======
-
+        -------
         results : list of lists
-            For each element self.data[i] of this tree, results[i] is a list of the
-            indices of its neighbors in other.data.
+            For each element ``self.data[i]`` of this tree, ``results[i]`` is a
+            list of the indices of its neighbors in ``other.data``.
+
         """
         results = [[] for i in range(self.n)]
+
         def traverse_checking(node1, rect1, node2, rect2):
-            if rect1.min_distance_rectangle(rect2, p)>r/(1.+eps):
+            if rect1.min_distance_rectangle(rect2, p) > r/(1.+eps):
                 return
-            elif rect1.max_distance_rectangle(rect2, p)<r*(1.+eps):
+            elif rect1.max_distance_rectangle(rect2, p) < r*(1.+eps):
                 traverse_no_checking(node1, node2)
             elif isinstance(node1, KDTree.leafnode):
                 if isinstance(node2, KDTree.leafnode):
                     d = other.data[node2.idx]
                     for i in node1.idx:
-                        results[i] += node2.idx[minkowski_distance(d,self.data[i],p)<=r].tolist()
+                        results[i] += node2.idx[minkowski_distance(d,self.data[i],p) <= r].tolist()
                 else:
                     less, greater = rect2.split(node2.split_dim, node2.split)
                     traverse_checking(node1,rect1,node2.less,less)
@@ -604,55 +676,53 @@ class KDTree(object):
         return results
 
     def query_pairs(self, r, p=2., eps=0):
-        """Find all pairs of points whose distance is at most r
+        """
+        Find all pairs of points within a distance.
 
         Parameters
-        ==========
-
+        ----------
         r : positive float
-            The maximum distance
-        p : float 1<=p<=infinity
-            Which Minkowski norm to use
-        eps : nonnegative float
-            Approximate search. Branches of the tree are not explored
-            if their nearest points are further than r/(1+eps), and branches
-            are added in bulk if their furthest points are nearer than r*(1+eps).
+            The maximum distance.
+        p : float, optional
+            Which Minkowski norm to use.  `p` has to meet the condition
+            ``1 <= p <= infinity``.
+        eps : float, optional
+            Approximate search.  Branches of the tree are not explored
+            if their nearest points are further than ``r/(1+eps)``, and
+            branches are added in bulk if their furthest points are nearer
+            than ``r * (1+eps)``.  `eps` has to be non-negative.
 
         Returns
-        =======
-
+        -------
         results : set
-            set of pairs (i,j), i<j, for which the corresponing positions are
-            close.
+            Set of pairs ``(i,j)``, with ``i < j``, for which the corresponding
+            positions are close.
 
         """
         results = set()
-        visited = set()
-        def test_set_visited(node1, node2):
-            i, j = sorted((id(node1),id(node2)))
-            if (i,j) in visited:
-                return True
-            else:
-                visited.add((i,j))
-                return False
+
         def traverse_checking(node1, rect1, node2, rect2):
-            if test_set_visited(node1, node2):
+            if rect1.min_distance_rectangle(rect2, p) > r/(1.+eps):
                 return
-
-            if id(node2)<id(node1):
-                # This node pair will be visited in the other order
-                #return
-                pass
-
-            if isinstance(node1, KDTree.leafnode):
+            elif rect1.max_distance_rectangle(rect2, p) < r*(1.+eps):
+                traverse_no_checking(node1, node2)
+            elif isinstance(node1, KDTree.leafnode):
                 if isinstance(node2, KDTree.leafnode):
-                    d = self.data[node2.idx]
-                    for i in node1.idx:
-                        for j in node2.idx[minkowski_distance(d,self.data[i],p)<=r]:
-                            if i<j:
-                                results.add((i,j))
-                            elif j<i:
-                                results.add((j,i))
+                    # Special care to avoid duplicate pairs
+                    if id(node1) == id(node2):
+                        d = self.data[node2.idx]
+                        for i in node1.idx:
+                            for j in node2.idx[minkowski_distance(d,self.data[i],p) <= r]:
+                                if i < j:
+                                    results.add((i,j))
+                    else:
+                        d = self.data[node2.idx]
+                        for i in node1.idx:
+                            for j in node2.idx[minkowski_distance(d,self.data[i],p) <= r]:
+                                if i < j:
+                                    results.add((i,j))
+                                elif j < i:
+                                    results.add((j,i))
                 else:
                     less, greater = rect2.split(node2.split_dim, node2.split)
                     traverse_checking(node1,rect1,node2.less,less)
@@ -661,82 +731,92 @@ class KDTree(object):
                 less, greater = rect1.split(node1.split_dim, node1.split)
                 traverse_checking(node1.less,less,node2,rect2)
                 traverse_checking(node1.greater,greater,node2,rect2)
-            elif rect1.min_distance_rectangle(rect2, p)>r/(1.+eps):
-                return
-            elif rect1.max_distance_rectangle(rect2, p)<r*(1.+eps):
-                traverse_no_checking(node1.less, node2)
-                traverse_no_checking(node1.greater, node2)
             else:
                 less1, greater1 = rect1.split(node1.split_dim, node1.split)
                 less2, greater2 = rect2.split(node2.split_dim, node2.split)
                 traverse_checking(node1.less,less1,node2.less,less2)
                 traverse_checking(node1.less,less1,node2.greater,greater2)
-                traverse_checking(node1.greater,greater1,node2.less,less2)
+
+                # Avoid traversing (node1.less, node2.greater) and
+                # (node1.greater, node2.less) (it's the same node pair twice
+                # over, which is the source of the complication in the
+                # original KDTree.query_pairs)
+                if id(node1) != id(node2):
+                    traverse_checking(node1.greater,greater1,node2.less,less2)
+
                 traverse_checking(node1.greater,greater1,node2.greater,greater2)
 
         def traverse_no_checking(node1, node2):
-            if test_set_visited(node1, node2):
-                return
-
-            if id(node2)<id(node1):
-                # This node pair will be visited in the other order
-                #return
-                pass
             if isinstance(node1, KDTree.leafnode):
                 if isinstance(node2, KDTree.leafnode):
-                    for i in node1.idx:
-                        for j in node2.idx:
-                            if i<j:
-                                results.add((i,j))
-                            elif j<i:
-                                results.add((j,i))
+                    # Special care to avoid duplicate pairs
+                    if id(node1) == id(node2):
+                        for i in node1.idx:
+                            for j in node2.idx:
+                                if i < j:
+                                    results.add((i,j))
+                    else:
+                        for i in node1.idx:
+                            for j in node2.idx:
+                                if i < j:
+                                    results.add((i,j))
+                                elif j < i:
+                                    results.add((j,i))
                 else:
                     traverse_no_checking(node1, node2.less)
                     traverse_no_checking(node1, node2.greater)
             else:
-                traverse_no_checking(node1.less, node2)
-                traverse_no_checking(node1.greater, node2)
+                # Avoid traversing (node1.less, node2.greater) and
+                # (node1.greater, node2.less) (it's the same node pair twice
+                # over, which is the source of the complication in the
+                # original KDTree.query_pairs)
+                if id(node1) == id(node2):
+                    traverse_no_checking(node1.less, node2.less)
+                    traverse_no_checking(node1.less, node2.greater)
+                    traverse_no_checking(node1.greater, node2.greater)
+                else:
+                    traverse_no_checking(node1.less, node2)
+                    traverse_no_checking(node1.greater, node2)
 
         traverse_checking(self.tree, Rectangle(self.maxes, self.mins),
                           self.tree, Rectangle(self.maxes, self.mins))
         return results
 
-
     def count_neighbors(self, other, r, p=2.):
-        """Count how many nearby pairs can be formed.
+        """
+        Count how many nearby pairs can be formed.
 
         Count the number of pairs (x1,x2) can be formed, with x1 drawn
-        from self and x2 drawn from other, and where distance(x1,x2,p)<=r.
+        from self and x2 drawn from `other`, and where
+        ``distance(x1, x2, p) <= r``.
         This is the "two-point correlation" described in Gray and Moore 2000,
         "N-body problems in statistical learning", and the code here is based
         on their algorithm.
 
         Parameters
-        ==========
-
-        other : KDTree
-
+        ----------
+        other : KDTree instance
+            The other tree to draw points from.
         r : float or one-dimensional array of floats
-            The radius to produce a count for. Multiple radii are searched with a single
-            tree traversal.
+            The radius to produce a count for. Multiple radii are searched with
+            a single tree traversal.
         p : float, 1<=p<=infinity
             Which Minkowski p-norm to use
 
         Returns
-        =======
+        -------
+        result : int or 1-D array of ints
+            The number of pairs. Note that this is internally stored in a numpy
+            int, and so may overflow if very large (2e9).
 
-        result : integer or one-dimensional array of integers
-            The number of pairs. Note that this is internally stored in a numpy int,
-            and so may overflow if very large (two billion).
         """
-
         def traverse(node1, rect1, node2, rect2, idx):
             min_r = rect1.min_distance_rectangle(rect2,p)
             max_r = rect1.max_distance_rectangle(rect2,p)
-            c_greater = r[idx]>max_r
+            c_greater = r[idx] > max_r
             result[idx[c_greater]] += node1.children*node2.children
-            idx = idx[(min_r<=r[idx]) & (r[idx]<=max_r)]
-            if len(idx)==0:
+            idx = idx[(min_r <= r[idx]) & (r[idx] <= max_r)]
+            if len(idx) == 0:
                 return
 
             if isinstance(node1,KDTree.leafnode):
@@ -762,6 +842,7 @@ class KDTree(object):
                     traverse(node1.less,less1,node2.greater,greater2,idx)
                     traverse(node1.greater,greater1,node2.less,less2,idx)
                     traverse(node1.greater,greater1,node2.greater,greater2,idx)
+
         R1 = Rectangle(self.maxes, self.mins)
         R2 = Rectangle(other.maxes, other.mins)
         if np.shape(r) == ():
@@ -769,7 +850,7 @@ class KDTree(object):
             result = np.zeros(1,dtype=int)
             traverse(self.tree, R1, other.tree, R2, np.arange(1))
             return result[0]
-        elif len(np.shape(r))==1:
+        elif len(np.shape(r)) == 1:
             r = np.asarray(r)
             n, = r.shape
             result = np.zeros(n,dtype=int)
@@ -779,35 +860,37 @@ class KDTree(object):
             raise ValueError("r must be either a single value or a one-dimensional array of values")
 
     def sparse_distance_matrix(self, other, max_distance, p=2.):
-        """Compute a sparse distance matrix
+        """
+        Compute a sparse distance matrix
 
         Computes a distance matrix between two KDTrees, leaving as zero
         any distance greater than max_distance.
 
         Parameters
-        ==========
-
+        ----------
         other : KDTree
 
         max_distance : positive float
 
-        Returns
-        =======
+        p : float, optional
 
+        Returns
+        -------
         result : dok_matrix
             Sparse matrix representing the results in "dictionary of keys" format.
+
         """
         result = scipy.sparse.dok_matrix((self.n,other.n))
 
         def traverse(node1, rect1, node2, rect2):
-            if rect1.min_distance_rectangle(rect2, p)>max_distance:
+            if rect1.min_distance_rectangle(rect2, p) > max_distance:
                 return
             elif isinstance(node1, KDTree.leafnode):
                 if isinstance(node2, KDTree.leafnode):
                     for i in node1.idx:
                         for j in node2.idx:
                             d = minkowski_distance(self.data[i],other.data[j],p)
-                            if d<=max_distance:
+                            if d <= max_distance:
                                 result[i,j] = d
                 else:
                     less, greater = rect2.split(node2.split_dim, node2.split)
@@ -838,19 +921,20 @@ def distance_matrix(x,y,p=2,threshold=1000000):
 
     Parameters
     ----------
-    x : array_like, `M` by `K`
+    x : (M, K) array_like
         TODO: description needed
-    y : array_like, `N` by `K`
+    y : (N, K) array_like
         TODO: description needed
     p : float, 1 <= p <= infinity
         Which Minkowski p-norm to use.
-    threshold : positive integer
-        If `M * N * K` > threshold, use a Python loop instead of creating
-        a very large temporary [what?  array?].
+    threshold : positive int
+        If ``M * N * K`` > `threshold`, algorithm uses a Python loop instead
+        of large temporary arrays.
 
     Returns
     -------
-    result : array_like, `M` by `N`
+    result : (M, N) ndarray
+        Distance matrix.
 
     Examples
     --------
@@ -871,8 +955,8 @@ def distance_matrix(x,y,p=2,threshold=1000000):
     if m*n*k <= threshold:
         return minkowski_distance(x[:,np.newaxis,:],y[np.newaxis,:,:],p)
     else:
-        result = np.empty((m,n),dtype=np.float) #FIXME: figure out the best dtype
-        if m<n:
+        result = np.empty((m,n),dtype=np.float)  # FIXME: figure out the best dtype
+        if m < n:
             for i in range(m):
                 result[i,:] = minkowski_distance(x[i],y,p)
         else:
