@@ -33,19 +33,24 @@
 # NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 # SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 
+from __future__ import division, print_function, absolute_import
 
 import os.path
+from scipy.lib.six.moves import xrange
 
 import numpy as np
 from numpy.linalg import norm
-from numpy.testing import verbose, TestCase, run_module_suite, \
-        assert_raises, assert_array_equal, assert_equal, assert_almost_equal
+from numpy.testing import (verbose, TestCase, run_module_suite,
+        assert_raises, assert_array_equal, assert_equal, assert_almost_equal,
+        assert_allclose)
 
-from scipy.spatial.distance import squareform, pdist, cdist, matching, \
-        jaccard, dice, sokalsneath, rogerstanimoto, russellrao, yule, \
-        num_obs_y, num_obs_dm, is_valid_dm, is_valid_y, minkowski, wminkowski, \
-        euclidean, sqeuclidean, cosine, correlation, mahalanobis, \
-        canberra, braycurtis, sokalmichener, _validate_vector
+from scipy.lib.six import u
+
+from scipy.spatial.distance import (squareform, pdist, cdist, matching,
+        jaccard, dice, sokalsneath, rogerstanimoto, russellrao, yule,
+        num_obs_y, num_obs_dm, is_valid_dm, is_valid_y, minkowski, wminkowski,
+        euclidean, sqeuclidean, cosine, correlation, mahalanobis,
+        canberra, braycurtis, sokalmichener, _validate_vector)
 
 
 _filenames = ["iris.txt",
@@ -73,12 +78,12 @@ _filenames = ["iris.txt",
               "pdist-chebychev-ml-iris.txt",
               "random-bool-data.txt"]
 
-_tdist = np.array([[0,    662,  877,  255,  412,  996],
-                      [662,  0,    295,  468,  268,  400],
-                      [877,  295,  0,    754,  564,  138],
-                      [255,  468,  754,  0,    219,  869],
-                      [412,  268,  564,  219,  0,    669],
-                      [996,  400,  138,  869,  669,  0  ]], dtype='double')
+_tdist = np.array([[0, 662, 877, 255, 412, 996],
+                      [662, 0, 295, 468, 268, 400],
+                      [877, 295, 0, 754, 564, 138],
+                      [255, 468, 754, 0, 219, 869],
+                      [412, 268, 564, 219, 0, 669],
+                      [996, 400, 138, 869, 669, 0]], dtype='double')
 
 _ytdist = squareform(_tdist)
 
@@ -87,11 +92,12 @@ _ytdist = squareform(_tdist)
 
 eo = {}
 
+
 def load_testing_files():
     "Loading test data files for the scipy.spatial.distance tests."
     for fn in _filenames:
         name = fn.replace(".txt", "").replace("-ml", "")
-        fqfn = os.path.join(os.path.dirname(__file__), fn)
+        fqfn = os.path.join(os.path.dirname(__file__), 'data', fn)
         fp = open(fqfn)
         eo[name] = np.loadtxt(fp)
         fp.close()
@@ -115,7 +121,7 @@ class TestCdist(TestCase):
         Y1 = cdist(X1, X2, 'euclidean')
         Y2 = cdist(X1, X2, 'test_euclidean')
         if verbose > 2:
-            print (Y1-Y2).max()
+            print((Y1-Y2).max())
         self.assertTrue(within_tol(Y1, Y2, eps))
 
     def test_cdist_euclidean_random_unicode(self):
@@ -124,10 +130,10 @@ class TestCdist(TestCase):
         # Get the data: the input matrix and the right output.
         X1 = eo['cdist-X1']
         X2 = eo['cdist-X2']
-        Y1 = cdist(X1, X2, u'euclidean')
-        Y2 = cdist(X1, X2, u'test_euclidean')
+        Y1 = cdist(X1, X2, u('euclidean'))
+        Y2 = cdist(X1, X2, u('test_euclidean'))
         if verbose > 2:
-            print (Y1-Y2).max()
+            print((Y1-Y2).max())
         self.assertTrue(within_tol(Y1, Y2, eps))
 
     def test_cdist_sqeuclidean_random(self):
@@ -139,7 +145,7 @@ class TestCdist(TestCase):
         Y1 = cdist(X1, X2, 'sqeuclidean')
         Y2 = cdist(X1, X2, 'test_sqeuclidean')
         if verbose > 2:
-            print (Y1-Y2).max()
+            print((Y1-Y2).max())
         self.assertTrue(within_tol(Y1, Y2, eps))
 
     def test_cdist_cityblock_random(self):
@@ -151,7 +157,7 @@ class TestCdist(TestCase):
         Y1 = cdist(X1, X2, 'cityblock')
         Y2 = cdist(X1, X2, 'test_cityblock')
         if verbose > 2:
-            print (Y1-Y2).max()
+            print((Y1-Y2).max())
         self.assertTrue(within_tol(Y1, Y2, eps))
 
     def test_cdist_hamming_double_random(self):
@@ -163,7 +169,7 @@ class TestCdist(TestCase):
         Y1 = cdist(X1, X2, 'hamming')
         Y2 = cdist(X1, X2, 'test_hamming')
         if verbose > 2:
-            print (Y1-Y2).max()
+            print((Y1-Y2).max())
         self.assertTrue(within_tol(Y1, Y2, eps))
 
     def test_cdist_hamming_bool_random(self):
@@ -175,7 +181,7 @@ class TestCdist(TestCase):
         Y1 = cdist(X1, X2, 'hamming')
         Y2 = cdist(X1, X2, 'test_hamming')
         if verbose > 2:
-            print (Y1-Y2).max()
+            print((Y1-Y2).max())
         self.assertTrue(within_tol(Y1, Y2, eps))
 
     def test_cdist_jaccard_double_random(self):
@@ -187,7 +193,7 @@ class TestCdist(TestCase):
         Y1 = cdist(X1, X2, 'jaccard')
         Y2 = cdist(X1, X2, 'test_jaccard')
         if verbose > 2:
-            print (Y1-Y2).max()
+            print((Y1-Y2).max())
         self.assertTrue(within_tol(Y1, Y2, eps))
 
     def test_cdist_jaccard_bool_random(self):
@@ -199,7 +205,7 @@ class TestCdist(TestCase):
         Y1 = cdist(X1, X2, 'jaccard')
         Y2 = cdist(X1, X2, 'test_jaccard')
         if verbose > 2:
-            print (Y1-Y2).max()
+            print((Y1-Y2).max())
         self.assertTrue(within_tol(Y1, Y2, eps))
 
     def test_cdist_chebychev_random(self):
@@ -211,7 +217,7 @@ class TestCdist(TestCase):
         Y1 = cdist(X1, X2, 'chebychev')
         Y2 = cdist(X1, X2, 'test_chebychev')
         if verbose > 2:
-            print (Y1-Y2).max()
+            print((Y1-Y2).max())
         self.assertTrue(within_tol(Y1, Y2, eps))
 
     def test_cdist_minkowski_random_p3d8(self):
@@ -223,7 +229,7 @@ class TestCdist(TestCase):
         Y1 = cdist(X1, X2, 'minkowski', p=3.8)
         Y2 = cdist(X1, X2, 'test_minkowski', p=3.8)
         if verbose > 2:
-            print (Y1-Y2).max()
+            print((Y1-Y2).max())
         self.assertTrue(within_tol(Y1, Y2, eps))
 
     def test_cdist_minkowski_random_p4d6(self):
@@ -235,7 +241,7 @@ class TestCdist(TestCase):
         Y1 = cdist(X1, X2, 'minkowski', p=4.6)
         Y2 = cdist(X1, X2, 'test_minkowski', p=4.6)
         if verbose > 2:
-            print (Y1-Y2).max()
+            print((Y1-Y2).max())
         self.assertTrue(within_tol(Y1, Y2, eps))
 
     def test_cdist_minkowski_random_p1d23(self):
@@ -247,10 +253,9 @@ class TestCdist(TestCase):
         Y1 = cdist(X1, X2, 'minkowski', p=1.23)
         Y2 = cdist(X1, X2, 'test_minkowski', p=1.23)
         if verbose > 2:
-            print (Y1-Y2).max()
+            print((Y1-Y2).max())
         self.assertTrue(within_tol(Y1, Y2, eps))
 
-
     def test_cdist_wminkowski_random_p3d8(self):
         "Tests cdist(X, 'wminkowski') on random data. (p=3.8)"
         eps = 1e-07
@@ -261,7 +266,7 @@ class TestCdist(TestCase):
         Y1 = cdist(X1, X2, 'wminkowski', p=3.8, w=w)
         Y2 = cdist(X1, X2, 'test_wminkowski', p=3.8, w=w)
         if verbose > 2:
-            print (Y1-Y2).max()
+            print((Y1-Y2).max())
         self.assertTrue(within_tol(Y1, Y2, eps))
 
     def test_cdist_wminkowski_random_p4d6(self):
@@ -274,7 +279,7 @@ class TestCdist(TestCase):
         Y1 = cdist(X1, X2, 'wminkowski', p=4.6, w=w)
         Y2 = cdist(X1, X2, 'test_wminkowski', p=4.6, w=w)
         if verbose > 2:
-            print (Y1-Y2).max()
+            print((Y1-Y2).max())
         self.assertTrue(within_tol(Y1, Y2, eps))
 
     def test_cdist_wminkowski_random_p1d23(self):
@@ -287,10 +292,9 @@ class TestCdist(TestCase):
         Y1 = cdist(X1, X2, 'wminkowski', p=1.23, w=w)
         Y2 = cdist(X1, X2, 'test_wminkowski', p=1.23, w=w)
         if verbose > 2:
-            print (Y1-Y2).max()
+            print((Y1-Y2).max())
         self.assertTrue(within_tol(Y1, Y2, eps))
 
-
     def test_cdist_seuclidean_random(self):
         "Tests cdist(X, 'seuclidean') on random data."
         eps = 1e-07
@@ -300,10 +304,9 @@ class TestCdist(TestCase):
         Y1 = cdist(X1, X2, 'seuclidean')
         Y2 = cdist(X1, X2, 'test_seuclidean')
         if verbose > 2:
-            print (Y1-Y2).max()
+            print((Y1-Y2).max())
         self.assertTrue(within_tol(Y1, Y2, eps))
 
-
     def test_cdist_cosine_random(self):
         "Tests cdist(X, 'cosine') on random data."
         eps = 1e-07
@@ -313,7 +316,7 @@ class TestCdist(TestCase):
         Y1 = cdist(X1, X2, 'cosine')
         Y2 = cdist(X1, X2, 'test_cosine')
         if verbose > 2:
-            print (Y1-Y2).max()
+            print((Y1-Y2).max())
         self.assertTrue(within_tol(Y1, Y2, eps))
 
     def test_cdist_correlation_random(self):
@@ -325,7 +328,7 @@ class TestCdist(TestCase):
         Y1 = cdist(X1, X2, 'correlation')
         Y2 = cdist(X1, X2, 'test_correlation')
         if verbose > 2:
-            print (Y1-Y2).max()
+            print((Y1-Y2).max())
         self.assertTrue(within_tol(Y1, Y2, eps))
 
     def test_cdist_mahalanobis_random(self):
@@ -337,7 +340,7 @@ class TestCdist(TestCase):
         Y1 = cdist(X1, X2, 'mahalanobis')
         Y2 = cdist(X1, X2, 'test_mahalanobis')
         if verbose > 2:
-            print (Y1-Y2).max()
+            print((Y1-Y2).max())
         self.assertTrue(within_tol(Y1, Y2, eps))
 
     def test_cdist_canberra_random(self):
@@ -349,7 +352,7 @@ class TestCdist(TestCase):
         Y1 = cdist(X1, X2, 'canberra')
         Y2 = cdist(X1, X2, 'test_canberra')
         if verbose > 2:
-            print (Y1-Y2).max()
+            print((Y1-Y2).max())
         self.assertTrue(within_tol(Y1, Y2, eps))
 
     def test_cdist_braycurtis_random(self):
@@ -361,8 +364,8 @@ class TestCdist(TestCase):
         Y1 = cdist(X1, X2, 'braycurtis')
         Y2 = cdist(X1, X2, 'test_braycurtis')
         if verbose > 2:
-            print Y1, Y2
-            print (Y1-Y2).max()
+            print(Y1, Y2)
+            print((Y1-Y2).max())
         self.assertTrue(within_tol(Y1, Y2, eps))
 
     def test_cdist_yule_random(self):
@@ -374,7 +377,7 @@ class TestCdist(TestCase):
         Y1 = cdist(X1, X2, 'yule')
         Y2 = cdist(X1, X2, 'test_yule')
         if verbose > 2:
-            print (Y1-Y2).max()
+            print((Y1-Y2).max())
         self.assertTrue(within_tol(Y1, Y2, eps))
 
     def test_cdist_matching_random(self):
@@ -386,7 +389,7 @@ class TestCdist(TestCase):
         Y1 = cdist(X1, X2, 'matching')
         Y2 = cdist(X1, X2, 'test_matching')
         if verbose > 2:
-            print (Y1-Y2).max()
+            print((Y1-Y2).max())
         self.assertTrue(within_tol(Y1, Y2, eps))
 
     def test_cdist_kulsinski_random(self):
@@ -398,7 +401,7 @@ class TestCdist(TestCase):
         Y1 = cdist(X1, X2, 'kulsinski')
         Y2 = cdist(X1, X2, 'test_kulsinski')
         if verbose > 2:
-            print (Y1-Y2).max()
+            print((Y1-Y2).max())
         self.assertTrue(within_tol(Y1, Y2, eps))
 
     def test_cdist_dice_random(self):
@@ -410,7 +413,7 @@ class TestCdist(TestCase):
         Y1 = cdist(X1, X2, 'dice')
         Y2 = cdist(X1, X2, 'test_dice')
         if verbose > 2:
-            print (Y1-Y2).max()
+            print((Y1-Y2).max())
         self.assertTrue(within_tol(Y1, Y2, eps))
 
     def test_cdist_rogerstanimoto_random(self):
@@ -422,7 +425,7 @@ class TestCdist(TestCase):
         Y1 = cdist(X1, X2, 'rogerstanimoto')
         Y2 = cdist(X1, X2, 'test_rogerstanimoto')
         if verbose > 2:
-            print (Y1-Y2).max()
+            print((Y1-Y2).max())
         self.assertTrue(within_tol(Y1, Y2, eps))
 
     def test_cdist_russellrao_random(self):
@@ -434,7 +437,7 @@ class TestCdist(TestCase):
         Y1 = cdist(X1, X2, 'russellrao')
         Y2 = cdist(X1, X2, 'test_russellrao')
         if verbose > 2:
-            print (Y1-Y2).max()
+            print((Y1-Y2).max())
         self.assertTrue(within_tol(Y1, Y2, eps))
 
     def test_cdist_sokalmichener_random(self):
@@ -446,7 +449,7 @@ class TestCdist(TestCase):
         Y1 = cdist(X1, X2, 'sokalmichener')
         Y2 = cdist(X1, X2, 'test_sokalmichener')
         if verbose > 2:
-            print (Y1-Y2).max()
+            print((Y1-Y2).max())
         self.assertTrue(within_tol(Y1, Y2, eps))
 
     def test_cdist_sokalsneath_random(self):
@@ -458,7 +461,7 @@ class TestCdist(TestCase):
         Y1 = cdist(X1, X2, 'sokalsneath')
         Y2 = cdist(X1, X2, 'test_sokalsneath')
         if verbose > 2:
-            print (Y1-Y2).max()
+            print((Y1-Y2).max())
         self.assertTrue(within_tol(Y1, Y2, eps))
 
 
@@ -485,7 +488,7 @@ class TestPdist(TestCase):
         X = eo['pdist-double-inp']
         Y_right = eo['pdist-euclidean']
 
-        Y_test1 = pdist(X, u'euclidean')
+        Y_test1 = pdist(X, u('euclidean'))
         self.assertTrue(within_tol(Y_test1, Y_right, eps))
 
     def test_pdist_euclidean_random_float32(self):
@@ -526,7 +529,7 @@ class TestPdist(TestCase):
 
         Y_test1 = pdist(X, 'euclidean')
         if verbose > 2:
-            print np.abs(Y_right - Y_test1).max()
+            print(np.abs(Y_right - Y_test1).max())
         self.assertTrue(within_tol(Y_test1, Y_right, eps))
 
     def test_pdist_euclidean_iris_nonC(self):
@@ -646,7 +649,7 @@ class TestPdist(TestCase):
 
         Y_test1 = pdist(X, 'cosine')
         if verbose > 2:
-            print np.abs(Y_test1 - Y_right).max()
+            print(np.abs(Y_test1 - Y_right).max())
         self.assertTrue(within_tol(Y_test1, Y_right, eps))
         #print "cosine-iris", np.abs(Y_test1 - Y_right).max()
 
@@ -709,7 +712,7 @@ class TestPdist(TestCase):
 
         Y_test1 = pdist(X, 'cityblock')
         if verbose > 2:
-            print "cityblock-iris-float32", np.abs(Y_test1 - Y_right).max()
+            print("cityblock-iris-float32", np.abs(Y_test1 - Y_right).max())
         self.assertTrue(within_tol(Y_test1, Y_right, eps))
 
     def test_pdist_cityblock_iris_nonC(self):
@@ -773,7 +776,7 @@ class TestPdist(TestCase):
 
         Y_test1 = pdist(X, 'correlation')
         if verbose > 2:
-            print "correlation-iris", np.abs(Y_test1 - Y_right).max()
+            print("correlation-iris", np.abs(Y_test1 - Y_right).max())
         self.assertTrue(within_tol(Y_test1, Y_right, eps))
 
     def test_pdist_correlation_iris_nonC(self):
@@ -867,7 +870,7 @@ class TestPdist(TestCase):
 
         Y_test1 = pdist(X, 'minkowski', 5.8)
         if verbose > 2:
-            print "minkowski-iris-5.8", np.abs(Y_test1 - Y_right).max()
+            print("minkowski-iris-5.8", np.abs(Y_test1 - Y_right).max())
         self.assertTrue(within_tol(Y_test1, Y_right, eps))
 
     def test_pdist_minkowski_5_8_iris_nonC(self):
@@ -894,16 +897,16 @@ class TestPdist(TestCase):
                        1.5, 3.0,
                        2.5]
         dist = pdist(x, metric=wminkowski, w=[1.0, 1.0, 1.0])
-        assert_array_equal(dist, p2_expected)
+        assert_allclose(dist, p2_expected, rtol=1e-14)
 
         dist = pdist(x, metric=wminkowski, w=[0.5, 1.0, 2.0], p=1)
-        assert_array_equal(dist, p1_expected)
+        assert_allclose(dist, p1_expected, rtol=1e-14)
 
         dist = pdist(x, metric='wminkowski', w=[1.0, 1.0, 1.0])
-        assert_array_equal(dist, p2_expected)
+        assert_allclose(dist, p2_expected, rtol=1e-14)
 
         dist = pdist(x, metric='wminkowski', w=[0.5, 1.0, 2.0], p=1)
-        assert_array_equal(dist, p1_expected)
+        assert_allclose(dist, p1_expected, rtol=1e-14)
 
     ################### pdist: hamming
     def test_pdist_hamming_random(self):
@@ -1056,7 +1059,7 @@ class TestPdist(TestCase):
 
         Y_test1 = pdist(X, 'chebychev')
         if verbose > 2:
-            print "chebychev", np.abs(Y_test1 - Y_right).max()
+            print("chebychev", np.abs(Y_test1 - Y_right).max())
         self.assertTrue(within_tol(Y_test1, Y_right, eps))
 
     def test_pdist_chebychev_random_nonC(self):
@@ -1087,7 +1090,7 @@ class TestPdist(TestCase):
         Y_right = eo['pdist-chebychev-iris']
         Y_test1 = pdist(X, 'chebychev')
         if verbose > 2:
-            print "chebychev-iris", np.abs(Y_test1 - Y_right).max()
+            print("chebychev-iris", np.abs(Y_test1 - Y_right).max())
         self.assertTrue(within_tol(Y_test1, Y_right, eps))
 
     def test_pdist_chebychev_iris_nonC(self):
@@ -1123,14 +1126,14 @@ class TestPdist(TestCase):
         D = eo['random-bool-data']
         B = np.bool_(D)
         if verbose > 2:
-            print B.shape, B.dtype
+            print(B.shape, B.dtype)
         eps = 1e-10
         y1 = pdist(B, "matching")
         y2 = pdist(B, "test_matching")
         y3 = pdist(D, "test_matching")
         if verbose > 2:
-            print np.abs(y1-y2).max()
-            print np.abs(y1-y3).max()
+            print(np.abs(y1-y2).max())
+            print(np.abs(y1-y3).max())
         self.assertTrue(within_tol(y1, y2, eps))
         self.assertTrue(within_tol(y2, y3, eps))
 
@@ -1156,14 +1159,14 @@ class TestPdist(TestCase):
         "Tests pdist(X, 'jaccard') to see if the two implementations match on random double input data."
         D = eo['random-bool-data']
         if verbose > 2:
-            print D.shape, D.dtype
+            print(D.shape, D.dtype)
         eps = 1e-10
         y1 = pdist(D, "jaccard")
         y2 = pdist(D, "test_jaccard")
         y3 = pdist(np.bool_(D), "test_jaccard")
         if verbose > 2:
-            print np.abs(y1-y2).max()
-            print np.abs(y2-y3).max()
+            print(np.abs(y1-y2).max())
+            print(np.abs(y2-y3).max())
         self.assertTrue(within_tol(y1, y2, eps))
         self.assertTrue(within_tol(y2, y3, eps))
 
@@ -1174,7 +1177,7 @@ class TestPdist(TestCase):
         m2 = yule(np.array([1, 0, 1, 1, 0], dtype=np.bool),
                   np.array([1, 1, 0, 1, 1], dtype=np.bool))
         if verbose > 2:
-            print m
+            print(m)
         self.assertTrue(np.abs(m - 2.0) <= 1e-10)
         self.assertTrue(np.abs(m2 - 2.0) <= 1e-10)
 
@@ -1185,7 +1188,7 @@ class TestPdist(TestCase):
         m2 = yule(np.array([1, 0, 1], dtype=np.bool),
                   np.array([1, 1, 0], dtype=np.bool))
         if verbose > 2:
-            print m
+            print(m)
         self.assertTrue(np.abs(m - 2.0) <= 1e-10)
         self.assertTrue(np.abs(m2 - 2.0) <= 1e-10)
 
@@ -1193,14 +1196,14 @@ class TestPdist(TestCase):
         "Tests pdist(X, 'yule') to see if the two implementations match on random double input data."
         D = eo['random-bool-data']
         if verbose > 2:
-            print D.shape, D.dtype
+            print(D.shape, D.dtype)
         eps = 1e-10
         y1 = pdist(D, "yule")
         y2 = pdist(D, "test_yule")
         y3 = pdist(np.bool_(D), "test_yule")
         if verbose > 2:
-            print np.abs(y1-y2).max()
-            print np.abs(y2-y3).max()
+            print(np.abs(y1-y2).max())
+            print(np.abs(y2-y3).max())
         self.assertTrue(within_tol(y1, y2, eps))
         self.assertTrue(within_tol(y2, y3, eps))
 
@@ -1211,7 +1214,7 @@ class TestPdist(TestCase):
         m2 = dice(np.array([1, 0, 1, 1, 0], dtype=np.bool),
                   np.array([1, 1, 0, 1, 1], dtype=np.bool))
         if verbose > 2:
-            print m
+            print(m)
         self.assertTrue(np.abs(m - (3.0/7.0)) <= 1e-10)
         self.assertTrue(np.abs(m2 - (3.0/7.0)) <= 1e-10)
 
@@ -1222,7 +1225,7 @@ class TestPdist(TestCase):
         m2 = dice(np.array([1, 0, 1], dtype=np.bool),
                   np.array([1, 1, 0], dtype=np.bool))
         if verbose > 2:
-            print m
+            print(m)
         self.assertTrue(np.abs(m - 0.5) <= 1e-10)
         self.assertTrue(np.abs(m2 - 0.5) <= 1e-10)
 
@@ -1230,14 +1233,14 @@ class TestPdist(TestCase):
         "Tests pdist(X, 'dice') to see if the two implementations match on random double input data."
         D = eo['random-bool-data']
         if verbose > 2:
-            print D.shape, D.dtype
+            print(D.shape, D.dtype)
         eps = 1e-10
         y1 = pdist(D, "dice")
         y2 = pdist(D, "test_dice")
         y3 = pdist(D, "test_dice")
         if verbose > 2:
-            print np.abs(y1-y2).max()
-            print np.abs(y2-y3).max()
+            print(np.abs(y1-y2).max())
+            print(np.abs(y2-y3).max())
         self.assertTrue(within_tol(y1, y2, eps))
         self.assertTrue(within_tol(y2, y3, eps))
 
@@ -1248,7 +1251,7 @@ class TestPdist(TestCase):
         m2 = sokalsneath(np.array([1, 0, 1, 1, 0], dtype=np.bool),
                          np.array([1, 1, 0, 1, 1], dtype=np.bool))
         if verbose > 2:
-            print m
+            print(m)
         self.assertTrue(np.abs(m - (3.0/4.0)) <= 1e-10)
         self.assertTrue(np.abs(m2 - (3.0/4.0)) <= 1e-10)
 
@@ -1259,7 +1262,7 @@ class TestPdist(TestCase):
         m2 = sokalsneath(np.array([1, 0, 1], dtype=np.bool),
                          np.array([1, 1, 0], dtype=np.bool))
         if verbose > 2:
-            print m
+            print(m)
         self.assertTrue(np.abs(m - (4.0/5.0)) <= 1e-10)
         self.assertTrue(np.abs(m2 - (4.0/5.0)) <= 1e-10)
 
@@ -1267,14 +1270,14 @@ class TestPdist(TestCase):
         "Tests pdist(X, 'sokalsneath') to see if the two implementations match on random double input data."
         D = eo['random-bool-data']
         if verbose > 2:
-            print D.shape, D.dtype
+            print(D.shape, D.dtype)
         eps = 1e-10
         y1 = pdist(D, "sokalsneath")
         y2 = pdist(D, "test_sokalsneath")
         y3 = pdist(np.bool_(D), "test_sokalsneath")
         if verbose > 2:
-            print np.abs(y1-y2).max()
-            print np.abs(y2-y3).max()
+            print(np.abs(y1-y2).max())
+            print(np.abs(y2-y3).max())
         self.assertTrue(within_tol(y1, y2, eps))
         self.assertTrue(within_tol(y2, y3, eps))
 
@@ -1285,7 +1288,7 @@ class TestPdist(TestCase):
         m2 = rogerstanimoto(np.array([1, 0, 1, 1, 0], dtype=np.bool),
                             np.array([1, 1, 0, 1, 1], dtype=np.bool))
         if verbose > 2:
-            print m
+            print(m)
         self.assertTrue(np.abs(m - (3.0/4.0)) <= 1e-10)
         self.assertTrue(np.abs(m2 - (3.0/4.0)) <= 1e-10)
 
@@ -1296,7 +1299,7 @@ class TestPdist(TestCase):
         m2 = rogerstanimoto(np.array([1, 0, 1], dtype=np.bool),
                             np.array([1, 1, 0], dtype=np.bool))
         if verbose > 2:
-            print m
+            print(m)
         self.assertTrue(np.abs(m - (4.0/5.0)) <= 1e-10)
         self.assertTrue(np.abs(m2 - (4.0/5.0)) <= 1e-10)
 
@@ -1304,14 +1307,14 @@ class TestPdist(TestCase):
         "Tests pdist(X, 'rogerstanimoto') to see if the two implementations match on random double input data."
         D = eo['random-bool-data']
         if verbose > 2:
-            print D.shape, D.dtype
+            print(D.shape, D.dtype)
         eps = 1e-10
         y1 = pdist(D, "rogerstanimoto")
         y2 = pdist(D, "test_rogerstanimoto")
         y3 = pdist(np.bool_(D), "test_rogerstanimoto")
         if verbose > 2:
-            print np.abs(y1-y2).max()
-            print np.abs(y2-y3).max()
+            print(np.abs(y1-y2).max())
+            print(np.abs(y2-y3).max())
         self.assertTrue(within_tol(y1, y2, eps))
         self.assertTrue(within_tol(y2, y3, eps))
 
@@ -1322,7 +1325,7 @@ class TestPdist(TestCase):
         m2 = russellrao(np.array([1, 0, 1, 1, 0], dtype=np.bool),
                         np.array([1, 1, 0, 1, 1], dtype=np.bool))
         if verbose > 2:
-            print m
+            print(m)
         self.assertTrue(np.abs(m - (3.0/5.0)) <= 1e-10)
         self.assertTrue(np.abs(m2 - (3.0/5.0)) <= 1e-10)
 
@@ -1333,7 +1336,7 @@ class TestPdist(TestCase):
         m2 = russellrao(np.array([1, 0, 1], dtype=np.bool),
                         np.array([1, 1, 0], dtype=np.bool))
         if verbose > 2:
-            print m
+            print(m)
         self.assertTrue(np.abs(m - (2.0/3.0)) <= 1e-10)
         self.assertTrue(np.abs(m2 - (2.0/3.0)) <= 1e-10)
 
@@ -1341,14 +1344,14 @@ class TestPdist(TestCase):
         "Tests pdist(X, 'russellrao') to see if the two implementations match on random double input data."
         D = eo['random-bool-data']
         if verbose > 2:
-            print D.shape, D.dtype
+            print(D.shape, D.dtype)
         eps = 1e-10
         y1 = pdist(D, "russellrao")
         y2 = pdist(D, "test_russellrao")
         y3 = pdist(np.bool_(D), "test_russellrao")
         if verbose > 2:
-            print np.abs(y1-y2).max()
-            print np.abs(y2-y3).max()
+            print(np.abs(y1-y2).max())
+            print(np.abs(y2-y3).max())
         self.assertTrue(within_tol(y1, y2, eps))
         self.assertTrue(within_tol(y2, y3, eps))
 
@@ -1356,14 +1359,14 @@ class TestPdist(TestCase):
         "Tests pdist(X, 'sokalmichener') to see if the two implementations match on random double input data."
         D = eo['random-bool-data']
         if verbose > 2:
-            print D.shape, D.dtype
+            print(D.shape, D.dtype)
         eps = 1e-10
         y1 = pdist(D, "sokalmichener")
         y2 = pdist(D, "test_sokalmichener")
         y3 = pdist(np.bool_(D), "test_sokalmichener")
         if verbose > 2:
-            print np.abs(y1-y2).max()
-            print np.abs(y2-y3).max()
+            print(np.abs(y1-y2).max())
+            print(np.abs(y2-y3).max())
         self.assertTrue(within_tol(y1, y2, eps))
         self.assertTrue(within_tol(y2, y3, eps))
 
@@ -1371,25 +1374,25 @@ class TestPdist(TestCase):
         "Tests pdist(X, 'kulsinski') to see if the two implementations match on random double input data."
         D = eo['random-bool-data']
         if verbose > 2:
-            print D.shape, D.dtype
+            print(D.shape, D.dtype)
         eps = 1e-10
         y1 = pdist(D, "kulsinski")
         y2 = pdist(D, "test_kulsinski")
         y3 = pdist(np.bool_(D), "test_kulsinski")
         if verbose > 2:
-            print np.abs(y1-y2).max()
+            print(np.abs(y1-y2).max())
         self.assertTrue(within_tol(y1, y2, eps))
 
     def test_pdist_canberra_match(self):
         "Tests pdist(X, 'canberra') to see if the two implementations match on the Iris data set."
         D = eo['iris']
         if verbose > 2:
-            print D.shape, D.dtype
+            print(D.shape, D.dtype)
         eps = 1e-10
         y1 = pdist(D, "canberra")
         y2 = pdist(D, "test_canberra")
         if verbose > 2:
-            print np.abs(y1-y2).max()
+            print(np.abs(y1-y2).max())
         self.assertTrue(within_tol(y1, y2, eps))
 
     def test_pdist_canberra_ticket_711(self):
@@ -1398,7 +1401,7 @@ class TestPdist(TestCase):
         pdist_y = pdist(([3.3], [3.4]), "canberra")
         right_y = 0.01492537
         if verbose > 2:
-            print np.abs(pdist_y-right_y).max()
+            print(np.abs(pdist_y-right_y).max())
         self.assertTrue(within_tol(pdist_y, right_y, eps))
 
 
@@ -1473,7 +1476,6 @@ class TestSomeDistanceFunctions(TestCase):
 
 class TestSquareForm(TestCase):
 
-    ################### squareform
     def test_squareform_empty_matrix(self):
         "Tests squareform on an empty matrix."
         A = np.zeros((0,0))
@@ -1501,11 +1503,19 @@ class TestSquareForm(TestCase):
         self.assertTrue(rv[0,1] == 8.3)
         self.assertTrue(rv[1,0] == 8.3)
 
+    def test_squareform_one_binary_vector(self):
+        """Tests squareform on a 1x1 binary matrix; conversion to double was
+        causing problems (see pull request 73)."""
+        v = np.ones((1,), dtype=np.bool)
+        rv = squareform(v)
+        self.assertTrue(rv.shape == (2,2))
+        self.assertTrue(rv[0,1])
+
     def test_squareform_2by2_matrix(self):
         "Tests squareform on a 2x2 matrix."
         A = np.zeros((2,2))
-        A[0,1]=0.8
-        A[1,0]=0.8
+        A[0,1] = 0.8
+        A[1,0] = 0.8
         rA = squareform(np.array(A, dtype='double'))
         self.assertTrue(rA.shape == (1,))
         self.assertTrue(rA[0] == 0.8)
@@ -1524,7 +1534,7 @@ class TestSquareForm(TestCase):
         s = A.shape
         k = 0
         if verbose >= 3:
-            print A.shape, Y.shape, Yr.shape
+            print(A.shape, Y.shape, Yr.shape)
         self.assertTrue(len(s) == 2)
         self.assertTrue(len(Yr.shape) == 1)
         self.assertTrue(s[0] == s[1])
@@ -1589,7 +1599,7 @@ class TestNumObsY(TestCase):
         return num_obs_y(self.make_y(n)) == n
 
     def make_y(self, n):
-        return np.random.rand((n*(n-1)/2))
+        return np.random.rand((n * (n - 1)) // 2)
 
 
 class TestNumObsDM(TestCase):
@@ -1602,7 +1612,7 @@ class TestNumObsDM(TestCase):
             Y = pdist(X)
             A = squareform(Y)
             if verbose >= 3:
-                print A.shape, Y.shape
+                print(A.shape, Y.shape)
             self.assertTrue(num_obs_dm(A) == n)
 
     def test_num_obs_dm_0(self):
@@ -1798,7 +1808,7 @@ class TestIsValidY(TestCase):
         return is_valid_y(y, throw=True)
 
     def correct_n_by_n(self, n):
-        y = np.random.rand(n*(n-1)/2)
+        y = np.random.rand((n * (n - 1)) // 2)
         return y
 
 
@@ -1898,5 +1908,5 @@ def test__validate_vector():
     assert_raises(ValueError, _validate_vector, x)
 
 
-if __name__=="__main__":
+if __name__ == "__main__":
     run_module_suite()
