@@ -24,6 +24,12 @@ double besselpoly(double a, double lambda, double nu);
 double gammaincinv(double a, double x);
 double gammasgn(double x);
 
+double struve_h(double v, double x);
+double struve_l(double v, double x);
+double struve_power_series(double v, double x, int is_h, double *err);
+double struve_asymp_large_z(double v, double z, int is_h, double *err);
+double struve_bessel_series(double v, double z, int is_h, double *err);
+
 #define gammaincinv_doc """gammaincinv(a, y) returns x such that gammainc(a, x) = y."""
 
 #endif /* C_MISC_MISC_H */
