@@ -1,6 +1,6 @@
-/*							mconf.h
+/*                                                     mconf.h
  *
- *	Common include file for math routines
+ *     Common include file for math routines
  *
  *
  *
@@ -58,14 +58,18 @@
  */
 
 /*
-Cephes Math Library Release 2.3:  June, 1995
-Copyright 1984, 1987, 1989, 1995 by Stephen L. Moshier
-*/
+ * Cephes Math Library Release 2.3:  June, 1995
+ * Copyright 1984, 1987, 1989, 1995 by Stephen L. Moshier
+ */
 
 #ifndef CEPHES_MCONF_H
 #define CEPHES_MCONF_H
 
+#include <Python.h>
+#include <numpy/npy_math.h>
+
 #include "cephes_names.h"
+#include "protos.h"
 
 /* Constant definitions for math error conditions
  */
@@ -76,47 +80,22 @@ Copyright 1984, 1987, 1989, 1995 by Stephen L. Moshier
 #define UNDERFLOW	4	/* underflow range error */
 #define TLOSS		5	/* total loss of precision */
 #define PLOSS		6	/* partial loss of precision */
-#define TOOMANY         7       /* too many iterations */
+#define TOOMANY         7	/* too many iterations */
 #define MAXITER        500
 
 #define EDOM		33
 #define ERANGE		34
 
-/* Complex numeral.  */
-typedef struct
-	{
-	double r;
-	double i;
-	} cmplx;
-
 /* Long double complex numeral.  */
 /*
-typedef struct
-	{
-	long double r;
-	long double i;
-	} cmplxl;
-*/
-
-/* Type of computer arithmetic */
-
-/* This is kind of improper, as the byte-order of floats may not
- * be the same as the byte-order of ints. However, it works.
- *
- * SciPy note: we bypass this detection and set UNK to 1 to prevent Endianess
- * issues.
+ * typedef struct
+ * {
+ * long double r;
+ * long double i;
+ * } cmplxl;
  */
 
-/*
-#include <pyconfig.h>
-#ifdef WORDS_BIGENDIAN
-# define MIEEE 1
-# define BIGENDIAN 1
-#else
-# define IBMPC 1
-# define BIGENDIAN 0
-#endif
-*/
+/* Type of computer arithmetic */
 
 /* UNKnown arithmetic, invokes coefficients given in
  * normal decimal format.  Beware of range boundary
@@ -155,37 +134,16 @@ typedef struct
 /* Define to support tiny denormal numbers, else undefine. */
 #define DENORMAL 1
 
-/* Define to ask for infinity support, else undefine. */
-#define INFINITIES 1
-#ifdef NOINFINITIES
-#undef INFINITIES
-#endif
-
-/* Define to ask for support of numbers that are Not-a-Number,
-   else undefine.  This may automatically define INFINITIES in some files. */
-#define NANS 1
-#ifdef NONANS
-#undef NANS
-#endif
-
 /* Define to distinguish between -0.0 and +0.0.  */
 #define MINUSZERO 1
 
 /* Define 1 for ANSI C atan2() function
-   See atan.c and clog.c. */
+ * See atan.c and clog.c. */
 #define ANSIC 1
 
-/* Get ANSI function prototypes, if you want them. */
-#if defined(__STDC__) || defined(_MSC_EXTENSIONS)
-#define ANSIPROT
-#include "protos.h"
-#else
-int mtherr();
-#endif
-
 /* Variable for error reporting.  See mtherr.c.  */
 extern int merror;
 
 #define gamma Gamma
 
-#endif /* CEPHES_MCONF_H */
+#endif				/* CEPHES_MCONF_H */
