@@ -1,5 +1,5 @@
 #! /usr/bin/env python
-# Last Change: Mon Jul 23 09:00 PM 2007 J
+# Last Change: Sat Mar 21 02:00 PM 2009 J
 
 # Copyright (c) 2001, 2002 Enthought, Inc.
 #
@@ -33,55 +33,60 @@
 """Some more special functions which may be useful for multivariate statistical
 analysis."""
 
+from __future__ import division, print_function, absolute_import
+
 import numpy as np
 from scipy.special import gammaln as loggam
 
+
+__all__ = ['multigammaln']
+
+
 def multigammaln(a, d):
-    """returns the log of multivariate gamma, also sometimes called the
+    """Returns the log of multivariate gamma, also sometimes called the
     generalized gamma.
 
-    :Parameters:
-        a : ndarray
-            the multivariate gamma is computed for each item of a
-        d : int
-            the dimension of the space of integration.
-
-    :Returns:
-        res : ndarray
-            the values of the log multivariate gamma at the given points a.
-
-    Note
-    ----
-
-    The formal definition of the multivariate gamma of dimension d for a real a
-    is :
-
-    \Gamma_d(a) = \int_{A>0}{e^{-tr(A)\cdot{|A|}^{a - (m+1)/2}dA}}
-
-    with the condition a > (d-1)/2, and A>0 being the set of all the positive
-    definite matrices of dimension s. Note that a is a scalar: the integration
-    is multivariate, the argument is not.
-
-    This can be proven to be equal to the much friendler equation:
-
-    \Gamma_d(a) = \pi^{d(d-1)/4}\prod_{i=1}^{d}{\Gamma(a - (i-1)/2)}.
-
-    Reference:
+    Parameters
     ----------
+    a : ndarray
+        The multivariate gamma is computed for each item of `a`.
+    d : int
+        The dimension of the space of integration.
 
+    Returns
+    -------
+    res : ndarray
+        The values of the log multivariate gamma at the given points `a`.
+
+    Notes
+    -----
+    The formal definition of the multivariate gamma of dimension d for a real a
+    is::
+
+        \Gamma_d(a) = \int_{A>0}{e^{-tr(A)\cdot{|A|}^{a - (m+1)/2}dA}}
+
+    with the condition ``a > (d-1)/2``, and ``A > 0`` being the set of all the
+    positive definite matrices of dimension s.  Note that a is a scalar: the
+    integrand only is multivariate, the argument is not (the function is
+    defined over a subset of the real set).
+
+    This can be proven to be equal to the much friendlier equation::
+
+        \Gamma_d(a) = \pi^{d(d-1)/4}\prod_{i=1}^{d}{\Gamma(a - (i-1)/2)}.
+
+    References
+    ----------
     R. J. Muirhead, Aspects of multivariate statistical theory (Wiley Series in
-    probability and mathematical statistics). """
+    probability and mathematical statistics).
+
+    """
     a = np.asarray(a)
     if not np.isscalar(d) or (np.floor(d) != d):
         raise ValueError("d should be a positive integer (dimension)")
     if np.any(a <= 0.5 * (d - 1)):
-        raise ValueError("condition a (%f) > 0.5 * (d-1) (%f) not met" \
+        raise ValueError("condition a (%f) > 0.5 * (d-1) (%f) not met"
                          % (a, 0.5 * (d-1)))
 
     res = (d * (d-1) * 0.25) * np.log(np.pi)
-    if a.size == 1:
-        axis = -1
-    else:
-        axis = 0
-    res += np.sum(loggam([(a - (j - 1.)/2) for j in range(1, d+1)]), axis)
+    res += np.sum(loggam([(a - (j - 1.)/2) for j in range(1, d+1)]), axis=0)
     return res
