@@ -1,6 +1,8 @@
+from __future__ import absolute_import, print_function
+
 import types
-from base_spec import base_converter
-import base_info
+from .base_spec import base_converter
+from . import base_info
 
 #----------------------------------------------------------------------------
 # C++ code template for converting code from python objects to C++ objects
@@ -68,6 +70,7 @@ PyObject* %(type_name)s_to_py(PyObject* obj)
 
 """
 
+
 class common_base_converter(base_converter):
 
     def __init__(self):
@@ -161,7 +164,7 @@ class common_base_converter(base_converter):
     def c_to_py_code(self):
         return simple_c_to_py_template % self.template_vars()
 
-    def declaration_code(self,templatize = 0,inline=0):
+    def declaration_code(self,templatize=0,inline=0):
         code = '%(py_var)s = %(var_lookup)s;\n'   \
                '%(c_type)s %(name)s = %(var_convert)s;\n' %  \
                self.template_vars(inline=inline)
@@ -169,8 +172,8 @@ class common_base_converter(base_converter):
 
     def cleanup_code(self):
         if self.use_ref_count:
-            code =  'Py_XDECREF(%(py_var)s);\n' % self.template_vars()
-            #code += 'printf("cleaning up %(py_var)s\\n");\n' % self.template_vars()
+            code = 'Py_XDECREF(%(py_var)s);\n' % self.template_vars()
+            # code += 'printf("cleaning up %(py_var)s\\n");\n' % self.template_vars()
         else:
             code = ""
         return code
@@ -178,8 +181,9 @@ class common_base_converter(base_converter):
     def __repr__(self):
         msg = "(file:: name: %s)" % self.name
         return msg
+
     def __cmp__(self,other):
-        #only works for equal
+        # only works for equal
         result = -1
         try:
             result = cmp(self.name,other.name) or \
@@ -191,6 +195,8 @@ class common_base_converter(base_converter):
 #----------------------------------------------------------------------------
 # Module Converter
 #----------------------------------------------------------------------------
+
+
 class module_converter(common_base_converter):
     def init_info(self):
         common_base_converter.init_info(self)
@@ -202,6 +208,8 @@ class module_converter(common_base_converter):
 #----------------------------------------------------------------------------
 # String Converter
 #----------------------------------------------------------------------------
+
+
 class string_converter(common_base_converter):
     def init_info(self):
         common_base_converter.init_info(self)
@@ -212,6 +220,7 @@ class string_converter(common_base_converter):
         self.to_c_return = "std::string(PyString_AsString(py_obj))"
         self.matching_types = [types.StringType]
         self.headers.append('<string>')
+
     def c_to_py_code(self):
         # !! Need to dedent returned code.
         code = """
@@ -225,21 +234,23 @@ class string_converter(common_base_converter):
 #----------------------------------------------------------------------------
 # Unicode Converter
 #----------------------------------------------------------------------------
+
+
 class unicode_converter(common_base_converter):
     def init_info(self):
         common_base_converter.init_info(self)
         self.type_name = 'unicode'
         self.check_func = 'PyUnicode_Check'
         # This isn't supported by gcc 2.95.3 -- MSVC works fine with it.
-        #self.c_type = 'std::wstring'
-        #self.to_c_return = "std::wstring(PyUnicode_AS_UNICODE(py_obj))"
+        # self.c_type = 'std::wstring'
+        # self.to_c_return = "std::wstring(PyUnicode_AS_UNICODE(py_obj))"
         self.c_type = 'Py_UNICODE*'
         self.return_type = self.c_type
         self.to_c_return = "PyUnicode_AS_UNICODE(py_obj)"
         self.matching_types = [types.UnicodeType]
-        #self.headers.append('<string>')
+        # self.headers.append('<string>')
 
-    def declaration_code(self,templatize = 0,inline=0):
+    def declaration_code(self,templatize=0,inline=0):
         # since wstring doesn't seem to work everywhere, we need to provide
         # the length variable Nxxx for the unicode string xxx.
         code = '%(py_var)s = %(var_lookup)s;\n'                     \
@@ -247,11 +258,12 @@ class unicode_converter(common_base_converter):
                'int N%(name)s = PyUnicode_GET_SIZE(%(py_var)s);\n'  \
                % self.template_vars(inline=inline)
 
-
         return code
 #----------------------------------------------------------------------------
 # File Converter
 #----------------------------------------------------------------------------
+
+
 class file_converter(common_base_converter):
     def init_info(self):
         common_base_converter.init_info(self)
@@ -290,16 +302,16 @@ class file_converter(common_base_converter):
 # Standard Python numeric --> C type maps
 #----------------------------------------------------------------------------
 num_to_c_types = {}
-num_to_c_types[type(1)]  = 'long'
+num_to_c_types[type(1)] = 'long'
 num_to_c_types[type(1.)] = 'double'
 num_to_c_types[type(1.+1.j)] = 'std::complex<double> '
 # !! hmmm. The following is likely unsafe...
-num_to_c_types[type(1L)]  = 'npy_longlong'
+num_to_c_types[long] = 'npy_longlong'
 
 #----------------------------------------------------------------------------
 # Numeric array Python numeric --> C type maps
 #----------------------------------------------------------------------------
-num_to_c_types['T'] = 'T' # for templates
+num_to_c_types['T'] = 'T'  # for templates
 num_to_c_types['G'] = 'std::complex<longdouble> '
 num_to_c_types['F'] = 'std::complex<float> '
 num_to_c_types['D'] = 'std::complex<double> '
@@ -320,6 +332,7 @@ num_to_c_types['L'] = 'npy_ulong'
 num_to_c_types['q'] = 'npy_longlong'
 num_to_c_types['Q'] = 'npy_ulonglong'
 
+
 class scalar_converter(common_base_converter):
     def init_info(self):
         common_base_converter.init_info(self)
@@ -328,6 +341,8 @@ class scalar_converter(common_base_converter):
         self.use_ref_count = 0
 
 # This has to be int for SCXX to work.
+
+
 class int_converter(scalar_converter):
     def init_info(self):
         scalar_converter.init_info(self)
@@ -338,6 +353,7 @@ class int_converter(scalar_converter):
         self.to_c_return = "(int) PyInt_AsLong(py_obj)"
         self.matching_types = [types.IntType]
 
+
 class long_converter(scalar_converter):
     def init_info(self):
         scalar_converter.init_info(self)
@@ -349,6 +365,7 @@ class long_converter(scalar_converter):
         self.to_c_return = "(longlong) PyLong_AsLongLong(py_obj)"
         self.matching_types = [types.LongType]
 
+
 class float_converter(scalar_converter):
     def init_info(self):
         scalar_converter.init_info(self)
@@ -360,6 +377,7 @@ class float_converter(scalar_converter):
         self.to_c_return = "PyFloat_AsDouble(py_obj)"
         self.matching_types = [types.FloatType]
 
+
 class complex_converter(scalar_converter):
     def init_info(self):
         scalar_converter.init_info(self)
@@ -377,10 +395,11 @@ class complex_converter(scalar_converter):
 #
 # Based on SCXX by Gordon McMillan
 #----------------------------------------------------------------------------
-import os, c_spec # yes, I import myself to find out my __file__ location.
-local_dir,junk = os.path.split(os.path.abspath(c_spec.__file__))
+import os
+local_dir,junk = os.path.split(os.path.abspath(__file__))
 scxx_dir = os.path.join(local_dir,'scxx')
 
+
 class scxx_converter(common_base_converter):
     def init_info(self):
         common_base_converter.init_info(self)
@@ -389,6 +408,7 @@ class scxx_converter(common_base_converter):
         self.include_dirs = [local_dir,scxx_dir]
         self.sources = [os.path.join(scxx_dir,'weave_imp.cpp'),]
 
+
 class list_converter(scxx_converter):
     def init_info(self):
         scxx_converter.init_info(self)
@@ -401,6 +421,7 @@ class list_converter(scxx_converter):
         # ref counting handled by py::list
         self.use_ref_count = 0
 
+
 class tuple_converter(scxx_converter):
     def init_info(self):
         scxx_converter.init_info(self)
@@ -413,6 +434,7 @@ class tuple_converter(scxx_converter):
         # ref counting handled by py::tuple
         self.use_ref_count = 0
 
+
 class dict_converter(scxx_converter):
     def init_info(self):
         scxx_converter.init_info(self)
@@ -428,6 +450,8 @@ class dict_converter(scxx_converter):
 #----------------------------------------------------------------------------
 # Instance Converter
 #----------------------------------------------------------------------------
+
+
 class instance_converter(scxx_converter):
     def init_info(self):
         scxx_converter.init_info(self)
@@ -445,6 +469,8 @@ class instance_converter(scxx_converter):
 #
 # catch all now handles callable objects
 #----------------------------------------------------------------------------
+
+
 class catchall_converter(scxx_converter):
     def init_info(self):
         scxx_converter.init_info(self)
@@ -455,15 +481,16 @@ class catchall_converter(scxx_converter):
         self.to_c_return = 'py::object(py_obj)'
         # ref counting handled by py::object
         self.use_ref_count = 0
+
     def type_match(self,value):
         return 1
 
 if __name__ == "__main__":
     x = list_converter().type_spec("x",1)
-    print x.py_to_c_code()
-    print
-    print x.c_to_py_code()
-    print
-    print x.declaration_code(inline=1)
-    print
-    print x.cleanup_code()
+    print(x.py_to_c_code())
+    print()
+    print(x.c_to_py_code())
+    print()
+    print(x.declaration_code(inline=1))
+    print()
+    print(x.cleanup_code())
